@@ -45,13 +45,19 @@
 #include "nsIDragDropHandler.h"
 #include "nsIDOMDragListener.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsITransferable.h"
 
 class nsIDOMNode;
 class nsISelection;
 class nsITransferable;
-class nsIOverrideDragSource;
-class nsIOverrideDropSite;
-
+class nsIImage;
+class nsIPresShell;
+class nsIPresContext;
+class nsIContent;
+class nsIDocument;
+class nsIURI;
+class nsILocalFile;
+class nsISimpleEnumerator;
 
 // {1f34bc80-1bc7-11d6-a384-d705dd0746fc}
 #define NS_CONTENTAREADRAGDROP_CID             \
@@ -67,11 +73,14 @@ class nsIOverrideDropSite;
 // to drag and drop. Registers itself with the DOM with AddChromeListeners()
 // and removes itself with RemoveChromeListeners().
 //
-class nsContentAreaDragDrop : public nsIDOMDragListener, public nsIDragDropHandler
+class nsContentAreaDragDrop : public nsIDOMDragListener,
+                              public nsIDragDropHandler,
+                              public nsIFlavorDataProvider
 {
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIDRAGDROPHANDLER
+  NS_DECL_NSIFLAVORDATAPROVIDER
   
   nsContentAreaDragDrop ( ) ;
   virtual ~nsContentAreaDragDrop ( ) ;
@@ -87,33 +96,24 @@ public:
 private:
 
     // Add/remove the relevant listeners
-  NS_IMETHOD AddDragListener();
-  NS_IMETHOD RemoveDragListener();
+  nsresult AddDragListener();
+  nsresult RemoveDragListener();
 
     // utility routines
-  static void FindFirstAnchor(nsIDOMNode* inNode, nsIDOMNode** outAnchor);
-  static void FindParentLinkNode(nsIDOMNode* inNode, nsIDOMNode** outParent);
-  static void GetAnchorURL(nsIDOMNode* inNode, nsAString& outURL);
-  static void CreateLinkText(const nsAString& inURL, const nsAString & inText,
-                              nsAString& outLinkText);
-  static void GetNodeString(nsIDOMNode* inNode, nsAString & outNodeString);
   static void NormalizeSelection(nsIDOMNode* inBaseNode, nsISelection* inSelection);
   static void GetEventDocument(nsIDOMEvent* inEvent, nsIDOMDocument** outDocument);
 
-  PRBool BuildDragData(nsIDOMEvent* inMouseEvent, nsAString & outURLString, nsAString & outTitleString,
-                        nsAString & outHTMLString, PRBool* outIsAnchor);
-  nsresult CreateTransferable(const nsAString & inURLString, const nsAString & inTitleString, 
-                                const nsAString & inHTMLString, PRBool inIsAnchor, 
-                                nsITransferable** outTrans);
+  static nsresult SaveURIToFileInDirectory(nsAString& inSourceURIString, nsILocalFile* inDestDirectory, nsILocalFile** outFile);
+  
+  nsresult CreateTransferable(nsIDOMEvent* inMouseEvent, nsITransferable** outTrans);
   void ExtractURLFromData(const nsACString & inFlavor, nsISupports* inDataWrapper, PRUint32 inDataLen,
                            nsAString & outURL);
+  nsresult GetHookEnumeratorFromEvent(nsIDOMEvent* inEvent, nsISimpleEnumerator** outEnumerator);
 
   PRPackedBool mListenerInstalled;
 
   nsCOMPtr<nsIDOMEventReceiver> mEventReceiver;
   nsIWebNavigation* mNavigator;                     // weak ref, this is probably my owning webshell
-  nsIOverrideDragSource* mOverrideDrag;             // weak, these could own us but probably will outlive us
-  nsIOverrideDropSite* mOverrideDrop;
 
 }; // class nsContentAreaDragDrop
 
