@@ -39,6 +39,7 @@
 
 #include "nsCOMPtr.h"
 #include "nsXBLPrototypeHandler.h"
+#include "nsXBLPrototypeBinding.h"
 #include "nsIContent.h"
 #include "nsIAtom.h"
 #include "nsIDOMKeyEvent.h"
@@ -65,7 +66,8 @@
 #include "nsPIDOMWindow.h"
 #include "nsPIWindowRoot.h"
 #include "nsIDOMWindowInternal.h"
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #include "nsIServiceManager.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
@@ -78,6 +80,8 @@
 #include "nsDOMCID.h"
 #include "nsUnicharUtils.h"
 #include "nsReadableUtils.h"
+#include "nsCRT.h"
+#include "nsXBLEventHandler.h"
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
                      NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
@@ -91,30 +95,40 @@ const PRInt32 nsXBLPrototypeHandler::cShift = (1<<1);
 const PRInt32 nsXBLPrototypeHandler::cAlt = (1<<2);
 const PRInt32 nsXBLPrototypeHandler::cControl = (1<<3);
 const PRInt32 nsXBLPrototypeHandler::cMeta = (1<<4);
+const PRInt32 nsXBLPrototypeHandler::cAllModifiers = cShift | cAlt | cControl | cMeta;
 
-nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent, const PRUnichar* aPhase,
-                                             const PRUnichar* aAction, const PRUnichar* aCommand,
-                                             const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                                             const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                             const PRUnichar* aClickCount)
+nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent,
+                                             const PRUnichar* aPhase,
+                                             const PRUnichar* aAction,
+                                             const PRUnichar* aCommand,
+                                             const PRUnichar* aKeyCode,
+                                             const PRUnichar* aCharCode,
+                                             const PRUnichar* aModifiers,
+                                             const PRUnichar* aButton,
+                                             const PRUnichar* aClickCount,
+                                             const PRUnichar* aPreventDefault,
+                                             nsXBLPrototypeBinding* aBinding)
+  : mHandlerText(nsnull),
+    mLineNumber(0),
+    mNextHandler(nsnull),
+    mPrototypeBinding(aBinding)
 {
-  NS_INIT_REFCNT();
-  gRefCnt++;
+  ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
     InitAccessKeys();
 
-  mHandlerText = nsnull;
-
-  ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode, aCharCode, aModifiers,
-                     aButton, aClickCount);
+  ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode,
+                     aCharCode, aModifiers, aButton, aClickCount,
+                     aPreventDefault);
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
+  : mLineNumber(0),
+    mNextHandler(nsnull),
+    mPrototypeBinding(nsnull)
 {
-  NS_INIT_REFCNT();
-  
-  gRefCnt++;
+  ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
     InitAccessKeys();
@@ -125,27 +139,30 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
 
 nsXBLPrototypeHandler::~nsXBLPrototypeHandler()
 {
-  gRefCnt--;
-  if (mType != NS_HANDLER_TYPE_XUL && mHandlerText)
+  --gRefCnt;
+  if (!(mType & NS_HANDLER_TYPE_XUL) && mHandlerText)
     nsMemory::Free(mHandlerText);
+
+  // We own the next handler in the chain, so delete it now.
+  delete mNextHandler;
 }
 
-NS_IMPL_ISUPPORTS1(nsXBLPrototypeHandler, nsIXBLPrototypeHandler)
-
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetHandlerElement(nsIContent** aResult)
+already_AddRefed<nsIContent>
+nsXBLPrototypeHandler::GetHandlerElement()
 {
-  if (mType == NS_HANDLER_TYPE_XUL) {
-    *aResult = mHandlerElement;
-    NS_IF_ADDREF(*aResult);
+  nsIContent* result;
+  if (mType & NS_HANDLER_TYPE_XUL) {
+    result = mHandlerElement;
+    NS_IF_ADDREF(result);
   }
   else
-    *aResult = nsnull;
-  return NS_OK;
+    result = nsnull;
+
+  return result;
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::AppendHandlerText(const nsAReadableString& aText) 
+void
+nsXBLPrototypeHandler::AppendHandlerText(const nsAString& aText) 
 {
   if (mHandlerText) {
     // Append our text to the existing text.
@@ -155,22 +172,6 @@ nsXBLPrototypeHandler::AppendHandlerText(const nsAReadableString& aText)
   }
   else
     mHandlerText = ToNewUnicode(aText);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetNextHandler(nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = mNextHandler;
-  NS_IF_ADDREF(*aResult);
-  return NS_OK;
-}
-  
-NS_IMETHODIMP
-nsXBLPrototypeHandler::SetNextHandler(nsIXBLPrototypeHandler* aHandler)
-{
-  mNextHandler = aHandler;
-  return NS_OK;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -193,27 +194,37 @@ nsXBLPrototypeHandler::InitAccessKeys()
 #endif
 
   // Get the menu access key value from prefs, overriding the default:
-  nsresult rv;
-  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
-  if (NS_SUCCEEDED(rv) && prefs)
-  {
-    prefs->GetIntPref("ui.key.menuAccessKey", &kMenuAccessKey);
-    prefs->GetIntPref("ui.key.accelKey", &kAccelKey);
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (prefBranch) {
+    prefBranch->GetIntPref("ui.key.menuAccessKey", &kMenuAccessKey);
+    prefBranch->GetIntPref("ui.key.accelKey", &kAccelKey);
   }
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
                                       nsIDOMEvent* aEvent)
 {
+  nsresult rv = NS_ERROR_FAILURE;
+
+  // Prevent default action?
+  if (mType & NS_HANDLER_TYPE_PREVENTDEFAULT) {
+    aEvent->PreventDefault();
+    // If we prevent default, then it's okay for
+    // mHandlerElement and mHandlerText to be null
+    rv = NS_OK;
+  }
+
   if (!mHandlerElement) // This works for both types of handlers.  In both cases, the union's var should be defined.
-    return NS_ERROR_FAILURE;
+    return rv;
 
   // See if our event receiver is a content node (and not us).
-  PRBool isXULKey = (mType == NS_HANDLER_TYPE_XUL);
+  PRBool isXULKey = (mType & NS_HANDLER_TYPE_XUL);
+  PRBool isXBLCommand = (mType & NS_HANDLER_TYPE_XBL_COMMAND);
 
-  //XUL handlers shouldn't be triggered by non-trusted events.
-  if (isXULKey) {
+  // XUL handlers and commands shouldn't be triggered by non-trusted
+  // events.
+  if (isXULKey || isXBLCommand) {
     nsCOMPtr<nsIPrivateDOMEvent> privateEvent = do_QueryInterface(aEvent);
     if (privateEvent) {
       PRBool trustedEvent;
@@ -230,7 +241,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
 
   // This is a special-case optimization to make command handling fast.
   // It isn't really a part of XBL, but it helps speed things up.
-  if (mType == NS_HANDLER_TYPE_XBL_COMMAND && !isReceiverCommandElement) {
+  if (isXBLCommand && !isReceiverCommandElement) {
     // See if preventDefault has been set.  If so, don't execute.
     PRBool preventDefault;
     nsCOMPtr<nsIDOMNSUIEvent> nsUIEvent(do_QueryInterface(aEvent));
@@ -263,7 +274,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
         nsCOMPtr<nsIContent> elt(do_QueryInterface(aReceiver));
         nsCOMPtr<nsIDocument> doc;
         if (elt)
-          elt->GetDocument(*getter_AddRefs(doc));
+          doc = elt->GetDocument();
 
         if (!doc)
           doc = do_QueryInterface(aReceiver);
@@ -271,19 +282,17 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
         if (!doc)
           return NS_ERROR_FAILURE;
 
-        nsCOMPtr<nsIScriptGlobalObject> globalObject;
-        doc->GetScriptGlobalObject(getter_AddRefs(globalObject));
-        privateWindow = do_QueryInterface(globalObject);
+        privateWindow = do_QueryInterface(doc->GetScriptGlobalObject());
       }
 
       privateWindow->GetRootFocusController(getter_AddRefs(focusController));
     }
 
-    nsDependentString command(mHandlerText);
-
+    NS_LossyConvertUCS2toASCII command(mHandlerText);
     if (focusController)
-      focusController->GetControllerForCommand(command, getter_AddRefs(controller));
-    else GetController(aReceiver, getter_AddRefs(controller)); // We're attached to the receiver possibly.
+      focusController->GetControllerForCommand(command.get(), getter_AddRefs(controller));
+    else
+      controller = GetController(aReceiver); // We're attached to the receiver possibly.
 
     nsAutoString type;
     mEventName->ToString(type);
@@ -312,7 +321,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     aEvent->PreventDefault();
     
     if (controller)
-      controller->DoCommand(command);
+      controller->DoCommand(command.get());
 
     return NS_OK;
   }
@@ -323,7 +332,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
   nsAutoString str;
   mEventName->ToString(str);
   onEvent += str;
-  nsCOMPtr<nsIAtom> onEventAtom = getter_AddRefs(NS_NewAtom(onEvent));
+  nsCOMPtr<nsIAtom> onEventAtom = do_GetAtom(onEvent);
 
   void* handler = nsnull;
   
@@ -375,12 +384,12 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
       nsCOMPtr<nsIContent> content(do_QueryInterface(aReceiver));
       if (!content)
         return NS_OK;
-      content->GetDocument(*getter_AddRefs(boundDocument));
+      boundDocument = content->GetDocument();
       if (!boundDocument)
         return NS_OK;
     }
 
-    boundDocument->GetScriptGlobalObject(getter_AddRefs(boundGlobal));
+    boundGlobal = boundDocument->GetScriptGlobalObject();
   }
 
   // If we still don't have a 'boundGlobal', we're doomed. bug 95465.
@@ -388,24 +397,23 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
   if (!boundGlobal)
     return NS_OK;
 
-  nsCOMPtr<nsIScriptContext> boundContext;
-  boundGlobal->GetContext(getter_AddRefs(boundContext));
+  nsIScriptContext *boundContext = boundGlobal->GetContext();
   if (!boundContext) return NS_OK;
 
   JSObject* scriptObject = nsnull;
 
+  // strong ref to a GC root we'll need to protect scriptObject in the case
+  // where it is not the global object (!winRoot).
+  nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
+
   if (winRoot) {
     scriptObject = boundGlobal->GetGlobalJSObject();
   } else {
     JSObject *global = boundGlobal->GetGlobalJSObject();
     JSContext *cx = (JSContext *)boundContext->GetNativeContext();
 
-    nsresult rv;
     nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
 
-    // root
-    nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
-
     // XXX: Don't use the global object!
     rv = xpc->WrapNative(cx, global, aReceiver, NS_GET_IID(nsISupports),
                          getter_AddRefs(wrapper));
@@ -417,12 +425,20 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
 
   if (isXULKey)
     boundContext->CompileEventHandler(scriptObject, onEventAtom, xulText,
+                                      nsnull, 0,
                                       PR_TRUE, &handler);
   else {
     nsDependentString handlerText(mHandlerText);
     if (handlerText.IsEmpty())
       return NS_ERROR_FAILURE;
+    
+    nsCAutoString bindingURI;
+    if (mPrototypeBinding)
+      mPrototypeBinding->DocURI()->GetSpec(bindingURI);
+    
     boundContext->CompileEventHandler(scriptObject, onEventAtom, handlerText,
+                                      bindingURI.get(),
+                                      mLineNumber,
                                       PR_TRUE, &handler);
   }
 
@@ -447,27 +463,19 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetEventName(nsIAtom** aResult)
+already_AddRefed<nsIAtom>
+nsXBLPrototypeHandler::GetEventName()
 {
-  *aResult = mEventName;
-  NS_IF_ADDREF(*aResult);
-  return NS_OK;
+  nsIAtom* eventName = mEventName;
+  NS_IF_ADDREF(eventName);
+  return eventName;
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::BindingAttached(nsIDOMEventReceiver* aReceiver)
 {
   nsresult ret;
-  nsMouseEvent event;
-  event.eventStructType = NS_EVENT;
-  event.message = NS_XUL_COMMAND;
-  event.isShift = PR_FALSE;
-  event.isControl = PR_FALSE;
-  event.isAlt = PR_FALSE;
-  event.isMeta = PR_FALSE;
-  event.clickCount = 0;
-  event.widget = nsnull;
+  nsMouseEvent event(NS_XUL_COMMAND);
 
   nsCOMPtr<nsIEventListenerManager> listenerManager;
   if (NS_FAILED(ret = aReceiver->GetListenerManager(getter_AddRefs(listenerManager)))) {
@@ -495,19 +503,11 @@ nsXBLPrototypeHandler::BindingAttached(nsIDOMEventReceiver* aReceiver)
   return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 nsXBLPrototypeHandler::BindingDetached(nsIDOMEventReceiver* aReceiver)
 {
   nsresult ret;
-  nsMouseEvent event;
-  event.eventStructType = NS_EVENT;
-  event.message = NS_XUL_COMMAND;
-  event.isShift = PR_FALSE;
-  event.isControl = PR_FALSE;
-  event.isAlt = PR_FALSE;
-  event.isMeta = PR_FALSE;
-  event.clickCount = 0;
-  event.widget = nsnull;
+  nsMouseEvent event(NS_XUL_COMMAND);
 
   nsCOMPtr<nsIEventListenerManager> listenerManager;
   if (NS_FAILED(ret = aReceiver->GetListenerManager(getter_AddRefs(listenerManager)))) {
@@ -535,8 +535,8 @@ nsXBLPrototypeHandler::BindingDetached(nsIDOMEventReceiver* aReceiver)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver, nsIController** aResult)
+already_AddRefed<nsIController>
+nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver)
 {
   // XXX Fix this so there's a generic interface that describes controllers, 
   // This code should have no special knowledge of what objects might have controllers.
@@ -567,27 +567,20 @@ nsXBLPrototypeHandler::GetController(nsIDOMEventReceiver* aReceiver, nsIControll
   // Return the first controller.
   // XXX This code should be checking the command name and using supportscommand and
   // iscommandenabled.
+  nsIController* controller;
   if (controllers) {
-    controllers->GetControllerAt(0, aResult);
+    controllers->GetControllerAt(0, &controller);  // return reference
   }
-  else *aResult = nsnull;
+  else controller = nsnull;
 
-  return NS_OK;
+  return controller;
 }
 
-
-NS_IMETHODIMP
-nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKeyEvent, PRBool* aResult)
+PRBool
+nsXBLPrototypeHandler::KeyEventMatched(nsIDOMKeyEvent* aKeyEvent)
 {
-  *aResult = PR_TRUE;
-
-  if (aEventType != mEventName.get()) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
-
   if (mDetail == -1 && mMisc == 0 && mKeyMask == 0)
-    return NS_OK; // No filters set up. It's generic.
+    return PR_TRUE; // No filters set up. It's generic.
 
   // Get the keycode and charcode of the key event.
   PRUint32 keyCode, charCode;
@@ -596,47 +589,36 @@ nsXBLPrototypeHandler::KeyEventMatched(nsIAtom* aEventType, nsIDOMKeyEvent* aKey
 
   PRBool keyMatched = (mDetail == PRInt32(mMisc ? charCode : keyCode));
 
-  if (!keyMatched) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (!keyMatched)
+    return PR_FALSE;
 
   // Now check modifier keys
-  PRBool result = ModifiersMatchMask(aKeyEvent);
-  *aResult = result;
-  return NS_OK;
+  PRInt32 modKeys = cAllModifiers;
+  // Don't check shift if we matched the char code and shift isn't specified
+  // in the handler.
+  if (mMisc && !(mKeyMask & cShift))
+    modKeys &= ~cShift;
+
+  return ModifiersMatchMask(aKeyEvent, modKeys);
 }
 
-NS_IMETHODIMP
-nsXBLPrototypeHandler::MouseEventMatched(nsIAtom* aEventType, nsIDOMMouseEvent* aMouseEvent, PRBool* aResult)
+PRBool
+nsXBLPrototypeHandler::MouseEventMatched(nsIDOMMouseEvent* aMouseEvent)
 {
-  *aResult = PR_TRUE;
-
-  if (aEventType != mEventName.get()) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
-
   if (mDetail == -1 && mMisc == 0 && mKeyMask == 0)
-    return NS_OK; // No filters set up. It's generic.
+    return PR_TRUE; // No filters set up. It's generic.
 
   unsigned short button;
   aMouseEvent->GetButton(&button);
-  if (mDetail != -1 && (button != mDetail)) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (mDetail != -1 && (button != mDetail))
+    return PR_FALSE;
 
   PRInt32 clickcount;
   aMouseEvent->GetDetail(&clickcount);
-  if (mMisc != 0 && (clickcount != mMisc)) {
-    *aResult = PR_FALSE;
-    return NS_OK;
-  }
+  if (mMisc != 0 && (clickcount != mMisc))
+    return PR_FALSE;
   
-  PRBool result = ModifiersMatchMask(aMouseEvent);
-  *aResult = result;
-  return NS_OK;
+  return ModifiersMatchMask(aMouseEvent, cAllModifiers);
 }
 
 struct keyCodeData {
@@ -771,7 +753,7 @@ static const keyCodeData gKeyCodes[] = {
 
 };
 
-PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAReadableString& aKeyName)
+PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAString& aKeyName)
 {
   nsCAutoString keyName;
   keyName.AssignWithConversion(aKeyName);
@@ -780,7 +762,7 @@ PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAReadableString& aKeyN
 
   PRUint32 keyNameLength = keyName.Length();
   const char* keyNameStr = keyName.get();
-  for (int i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
+  for (PRUint16 i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
     if (keyNameLength == gKeyCodes[i].strlength &&
         !nsCRT::strcmp(gKeyCodes[i].str, keyNameStr))
       return gKeyCodes[i].keycode;
@@ -808,29 +790,36 @@ PRInt32 nsXBLPrototypeHandler::KeyToMask(PRInt32 key)
 }
 
 void
-nsXBLPrototypeHandler::GetEventType(nsAWritableString& aEvent)
+nsXBLPrototypeHandler::GetEventType(nsAString& aEvent)
 {
   mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::event, aEvent);
   
-  if (aEvent.IsEmpty() && mType == NS_HANDLER_TYPE_XUL)
+  if (aEvent.IsEmpty() && (mType & NS_HANDLER_TYPE_XUL))
     // If no type is specified for a XUL <key> element, let's assume that we're "keypress".
     aEvent = NS_LITERAL_STRING("keypress");
 }
 
 void
 nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement, 
-                                          const PRUnichar* aEvent, const PRUnichar* aPhase,
-                                          const PRUnichar* aAction, const PRUnichar* aCommand,
-                                          const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                                          const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                          const PRUnichar* aClickCount)
+                                          const PRUnichar* aEvent,
+                                          const PRUnichar* aPhase,
+                                          const PRUnichar* aAction,
+                                          const PRUnichar* aCommand,
+                                          const PRUnichar* aKeyCode,
+                                          const PRUnichar* aCharCode,
+                                          const PRUnichar* aModifiers,
+                                          const PRUnichar* aButton,
+                                          const PRUnichar* aClickCount,
+                                          const PRUnichar* aPreventDefault)
 {
+  mType = 0;
+
   if (aKeyElement) {
-    mType = NS_HANDLER_TYPE_XUL;
+    mType |= NS_HANDLER_TYPE_XUL;
     mHandlerElement = aKeyElement;
   }
   else {
-    mType = aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
+    mType |= aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
     mHandlerText = nsnull;
   }
 
@@ -846,15 +835,13 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   nsAutoString event(aEvent);
   if (event.IsEmpty()) {
-    if (mType == NS_HANDLER_TYPE_XUL)
+    if (mType & NS_HANDLER_TYPE_XUL)
       GetEventType(event);
     if (event.IsEmpty())
       return;
   }
 
-  if (event.IsEmpty())
-    return;
-  mEventName = getter_AddRefs(NS_NewAtom(event));
+  mEventName = do_GetAtom(event);
 
   if (aPhase) {
     const nsDependentString phase(aPhase);
@@ -874,7 +861,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   // Modifiers are supported by both types of handlers (XUL and XBL).  
   nsAutoString modifiers(aModifiers);
-  if (mType == NS_HANDLER_TYPE_XUL)
+  if (mType & NS_HANDLER_TYPE_XUL)
     mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::modifiers, modifiers);
   
   if (!modifiers.IsEmpty()) {
@@ -903,7 +890,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   nsAutoString key(aCharCode);
   if (key.IsEmpty()) {
-    if (mType == NS_HANDLER_TYPE_XUL) {
+    if (mType & NS_HANDLER_TYPE_XUL) {
       mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::key, key);
       if (key.IsEmpty()) 
         mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::charcode, key);
@@ -922,89 +909,49 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   }
   else {
     key.Assign(aKeyCode);
-    if (mType == NS_HANDLER_TYPE_XUL)
+    if (mType & NS_HANDLER_TYPE_XUL)
       mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::keycode, key);
     
     if (!key.IsEmpty())
       mDetail = GetMatchingKeyCode(key);
   }
 
+  nsAutoString preventDefault(aPreventDefault);
+  if (preventDefault.Equals(NS_LITERAL_STRING("true")))
+    mType |= NS_HANDLER_TYPE_PREVENTDEFAULT;
 }
 
 PRBool
-nsXBLPrototypeHandler::ModifiersMatchMask(nsIDOMUIEvent* aEvent)
+nsXBLPrototypeHandler::ModifiersMatchMask(nsIDOMUIEvent* aEvent,
+                                          PRInt32 aModifiersMask)
 {
   nsCOMPtr<nsIDOMKeyEvent> key(do_QueryInterface(aEvent));
   nsCOMPtr<nsIDOMMouseEvent> mouse(do_QueryInterface(aEvent));
 
   PRBool keyPresent;
-  key ? key->GetMetaKey(&keyPresent) : mouse->GetMetaKey(&keyPresent);
-  if (keyPresent != ((mKeyMask & cMeta) != 0))
-    return PR_FALSE;
+  if (aModifiersMask & cMeta) {
+    key ? key->GetMetaKey(&keyPresent) : mouse->GetMetaKey(&keyPresent);
+    if (keyPresent != ((mKeyMask & cMeta) != 0))
+      return PR_FALSE;
+  }
 
-  key ? key->GetShiftKey(&keyPresent) : mouse->GetShiftKey(&keyPresent);
-  if (keyPresent != ((mKeyMask & cShift) != 0))
-    return PR_FALSE;
-  
-  key ? key->GetAltKey(&keyPresent) : mouse->GetAltKey(&keyPresent);
-  if (keyPresent != ((mKeyMask & cAlt) != 0))
-    return PR_FALSE;
+  if (aModifiersMask & cShift) {
+    key ? key->GetShiftKey(&keyPresent) : mouse->GetShiftKey(&keyPresent);
+    if (keyPresent != ((mKeyMask & cShift) != 0))
+      return PR_FALSE;
+  }
 
-  key ? key->GetCtrlKey(&keyPresent) : mouse->GetCtrlKey(&keyPresent);
-  if (keyPresent != ((mKeyMask & cControl) != 0))
-    return PR_FALSE;
+  if (aModifiersMask & cAlt) {
+    key ? key->GetAltKey(&keyPresent) : mouse->GetAltKey(&keyPresent);
+    if (keyPresent != ((mKeyMask & cAlt) != 0))
+      return PR_FALSE;
+  }
+
+  if (aModifiersMask & cControl) {
+    key ? key->GetCtrlKey(&keyPresent) : mouse->GetCtrlKey(&keyPresent);
+    if (keyPresent != ((mKeyMask & cControl) != 0))
+      return PR_FALSE;
+  }
 
   return PR_TRUE;
 }
-
-
-nsresult
-nsXBLPrototypeHandler::GetTextData(nsIContent *aParent, nsString& aResult)
-{
-  aResult.Truncate(0);
-
-  nsCOMPtr<nsIContent> textChild;
-  PRInt32 textCount;
-  aParent->ChildCount(textCount);
-  nsAutoString answer;
-  for (PRInt32 j = 0; j < textCount; j++) {
-    // Get the child.
-    aParent->ChildAt(j, *getter_AddRefs(textChild));
-    nsCOMPtr<nsIDOMText> text(do_QueryInterface(textChild));
-    if (text) {
-      nsAutoString data;
-      text->GetData(data);
-      aResult += data;
-    }
-  }
-  return NS_OK;
-}
-
-///////////////////////////////////////////////////////////////////////////////////
-
-nsresult
-NS_NewXBLPrototypeHandler(const PRUnichar* aEvent, const PRUnichar* aPhase,
-                          const PRUnichar* aAction, const PRUnichar* aCommand,
-                          const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
-                          const PRUnichar* aModifiers, const PRUnichar* aButton,
-                          const PRUnichar* aClickCount, 
-                          nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = new nsXBLPrototypeHandler(aEvent, aPhase, aAction, aCommand, aKeyCode,
-                                       aCharCode, aModifiers, aButton, aClickCount);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(*aResult);
-  return NS_OK;
-}
-
-nsresult
-NS_NewXULKeyHandler(nsIContent* aHandlerElement, nsIXBLPrototypeHandler** aResult)
-{
-  *aResult = new nsXBLPrototypeHandler(aHandlerElement);
-  if (!*aResult)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(*aResult);
-  return NS_OK;
-}
-
