@@ -74,13 +74,6 @@ nsLDAPServiceEntry::~nsLDAPServiceEntry()
 //
 PRBool nsLDAPServiceEntry::Init()
 {
-    nsresult rv;
-
-    rv = NS_NewISupportsArray(getter_AddRefs(mListeners));
-    if (NS_FAILED(rv)) {
-        return PR_FALSE;
-    }
-
     return PR_TRUE;
 }
 
@@ -120,7 +113,7 @@ PRUint32 nsLDAPServiceEntry::GetLeases()
 
 // Get/Set the nsLDAPServer object for this entry.
 //
-nsILDAPServer *nsLDAPServiceEntry::GetServer()
+already_AddRefed<nsILDAPServer> nsLDAPServiceEntry::GetServer()
 {
     nsILDAPServer *server;
 
@@ -139,7 +132,7 @@ PRBool nsLDAPServiceEntry::SetServer(nsILDAPServer *aServer)
 
 // Get/Set/Clear the nsLDAPConnection object for this entry.
 //
-nsILDAPConnection *nsLDAPServiceEntry::GetConnection()
+already_AddRefed<nsILDAPConnection> nsLDAPServiceEntry::GetConnection()
 {
     nsILDAPConnection *conn;
 
@@ -153,7 +146,7 @@ void nsLDAPServiceEntry::SetConnection(nsILDAPConnection *aConnection)
 
 // Get/Set the nsLDAPMessage object for this entry (it's a "cache").
 //
-nsILDAPMessage *nsLDAPServiceEntry::GetMessage()
+already_AddRefed<nsILDAPMessage> nsLDAPServiceEntry::GetMessage()
 {
     nsILDAPMessage *message;
 
@@ -168,31 +161,26 @@ void nsLDAPServiceEntry::SetMessage(nsILDAPMessage *aMessage)
 // Push/Pop pending listeners/callback for this server entry. This is
 // implemented as a "stack" on top of the nsVoidArrays, since we can
 // potentially have more than one listener waiting for the connection
-// to be avilable for consumption.
+// to be available for consumption.
 //
-nsILDAPMessageListener *nsLDAPServiceEntry::PopListener()
+already_AddRefed<nsILDAPMessageListener> nsLDAPServiceEntry::PopListener()
 {
     nsILDAPMessageListener *listener;
-    PRUint32 count;
-
-    mListeners->Count(&count);
+    PRInt32 count = mListeners.Count();
     if (!count) {
         return 0;
     }
 
-    listener = NS_STATIC_CAST(nsILDAPMessageListener *,
-                              mListeners->ElementAt(0));
-    mListeners->RemoveElementAt(0);
+    listener = mListeners[0];
+    NS_ADDREF(listener); // keep it alive
+    mListeners.RemoveObjectAt(0);
 
     return listener;
 }
 PRBool nsLDAPServiceEntry::PushListener(nsILDAPMessageListener *listener)
 {
     PRBool ret;
-    PRUint32 count;
-
-    mListeners->Count(&count);
-    ret = mListeners->InsertElementAt(listener, count);
+    ret = mListeners.InsertObjectAt(listener, mListeners.Count());
 
     return ret;
 }
@@ -226,7 +214,7 @@ PRBool nsLDAPServiceEntry::DeleteEntry()
 // 
 NS_IMPL_THREADSAFE_ISUPPORTS2(nsLDAPService,
                               nsILDAPService,
-                              nsILDAPMessageListener);
+                              nsILDAPMessageListener)
 
 
 // constructor
@@ -236,7 +224,6 @@ nsLDAPService::nsLDAPService()
       mServers(0),
       mConnections(0)
 {
-    NS_INIT_ISUPPORTS();
 }
 
 // destructor
@@ -402,7 +389,7 @@ NS_IMETHODIMP nsLDAPService::GetServer(const PRUnichar *aKey,
         *_retval = 0;
         return NS_ERROR_FAILURE;
     }
-    if (!(*_retval = entry->GetServer())) {
+    if (!(*_retval = entry->GetServer().get())) {
         return NS_ERROR_FAILURE;
     }
 
@@ -436,8 +423,8 @@ NS_IMETHODIMP nsLDAPService::RequestConnection(const PRUnichar *aKey,
         }
         entry->SetTimestamp();
 
-        conn = getter_AddRefs(entry->GetConnection());
-        message = getter_AddRefs(entry->GetMessage());
+        conn = entry->GetConnection();
+        message = entry->GetMessage();
     }
 
     if (conn) {
@@ -494,7 +481,7 @@ NS_IMETHODIMP nsLDAPService::GetConnection(const PRUnichar *aKey,
     }
     entry->SetTimestamp();
     entry->IncrementLeases();
-    if (!(*_retval = entry->GetConnection())){
+    if (!(*_retval = entry->GetConnection().get())){
         return NS_ERROR_FAILURE;
     }
 
@@ -649,7 +636,7 @@ nsLDAPService::OnLDAPMessage(nsILDAPMessage *aMessage)
                 return NS_ERROR_FAILURE;
             }
 
-            message = getter_AddRefs(entry->GetMessage());
+            message = entry->GetMessage();
             if (message) {
                 // We already have a message, lets keep that one.
                 //
@@ -697,10 +684,10 @@ nsLDAPService::OnLDAPMessage(nsILDAPMessage *aMessage)
     return NS_OK;
 }
 
-// void onLDAPInit (in nsresult aStatus); */
+// void onLDAPInit (in nsILDAPConnection aConn, in nsresult aStatus); */
 //
 NS_IMETHODIMP
-nsLDAPService::OnLDAPInit(nsresult aStatus)
+nsLDAPService::OnLDAPInit(nsILDAPConnection *aConn, nsresult aStatus)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -709,7 +696,7 @@ nsLDAPService::OnLDAPInit(nsresult aStatus)
 //
 nsresult
 nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
-                                            nsILDAPMessageListener *aListener)
+                                   nsILDAPMessageListener *aListener)
 {
     nsCOMPtr<nsILDAPOperation> operation;
     nsCOMPtr<nsILDAPServer> server;
@@ -717,28 +704,33 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
     nsCOMPtr<nsILDAPConnection> conn, conn2;
     nsCOMPtr<nsILDAPMessage> message;
     nsCAutoString host;
-    nsXPIDLString binddn;
-    nsXPIDLString password;
+    nsCAutoString binddn;
+    nsCAutoString password;
     PRInt32 port;
     PRUint32 options;
+    PRUint32 protocolVersion;
     nsresult rv;
 
-    server = getter_AddRefs(aEntry->GetServer());
+    server = aEntry->GetServer();
     if (!server) {
         return NS_ERROR_FAILURE;
     }
 
-    // Get username and password from the server entry.
+    // Get username, password, and protocol version from the server entry.
     //
-    rv = server->GetBinddn(getter_Copies(binddn));
+    rv = server->GetBinddn(binddn);
     if (NS_FAILED(rv)) {
         return NS_ERROR_FAILURE;
     }
-    rv = server->GetPassword(getter_Copies(password));
+    rv = server->GetPassword(password);
     if (NS_FAILED(rv)) {
         return NS_ERROR_FAILURE;
     }
-            
+    rv = server->GetProtocolVersion(&protocolVersion);
+    if (NS_FAILED(rv)) {
+        return NS_ERROR_FAILURE;
+    }
+
     // Get the host and port out of the URL, which is in the server entry.
     //
     rv = server->GetUrl(getter_AddRefs(url));
@@ -771,7 +763,7 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
     //
     rv = conn->Init(host.get(), port, 
                     (options & nsILDAPURL::OPT_SECURE) ? PR_TRUE : PR_FALSE, 
-                    nsnull, this);
+                    binddn, this, nsnull, protocolVersion);
     if (NS_FAILED(rv)) {
         switch (rv) {
         // Only pass along errors we are aware of
@@ -796,8 +788,8 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
     {
         nsAutoLock lock(mLock);
 
-        conn2 = getter_AddRefs(aEntry->GetConnection());
-        message = getter_AddRefs(aEntry->GetMessage());
+        conn2 = aEntry->GetConnection();
+        message = aEntry->GetMessage();
     }
 
     if (conn2) {
@@ -840,7 +832,7 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
         return NS_ERROR_FAILURE;
     }
 
-    rv = operation->Init(conn, this);
+    rv = operation->Init(conn, this, nsnull);
     if (NS_FAILED(rv)) {
         return NS_ERROR_UNEXPECTED; // this should never happen
     }
@@ -849,7 +841,7 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
     //
     // Here we need to support the password, see bug #75990
     // 
-    rv = operation->SimpleBind(0);
+    rv = operation->SimpleBind(password);
     if (NS_FAILED(rv)) {
         switch (rv) {
         // Only pass along errors we are aware of
@@ -869,12 +861,12 @@ nsLDAPService::EstablishConnection(nsLDAPServiceEntry *aEntry,
 
 /* AString createFilter (in unsigned long aMaxSize, in AString aPattern, in AString aPrefix, in AString aSuffix, in AString aAttr, in AString aValue); */
 NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize, 
-                                          const nsAString & aPattern,
-                                          const nsAString & aPrefix,
-                                          const nsAString & aSuffix,
-                                          const nsAString & aAttr,
-                                          const nsAString & aValue,
-                                          nsAString & _retval)
+                                          const nsACString & aPattern,
+                                          const nsACString & aPrefix,
+                                          const nsACString & aSuffix,
+                                          const nsACString & aAttr,
+                                          const nsACString & aValue,
+                                          nsACString & _retval)
 {
     if (!aMaxSize) {
         return NS_ERROR_INVALID_ARG;
@@ -882,7 +874,7 @@ NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize,
 
     // prepare to tokenize |value| for %vM ... %vN
     //
-    nsReadingIterator<PRUnichar> iter, iterEnd; // setup the iterators
+    nsReadingIterator<char> iter, iterEnd; // setup the iterators
     aValue.BeginReading(iter);
     aValue.EndReading(iterEnd);
 
@@ -924,11 +916,11 @@ NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize,
     //
     nsresult rv;
     int result = ldap_create_filter(buffer, aMaxSize, 
-                   NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(aPattern).get()),
-                   NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(aPrefix).get()), 
-                   NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(aSuffix).get()), 
-                   NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(aAttr).get()),
-                   NS_CONST_CAST(char *, NS_ConvertUCS2toUTF8(aValue).get()),
+                   NS_CONST_CAST(char *, PromiseFlatCString(aPattern).get()),
+                   NS_CONST_CAST(char *, PromiseFlatCString(aPrefix).get()),
+                   NS_CONST_CAST(char *, PromiseFlatCString(aSuffix).get()),
+                   NS_CONST_CAST(char *, PromiseFlatCString(aAttr).get()),
+                   NS_CONST_CAST(char *, PromiseFlatCString(aValue).get()),
                    valueWords);
     switch (result) {
     case LDAP_SUCCESS:
@@ -954,7 +946,7 @@ NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize,
         break;
     }
 
-    _retval = NS_ConvertUTF8toUCS2(buffer);
+    _retval.Assign(buffer);
 
     // done with the array and the buffer
     //
@@ -967,8 +959,8 @@ NS_IMETHODIMP nsLDAPService::CreateFilter(PRUint32 aMaxSize,
 // Count the number of space-separated tokens between aIter and aIterEnd
 //
 PRUint32
-nsLDAPService::CountTokens(nsReadingIterator<PRUnichar> aIter,
-                           nsReadingIterator<PRUnichar> aIterEnd)
+nsLDAPService::CountTokens(nsReadingIterator<char> aIter,
+                           nsReadingIterator<char> aIterEnd)
 {
     PRUint32 count(0);
 
@@ -978,7 +970,8 @@ nsLDAPService::CountTokens(nsReadingIterator<PRUnichar> aIter,
     
         // move past any leading spaces
         //
-        while (aIter != aIterEnd && nsCRT::IsAsciiSpace(*aIter)) {
+        while (aIter != aIterEnd &&
+               ldap_utf8isspace(NS_CONST_CAST(char *, aIter.get()))){
             ++aIter;
         }
 
@@ -986,7 +979,7 @@ nsLDAPService::CountTokens(nsReadingIterator<PRUnichar> aIter,
         //
         while (aIter != aIterEnd) {
 
-            if (nsCRT::IsAsciiSpace(*aIter)) {
+            if (ldap_utf8isspace(NS_CONST_CAST(char *, aIter.get()))) {
                 ++count;    // token finished; increment the count
                 ++aIter;    // move past the space
                 break;
@@ -1010,56 +1003,25 @@ nsLDAPService::CountTokens(nsReadingIterator<PRUnichar> aIter,
 
 // return the next token in this iterator
 //
-char *
-nsLDAPService::NextToken(nsReadingIterator<PRUnichar> & aIter,
-                         nsReadingIterator<PRUnichar> & aIterEnd)
+char*
+nsLDAPService::NextToken(nsReadingIterator<char> & aIter,
+                         nsReadingIterator<char> & aIterEnd)
 {
     // move past any leading whitespace
     //
-    while ( aIter != aIterEnd && nsCRT::IsAsciiSpace(*aIter) ) {
+    while (aIter != aIterEnd &&
+           ldap_utf8isspace(NS_CONST_CAST(char *, aIter.get()))) {
         ++aIter;
     }
 
-    nsAString::const_iterator start(aIter);
+    nsACString::const_iterator start(aIter);
 
     // copy the token into our local variable
     //
-    while ( aIter != aIterEnd && !nsCRT::IsAsciiSpace(*aIter) ) {
+    while (aIter != aIterEnd &&
+           !ldap_utf8isspace(NS_CONST_CAST(char *, aIter.get()))) {
         ++aIter;
     }
 
-    return ToNewUTF8String(Substring(start, aIter));
-}
-
-// Note that these 2 functions might go away in the future, see bug 84186.
-//
-// string UCS2ToUTF8 (in AString aString);
-NS_IMETHODIMP
-nsLDAPService::UCS2toUTF8(const nsAString &aString,
-                                        char **_retval)
-{
-    char *str;
-
-    if (!_retval) {
-        NS_ERROR("nsLDAPService::UCS2toUTF8: null pointer ");
-        return NS_ERROR_NULL_POINTER;
-    }
-
-    str = ToNewUTF8String(aString);
-    if (!str) {
-        NS_ERROR("nsLDAPService::UCS2toUTF8: out of memory ");
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-    *_retval = str;
-    return NS_OK;
-}
-
-// AString UTF8ToUCS2 (in string aString);
-NS_IMETHODIMP
-nsLDAPService::UTF8toUCS2(const char *aString,
-                                        nsAString &_retval)
-{
-    _retval = NS_ConvertUTF8toUCS2(aString);
-    return NS_OK;
+    return ToNewCString(Substring(start, aIter));
 }
