@@ -39,11 +39,16 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsGlobalWindow.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIScriptContext.h"
 #include "nsIWebShell.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellLoadInfo.h"
 #include "nsIWebNavigation.h"
+#include "nsCDefaultURIFixup.h"
+#include "nsIURIFixup.h"
 #include "nsIURL.h"
+#include "nsIJARURI.h"
 #include "nsIIOService.h"
 #include "nsIServiceManager.h"
 #include "nsNetUtil.h"
@@ -54,7 +59,6 @@
 #include "nsEscape.h"
 #include "nsJSUtils.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsICodebasePrincipal.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
@@ -62,18 +66,17 @@
 #include "nsXPIDLString.h"
 #include "nsDOMError.h"
 #include "nsDOMClassInfo.h"
-#include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsCRT.h"
+#include "nsIProtocolHandler.h"
+#include "nsReadableUtils.h"
 
-
-static nsresult EscapeNonAsciiInURI(const nsAString& aHref, nsACString& aEscapedHref)
+static nsresult
+GetDocumentCharacterSetForURI(const nsAString& aHref, nsACString& aCharset)
 {
-  aEscapedHref.Truncate(0);
+  aCharset.Truncate();
 
   nsresult rv;
 
-  // Get a document charset, no escaping in case of failure.
   nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -82,76 +85,27 @@ static nsresult EscapeNonAsciiInURI(const nsAString& aHref, nsACString& aEscaped
   rv = stack->Peek(&cx);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIScriptGlobalObject> nativeGlob;
-  nsJSUtils::GetDynamicScriptGlobal(cx, getter_AddRefs(nativeGlob));
-  NS_ENSURE_TRUE(nativeGlob, NS_ERROR_FAILURE);
+  if (cx) {
+    nsCOMPtr<nsIDOMWindow> window =
+      do_QueryInterface(nsJSUtils::GetDynamicScriptGlobal(cx));
+    NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsIDOMWindow> window = do_QueryInterface(nativeGlob);
-  NS_ENSURE_TRUE(window, NS_ERROR_FAILURE);
+    nsCOMPtr<nsIDOMDocument> domDoc;
+    rv = window->GetDocument(getter_AddRefs(domDoc));
+    NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  rv = window->GetDocument(getter_AddRefs(domDoc));
-  NS_ENSURE_SUCCESS(rv, rv);
-  if (!domDoc)
-    return NS_OK;
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
 
-  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-  NS_ENSURE_TRUE(doc, NS_ERROR_FAILURE);
-
-  nsAutoString charset;
-  rv = doc->GetDocumentCharacterSet(charset);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Convert from Unicode to a document charset.
-  nsCOMPtr <nsICharsetConverterManager2> ccm2 = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr <nsIAtom> charsetAtom;
-  rv = ccm2->GetCharsetAtom(charset.get(), getter_AddRefs(charsetAtom));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIUnicodeEncoder> encoder;
-  rv = ccm2->GetUnicodeEncoder(charsetAtom, getter_AddRefs(encoder));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  const nsAFlatString &tmp = PromiseFlatString(aHref);
-  PRInt32 len = tmp.Length();
-  PRInt32 bufferLen;
-
-  rv = encoder->GetMaxLength(tmp.get(), len, &bufferLen);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  char* buffer = (char *) nsMemory::Alloc(bufferLen);
-  NS_ENSURE_TRUE(buffer, NS_ERROR_OUT_OF_MEMORY);
-
-  rv = encoder->Convert(tmp.get(), &len, buffer, &bufferLen);
-  if (NS_FAILED(rv)) {
-    nsMemory::Free(buffer);
-    return rv;
-  }
-
-  // Escape any 8 bit data.
-  const char* hexChars = "0123456789ABCDEF";
-  for (PRUint32 i = 0; i < (PRUint32) len; i++) {
-    unsigned char c = (unsigned char) buffer[i];
-    if (c < 128) {
-      aEscapedHref.Append((char) c);
-    }
-    else {
-      aEscapedHref.Append('%');
-      aEscapedHref.Append(hexChars[c >> 4]);     /* high nibble */
-      aEscapedHref.Append(hexChars[c & 0x0f]);   /* low nibble */
+    if (doc) {
+      aCharset = doc->GetDocumentCharacterSet();
     }
   }
 
-  nsMemory::Free(buffer);
-
-  return rv;
+  return NS_OK;
 }
 
 LocationImpl::LocationImpl(nsIDocShell *aDocShell)
 {
-  NS_INIT_REFCNT();
   mDocShell = aDocShell; // Weak Reference
 }
 
@@ -172,7 +126,8 @@ NS_INTERFACE_MAP_END
 NS_IMPL_ADDREF(LocationImpl)
 NS_IMPL_RELEASE(LocationImpl)
 
-NS_IMETHODIMP_(void) LocationImpl::SetDocShell(nsIDocShell *aDocShell)
+void
+LocationImpl::SetDocShell(nsIDocShell *aDocShell)
 {
    mDocShell = aDocShell; // Weak Reference
 }
@@ -180,6 +135,8 @@ NS_IMETHODIMP_(void) LocationImpl::SetDocShell(nsIDocShell *aDocShell)
 nsresult
 LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 {
+  *aLoadInfo = nsnull;
+
   nsresult result;
   // Get JSContext from stack.
   nsCOMPtr<nsIJSContextStack>
@@ -193,6 +150,16 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
   if (NS_FAILED(stack->Peek(&cx)))
     return NS_ERROR_FAILURE;
 
+  if (!cx) {
+    // No cx means that there's no JS running, or at least no JS that
+    // was run through code that properly pushed a context onto the
+    // context stack (as all code that runs JS off of web pages
+    // does). Going further from here will crash, so lets not do
+    // that...
+
+    return NS_OK;
+  }
+
   // Get security manager.
   nsCOMPtr<nsIScriptSecurityManager>
     secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &result));
@@ -232,8 +199,65 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
   return NS_OK;
 }
 
+// Walk up the docshell hierarchy and find a usable base URI. Basically 
+// anything that would allow a relative uri.
+
 nsresult
-LocationImpl::GetURI(nsIURI** aURI)
+LocationImpl::FindUsableBaseURI(nsIURI * aBaseURI, nsIDocShell * aParent, nsIURI ** aUsableURI)
+{
+  if (!aBaseURI || !aParent)
+    return NS_ERROR_FAILURE;
+  NS_ENSURE_ARG_POINTER(aUsableURI);
+    
+  *aUsableURI = nsnull;
+  nsresult rv = NS_OK;    
+  nsCOMPtr<nsIDocShell> parentDS = aParent;
+  nsCOMPtr<nsIURI> baseURI = aBaseURI;
+  nsCOMPtr<nsIIOService> ioService(do_GetService(NS_IOSERVICE_CONTRACTID, &rv));
+
+  while(NS_SUCCEEDED(rv) && baseURI && ioService) {
+    // Check if the current base uri supports relative uris.
+    // We make this check by looking at the protocol flags of
+    // the protocol handler. If the protocol flags has URI_NORELATIVE,
+    // it means that the base uri does not support relative uris.
+    nsCAutoString scheme;
+    baseURI->GetScheme(scheme);
+    nsCOMPtr<nsIProtocolHandler> protocolHandler;
+    // Get the protocol handler for the base uri.
+    ioService->GetProtocolHandler(scheme.get(), getter_AddRefs(protocolHandler));
+    if (!protocolHandler)
+      return NS_ERROR_FAILURE;
+    PRUint32 pFlags; // Is there a default value for the protocol flags?
+    protocolHandler->GetProtocolFlags(&pFlags);
+    if (!(pFlags & nsIProtocolHandler::URI_NORELATIVE)) {
+      *aUsableURI = baseURI;
+      NS_ADDREF(*aUsableURI);
+      return NS_OK;
+    }
+
+    // Get the same type parent docshell
+    nsCOMPtr<nsIDocShellTreeItem> docShellAsTreeItem(do_QueryInterface(parentDS));
+    if (!docShellAsTreeItem)
+      return NS_ERROR_FAILURE;
+    nsCOMPtr<nsIDocShellTreeItem> parentDSTreeItem;
+    docShellAsTreeItem->GetSameTypeParent(getter_AddRefs(parentDSTreeItem));      
+    nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(parentDSTreeItem));
+
+    // Get the parent docshell's uri
+    if (webNav) {
+      rv = webNav->GetCurrentURI(getter_AddRefs(baseURI));
+      parentDS = do_QueryInterface(parentDSTreeItem);
+    }
+    else
+      return NS_ERROR_FAILURE;
+  }  // while 
+
+  return rv;
+}
+
+
+nsresult
+LocationImpl::GetURI(nsIURI** aURI, PRBool aGetInnermostURI)
 {
   *aURI = nsnull;
 
@@ -244,7 +268,30 @@ LocationImpl::GetURI(nsIURI** aURI)
     return rv;
   }
 
-  return webNav->GetCurrentURI(aURI);
+  nsCOMPtr<nsIURI> uri;
+  rv = webNav->GetCurrentURI(getter_AddRefs(uri));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // It is valid for docshell to return a null URI. Don't try to fixup
+  // if this happens.
+  if (!uri) {
+    return NS_OK;
+  }
+
+  if (aGetInnermostURI) {
+    nsCOMPtr<nsIJARURI> jarURI(do_QueryInterface(uri));
+    while (jarURI) {
+      jarURI->GetJARFile(getter_AddRefs(uri));
+      jarURI = do_QueryInterface(uri);
+    }
+  }
+
+  NS_ASSERTION(uri, "nsJARURI screwed up?");
+
+  nsCOMPtr<nsIURIFixup> urifixup(do_GetService(NS_URIFIXUP_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return urifixup->CreateExposableURI(uri, aURI);
 }
 
 nsresult
@@ -300,7 +347,8 @@ LocationImpl::GetHash(nsAString& aHash)
     NS_UnescapeURL(ref);
 
     if (NS_SUCCEEDED(result) && !ref.IsEmpty()) {
-      aHash.Assign(NS_LITERAL_STRING("#") + NS_ConvertASCIItoUCS2(ref));
+      aHash.Assign(PRUnichar('#'));
+      AppendASCIItoUTF16(ref, aHash);
     }
   }
 
@@ -338,12 +386,12 @@ LocationImpl::SetHash(const nsAString& aHash)
 NS_IMETHODIMP
 LocationImpl::GetHost(nsAString& aHost)
 {
-  aHost.SetLength(0);
+  aHost.Truncate();
 
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
-  result = GetURI(getter_AddRefs(uri));
+  result = GetURI(getter_AddRefs(uri), PR_TRUE);
 
   if (uri) {
     nsCAutoString hostport;
@@ -351,7 +399,7 @@ LocationImpl::GetHost(nsAString& aHost)
     result = uri->GetHostPort(hostport);
 
     if (NS_SUCCEEDED(result)) {
-      aHost = NS_ConvertUTF8toUCS2(hostport);
+      AppendUTF8toUTF16(hostport, aHost);
     }
   }
 
@@ -377,12 +425,12 @@ LocationImpl::SetHost(const nsAString& aHost)
 NS_IMETHODIMP
 LocationImpl::GetHostname(nsAString& aHostname)
 {
-  aHostname.SetLength(0);
+  aHostname.Truncate();
 
   nsCOMPtr<nsIURI> uri;
   nsresult result;
 
-  result = GetURI(getter_AddRefs(uri));
+  result = GetURI(getter_AddRefs(uri), PR_TRUE);
 
   if (uri) {
     nsCAutoString host;
@@ -390,7 +438,7 @@ LocationImpl::GetHostname(nsAString& aHostname)
     result = uri->GetHost(host);
 
     if (NS_SUCCEEDED(result)) {
-      aHostname = NS_ConvertUTF8toUCS2(host);
+      AppendUTF8toUTF16(host, aHostname);
     }
   }
 
@@ -416,7 +464,7 @@ LocationImpl::SetHostname(const nsAString& aHostname)
 NS_IMETHODIMP
 LocationImpl::GetHref(nsAString& aHref)
 {
-  aHref.SetLength(0);
+  aHref.Truncate();
 
   nsCOMPtr<nsIURI> uri;
   nsresult result;
@@ -429,7 +477,7 @@ LocationImpl::GetHref(nsAString& aHref)
     result = uri->GetSpec(uriString);
 
     if (NS_SUCCEEDED(result)) {
-      aHref = NS_ConvertUTF8toUCS2(uriString);
+      AppendUTF8toUTF16(uriString, aHref);
     }
   }
 
@@ -494,20 +542,20 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
                               nsIURI* aBase, PRBool aReplace)
 {
   nsresult result;
-  nsCOMPtr<nsIURI> newUri;
+  nsCOMPtr<nsIURI> newUri, baseURI;
 
-  // 'aHref' is supposed to be already escaped by the script
-  // but if it contains 8 bit characters without escaped then force to escape it.
-  // This is for comptibility reason, some of the existing pages do not escape hrefs
-  // and they are working on 4.x.
-  nsCAutoString escapedHref;
-  if (!nsCRT::IsAscii(PromiseFlatString(aHref).get()))
-    (void) EscapeNonAsciiInURI(aHref, escapedHref);
+  // Try to make sure the base url is something that will be useful. 
+  result = FindUsableBaseURI(aBase,  mDocShell, getter_AddRefs(baseURI));
+  if (!baseURI)  {
+    // If nothing useful was found, just use what you have.
+    baseURI = aBase;
+  }
 
-  if (escapedHref.IsEmpty())
-    result = NS_NewURI(getter_AddRefs(newUri), aHref, nsnull, aBase);
+  nsCAutoString docCharset;
+  if (NS_SUCCEEDED(GetDocumentCharacterSetForURI(aHref, docCharset)))
+    result = NS_NewURI(getter_AddRefs(newUri), aHref, docCharset.get(), baseURI);
   else
-    result = NS_NewURI(getter_AddRefs(newUri), escapedHref, nsnull, aBase);
+    result = NS_NewURI(getter_AddRefs(newUri), aHref, nsnull, baseURI);
 
   if (newUri && mDocShell) {
     nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
@@ -537,10 +585,11 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
 
       result = stack->Peek(&cx);
       if (cx) {
-        nsIScriptContext* scriptCX = (nsIScriptContext*)JS_GetContextPrivate(cx);
-       
-        if (scriptCX) {
-          scriptCX->GetProcessingScriptTag(&inScriptTag);
+        nsIScriptContext *scriptContext =
+          nsJSUtils::GetDynamicScriptContext(cx);
+
+        if (scriptContext) {
+          inScriptTag = scriptContext->GetProcessingScriptTag();
         }  
       } //cx
     }  // stack
@@ -560,7 +609,7 @@ LocationImpl::SetHrefWithBase(const nsAString& aHref,
 NS_IMETHODIMP
 LocationImpl::GetPathname(nsAString& aPathname)
 {
-  aPathname.SetLength(0);
+  aPathname.Truncate();
 
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
@@ -574,7 +623,7 @@ LocationImpl::GetPathname(nsAString& aPathname)
     result = url->GetFilePath(file);
 
     if (NS_SUCCEEDED(result)) {
-      aPathname = NS_ConvertUTF8toUCS2(file);
+      AppendUTF8toUTF16(file, aPathname);
     }
   }
 
@@ -605,7 +654,7 @@ LocationImpl::GetPort(nsAString& aPort)
   nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetURI(getter_AddRefs(uri));
+  result = GetURI(getter_AddRefs(uri), PR_TRUE);
 
   if (uri) {
     PRInt32 port;
@@ -667,7 +716,7 @@ LocationImpl::GetProtocol(nsAString& aProtocol)
     result = uri->GetScheme(protocol);
 
     if (NS_SUCCEEDED(result)) {
-      aProtocol.Assign(NS_ConvertASCIItoUCS2(protocol));
+      CopyASCIItoUTF16(protocol, aProtocol);
       aProtocol.Append(PRUnichar(':'));
     }
   }
@@ -709,7 +758,8 @@ LocationImpl::GetSearch(nsAString& aSearch)
     result = url->GetQuery(search);
 
     if (NS_SUCCEEDED(result) && !search.IsEmpty()) {
-      aSearch.Assign(NS_LITERAL_STRING("?") + NS_ConvertUTF8toUCS2(search));
+      aSearch.Assign(PRUnichar('?'));
+      AppendUTF8toUTF16(search, aSearch);
     }
   }
 
@@ -862,30 +912,27 @@ LocationImpl::GetSourceDocument(JSContext* cx, nsIDocument** aDocument)
   // could break if any of the connections along the way change.
   // I wish there were a better way.
 
-  nsresult result = NS_ERROR_FAILURE;
+  nsresult rv = NS_ERROR_FAILURE;
 
   // We need to use the dynamically scoped global and assume that the
   // current JSContext is a DOM context with a nsIScriptGlobalObject so
   // that we can get the url of the caller.
   // XXX This will fail on non-DOM contexts :(
 
-  nsCOMPtr<nsIScriptGlobalObject> nativeGlob;
-  nsJSUtils::GetDynamicScriptGlobal(cx, getter_AddRefs(nativeGlob));
+  nsCOMPtr<nsIDOMWindow> window =
+    do_QueryInterface(nsJSUtils::GetDynamicScriptGlobal(cx), &rv);
 
-  if (nativeGlob) {
-    nsCOMPtr<nsIDOMWindow> window = do_QueryInterface(nativeGlob, &result);
-
-    if (window) {
-      nsCOMPtr<nsIDOMDocument> domDoc;
-      result = window->GetDocument(getter_AddRefs(domDoc));
-      if (domDoc) {
-        return CallQueryInterface(domDoc, aDocument);
-      }
+  if (window) {
+    nsCOMPtr<nsIDOMDocument> domDoc;
+    rv = window->GetDocument(getter_AddRefs(domDoc));
+    if (domDoc) {
+      return CallQueryInterface(domDoc, aDocument);
     }
   } else {
     *aDocument = nsnull;
   }
-  return result;
+
+  return rv;
 }
 
 nsresult
@@ -894,14 +941,11 @@ LocationImpl::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
   if (doc) {
-    rv = doc->GetBaseURL(*sourceURL);
-
-    if (!*sourceURL) {
-      doc->GetDocumentURL(sourceURL);
-    }
+    NS_IF_ADDREF(*sourceURL = doc->GetBaseURI());
   } else {
     *sourceURL = nsnull;
   }
+
   return rv;
 }
 
@@ -911,7 +955,7 @@ LocationImpl::GetSourceURL(JSContext* cx, nsIURI** sourceURL)
   nsCOMPtr<nsIDocument> doc;
   nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
   if (doc) {
-    doc->GetDocumentURL(sourceURL);
+    NS_IF_ADDREF(*sourceURL = doc->GetDocumentURI());
   } else {
     *sourceURL = nsnull;
   }
