@@ -76,7 +76,6 @@ PyG_Base::CreateNew(PyObject *pPyInstance, const nsIID &iid, void **ppResult)
 PyG_Base::PyG_Base(PyObject *instance, const nsIID &iid)
 {
 	// Note that "instance" is the _policy_ instance!!
-	NS_INIT_REFCNT();
 	PR_AtomicIncrement(&cGateways);
 	m_pBaseObject = GetDefaultGateway(instance);
 	// m_pWeakRef is an nsCOMPtr and needs no init.
@@ -90,7 +89,14 @@ PyG_Base::PyG_Base(PyObject *instance, const nsIID &iid)
 	// If XPCOM reference count logging is enabled, then allow us to give the Python class.
 	PyObject *realInstance = PyObject_GetAttrString(instance, "_obj_");
 	PyObject *r = PyObject_Repr(realInstance);
-	const char *szRepr = PyString_AsString(r);
+	const char *szRepr;
+	if (r==NULL) {
+		PyXPCOM_LogError("Getting the __repr__ of the object failed");
+		PyErr_Clear();
+		szRepr = "(repr failed!)";
+	}
+	else
+		szRepr = PyString_AsString(r);
 	if (szRepr==NULL) szRepr = "";
 	int reprOffset = *szRepr=='<' ? 1 : 0;
 	static const char *reprPrefix = "component:";
@@ -517,7 +523,7 @@ static nsresult do_dispatch(
 	method = PyObject_GetAttrString(real_ob, (char *)szMethodName);
 	if ( !method ) {
 		PyErr_Clear();
-		ret = NS_COMFALSE;
+		ret = NS_PYXPCOM_NO_SUCH_METHOD;
 		goto done;
 	}
 	// Make the call
@@ -564,7 +570,7 @@ nsresult PyG_Base::InvokeNativeViaPolicy(
 	nsresult nr = InvokeNativeViaPolicyInternal(szMethodName, ppResult, szFormat, va);
 	va_end(va);
 
-	if (nr==NS_COMFALSE) {
+	if (nr == NS_PYXPCOM_NO_SUCH_METHOD) {
 		// Only problem was missing method.
 		PyErr_Format(PyExc_AttributeError, "The object does not have a '%s' function.", szMethodName);
 	}
@@ -587,7 +593,7 @@ nsresult PyG_Base::InvokeNativeGetViaPolicy(
 	strncat(buf, szPropertyName, sizeof(buf)*sizeof(buf[0])-strlen(buf)-1);
 	buf[sizeof(buf)/sizeof(buf[0])-1] = '\0';
 	ret = InvokeNativeViaPolicyInternal(buf, ppResult, nsnull, nsnull);
-	if (ret == NS_COMFALSE) {
+	if (ret == NS_PYXPCOM_NO_SUCH_METHOD) {
 		// No method of that name - just try a property.
 		// Bit to a hack here to maintain the use of a policy.
 		// We actually get the policies underlying object
@@ -626,7 +632,6 @@ nsresult PyG_Base::InvokeNativeSetViaPolicy(
 {
 	if ( m_pPyObject == NULL || szPropertyName == NULL )
 		return NS_ERROR_NULL_POINTER;
-	PyObject *ob_ret = NULL;
 	nsresult ret = NS_OK;
 	PyObject *real_ob = NULL;
 	char buf[256];
@@ -637,7 +642,7 @@ nsresult PyG_Base::InvokeNativeSetViaPolicy(
 	va_start(va, szPropertyName);
 	ret = InvokeNativeViaPolicyInternal(buf, NULL, "O", va);
 	va_end(va);
-	if (ret == NS_COMFALSE) {
+	if (ret == NS_PYXPCOM_NO_SUCH_METHOD) {
 		// No method of that name - just try a property.
 		// Bit to a hack here to maintain the use of a policy.
 		// We actually get the policies underlying object
