@@ -1,36 +1,41 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
  *
- * The contents of this file are subject to the Netscape Public
- * License Version 1.1 (the "License"); you may not use this file
- * except in compliance with the License. You may obtain a copy of
- * the License at http://www.mozilla.org/NPL/
+ * ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
- * implied. See the License for the specific language governing
- * rights and limitations under the License.
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
  *
  * The Original Code is Mozilla Communicator client code, released
  * March 31, 1998.
  *
- * The Initial Developer of the Original Code is Netscape
- * Communications Corporation.  Portions created by Netscape are
- * Copyright (C) 1998 Netscape Communications Corporation. All
- * Rights Reserved.
+ * The Initial Developer of the Original Code is
+ * Netscape Communications Corporation.
+ * Portions created by the Initial Developer are Copyright (C) 1998
+ * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *
- * Alternatively, the contents of this file may be used under the
- * terms of the GNU Public License (the "GPL"), in which case the
- * provisions of the GPL are applicable instead of those above.
- * If you wish to allow use of your version of this file only
- * under the terms of the GPL and not to allow others to use your
- * version of this file under the NPL, indicate your decision by
- * deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL.  If you do not delete
- * the provisions above, a recipient may use your version of this
- * file under either the NPL or the GPL.
- */
+ * Alternatively, the contents of this file may be used under the terms of
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
 
 /*
  * JS Mark-and-Sweep Garbage Collector.
@@ -44,7 +49,7 @@
  */
 #include "jsstddef.h"
 #include <stdlib.h>     /* for free, called by JS_ARENA_DESTROY */
-#include <string.h>	/* for memset, called by jsarena.h macros if DEBUG */
+#include <string.h>     /* for memset, called by jsarena.h macros if DEBUG */
 #include "jstypes.h"
 #include "jsarena.h" /* Added by JSIFY */
 #include "jsutil.h" /* Added by JSIFY */
@@ -53,6 +58,7 @@
 #include "jsatom.h"
 #include "jscntxt.h"
 #include "jsconfig.h"
+#include "jsdbgapi.h"
 #include "jsfun.h"
 #include "jsgc.h"
 #include "jsinterp.h"
@@ -90,7 +96,7 @@ struct JSGCThing {
 };
 
 /*
- * A GC arena contains one flag byte for every thing in its heap, and supports
+ * A GC arena contains one flag byte for each thing in its heap, and supports
  * O(1) lookup of a flag given its thing's address.
  *
  * To implement this, we take advantage of the thing/flags numerology: given
@@ -145,7 +151,7 @@ struct JSGCThing {
  * no discontinuities over the split created by the thing pages.  So if, for a
  * given JSGCPageInfo *pi, we find that
  *
- *  pi->flags + ((jsuword)thing % 1023) / sizeof(JSGCThing) >= pi->split
+ *  pi->flags + ((jsuword)thing % 1024) / sizeof(JSGCThing) >= pi->split
  *
  * then we must add GC_THINGS_SIZE to the nominal flags pointer to jump over
  * all the thing pages that split the flags into two discontiguous spans.
@@ -263,18 +269,19 @@ js_InitGC(JSRuntime *rt, uint32 maxbytes)
     JS_ASSERT(sizeof(JSStackHeader) >= 2 * sizeof(jsval));
 
     if (!gc_finalizers[GCX_OBJECT]) {
-	gc_finalizers[GCX_OBJECT] = (GCFinalizeOp)js_FinalizeObject;
-	gc_finalizers[GCX_STRING] = (GCFinalizeOp)js_FinalizeString;
+        gc_finalizers[GCX_OBJECT] = (GCFinalizeOp)js_FinalizeObject;
+        gc_finalizers[GCX_STRING] = (GCFinalizeOp)js_FinalizeString;
 #ifdef DEBUG
-	gc_finalizers[GCX_DOUBLE] = (GCFinalizeOp)js_FinalizeDouble;
+        gc_finalizers[GCX_DOUBLE] = (GCFinalizeOp)js_FinalizeDouble;
 #endif
-	gc_finalizers[GCX_MUTABLE_STRING] = (GCFinalizeOp)js_FinalizeString;
+        gc_finalizers[GCX_MUTABLE_STRING] = (GCFinalizeOp)js_FinalizeString;
     }
 
     JS_InitArenaPool(&rt->gcArenaPool, "gc-arena", GC_ARENA_SIZE,
-		     sizeof(JSGCThing));
+                     sizeof(JSGCThing));
     if (!JS_DHashTableInit(&rt->gcRootsHash, JS_DHashGetStubOps(), NULL,
                            sizeof(JSGCRootHashEntry), GC_ROOTS_SIZE)) {
+        rt->gcRootsHash.ops = NULL;
         return JS_FALSE;
     }
     rt->gcLocksHash = NULL;     /* create lazily */
@@ -339,8 +346,8 @@ js_FinishGC(JSRuntime *rt)
     JS_FinishArenaPool(&rt->gcArenaPool);
     JS_ArenaFinish();
 
+    if (rt->gcRootsHash.ops) {
 #ifdef DEBUG
-    {
         uint32 leakedroots = 0;
 
         /* Warn (but don't assert) debug builds of any remaining roots. */
@@ -360,10 +367,11 @@ js_FinishGC(JSRuntime *rt)
                         (unsigned long) leakedroots);
             }
         }
-    }
 #endif
 
-    JS_DHashTableFinish(&rt->gcRootsHash);
+        JS_DHashTableFinish(&rt->gcRootsHash);
+        rt->gcRootsHash.ops = NULL;
+    }
     if (rt->gcLocksHash) {
         JS_DHashTableDestroy(rt->gcLocksHash);
         rt->gcLocksHash = NULL;
@@ -376,7 +384,7 @@ js_AddRoot(JSContext *cx, void *rp, const char *name)
 {
     JSBool ok = js_AddRootRT(cx->runtime, rp, name);
     if (!ok)
-	JS_ReportOutOfMemory(cx);
+        JS_ReportOutOfMemory(cx);
     return ok;
 }
 
@@ -470,10 +478,10 @@ js_AllocGCThing(JSContext *cx, uintN flags)
 retry:
     thing = rt->gcFreeList;
     if (thing) {
-	rt->gcFreeList = thing->next;
-	flagp = thing->flagp;
-	METER(rt->gcStats.freelen--);
-	METER(rt->gcStats.recycle++);
+        rt->gcFreeList = thing->next;
+        flagp = thing->flagp;
+        METER(rt->gcStats.freelen--);
+        METER(rt->gcStats.recycle++);
     } else {
         if (rt->gcBytes < rt->gcMaxBytes &&
             (tried_gc || rt->gcMallocBytes < rt->gcMaxBytes))
@@ -505,13 +513,21 @@ retry:
             JS_ArenaCountAllocation(pool, nb);
         }
 
-        /* Consider doing a "last ditch" GC if thing couldn't be allocated. */
+        /*
+         * Consider doing a "last ditch" GC if thing couldn't be allocated.
+         *
+         * Keep rt->gcLock across the call into js_GC so we don't starve and
+         * lose to racing threads who deplete the heap just after js_GC has
+         * replenished it (or has synchronized with a racing GC that collected
+         * a bunch of garbage).  This unfair scheduling can happen on certain
+         * operating systems.  For the gory details, see Mozilla bug 162779
+         * (http://bugzilla.mozilla.org/show_bug.cgi?id=162779).
+         */
         if (!thing) {
             if (!tried_gc) {
-                JS_UNLOCK_GC(rt);
-                js_GC(cx, GC_KEEP_ATOMS);
+                rt->gcPoke = JS_TRUE;
+                js_GC(cx, GC_KEEP_ATOMS | GC_ALREADY_LOCKED);
                 tried_gc = JS_TRUE;
-                JS_LOCK_GC(rt);
                 METER(rt->gcStats.retry++);
                 goto retry;
             }
@@ -541,18 +557,25 @@ retry:
 JSBool
 js_LockGCThing(JSContext *cx, void *thing)
 {
-    JSRuntime *rt;
+    JSBool ok = js_LockGCThingRT(cx->runtime, thing);
+    if (!ok)
+        JS_ReportOutOfMemory(cx);
+    return ok;
+}
+
+JSBool
+js_LockGCThingRT(JSRuntime *rt, void *thing)
+{
     uint8 *flagp, flags, lockbits;
     JSBool ok;
     JSGCLockHashEntry *lhe;
 
     if (!thing)
-	return JS_TRUE;
+        return JS_TRUE;
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
 
-    ok = JS_TRUE;
-    rt = cx->runtime;
+    ok = JS_FALSE;
     JS_LOCK_GC(rt);
     lockbits = (flags & GCF_LOCKMASK);
 
@@ -566,7 +589,7 @@ js_LockGCThing(JSContext *cx, void *thing)
                                          sizeof(JSGCLockHashEntry),
                                          GC_ROOTS_SIZE);
                     if (!rt->gcLocksHash)
-                        goto outofmem;
+                        goto error;
                 } else {
 #ifdef DEBUG
                     JSDHashEntryHdr *hdr = 
@@ -578,7 +601,7 @@ js_LockGCThing(JSContext *cx, void *thing)
                 lhe = (JSGCLockHashEntry *)
                     JS_DHashTableOperate(rt->gcLocksHash, thing, JS_DHASH_ADD);
                 if (!lhe)
-                    goto outofmem;
+                    goto error;
                 lhe->thing = thing;
                 lhe->count = 1;
                 *flagp = (uint8)(flags + GCF_LOCK);
@@ -597,33 +620,27 @@ js_LockGCThing(JSContext *cx, void *thing)
             *flagp = (uint8)(flags + GCF_LOCK);
         }
     } else {
-	METER(rt->gcStats.stuck++);
+        METER(rt->gcStats.stuck++);
     }
 
     METER(rt->gcStats.lock++);
-out:
+    ok = JS_TRUE;
+error:
     JS_UNLOCK_GC(rt);
     return ok;
-
-outofmem:
-    JS_ReportOutOfMemory(cx);
-    ok = JS_FALSE;
-    goto out;
 }
 
 JSBool
-js_UnlockGCThing(JSContext *cx, void *thing)
+js_UnlockGCThingRT(JSRuntime *rt, void *thing)
 {
-    JSRuntime *rt;
     uint8 *flagp, flags, lockbits;
     JSGCLockHashEntry *lhe;
 
     if (!thing)
-	return JS_TRUE;
+        return JS_TRUE;
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
 
-    rt = cx->runtime;
     JS_LOCK_GC(rt);
     lockbits = (flags & GCF_LOCKMASK);
 
@@ -647,7 +664,7 @@ js_UnlockGCThing(JSContext *cx, void *thing)
             *flagp = (uint8)(flags - GCF_LOCK);
         }
     } else {
-	METER(rt->gcStats.unstuck++);
+        METER(rt->gcStats.unstuck++);
     }
 
     rt->gcPoke = JS_TRUE;
@@ -698,18 +715,22 @@ gc_object_class_name(void* thing)
         break;
       }
 
+      case GCX_STRING:
       case GCX_MUTABLE_STRING: {
         JSString *str = (JSString *)thing;
         if (JSSTRING_IS_DEPENDENT(str)) {
             JS_snprintf(depbuf, sizeof depbuf, "start:%u, length:%u",
                         JSSTRDEP_START(str), JSSTRDEP_LENGTH(str));
             className = depbuf;
+        } else {
+            className = "string";
         }
         break;
       }
 
-      default:
-        JS_ASSERT(0);
+      case GCX_DOUBLE:
+        className = "double";
+        break;
     }
 
     return className;
@@ -744,7 +765,7 @@ gc_dump_thing(JSGCThing *thing, uint8 flags, GCMarkNode *prev, FILE *fp)
                                   ? NULL
                                   : JSVAL_TO_PRIVATE(privateValue);
         const char *className = gc_object_class_name(thing);
-        fprintf(fp, "object %08p %s", privateThing, className);
+        fprintf(fp, "object %8p %s", privateThing, className);
         break;
       }
       case GCX_DOUBLE:
@@ -774,21 +795,21 @@ js_MarkAtom(JSContext *cx, JSAtom *atom, void *arg)
     jsval key;
 
     if (atom->flags & ATOM_MARK)
-	return;
+        return;
     atom->flags |= ATOM_MARK;
     key = ATOM_KEY(atom);
     if (JSVAL_IS_GCTHING(key)) {
 #ifdef GC_MARK_DEBUG
-	char name[32];
+        char name[32];
 
-	if (JSVAL_IS_STRING(key)) {
-	    JS_snprintf(name, sizeof name, "'%s'",
-			JS_GetStringBytes(JSVAL_TO_STRING(key)));
-	} else {
-	    JS_snprintf(name, sizeof name, "<%x>", key);
-	}
+        if (JSVAL_IS_STRING(key)) {
+            JS_snprintf(name, sizeof name, "'%s'",
+                        JS_GetStringBytes(JSVAL_TO_STRING(key)));
+        } else {
+            JS_snprintf(name, sizeof name, "<%x>", key);
+        }
 #endif
-	GC_MARK(cx, JSVAL_TO_GCTHING(key), name, arg);
+        GC_MARK(cx, JSVAL_TO_GCTHING(key), name, arg);
     }
 }
 
@@ -807,7 +828,7 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 #endif
 
     if (!thing)
-	return;
+        return;
 
     flagp = js_GetGCThingFlags(thing);
     flags = *flagp;
@@ -818,12 +839,12 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 #endif
 
     if (flags & GCF_MARK)
-	return;
+        return;
 
     *flagp |= GCF_MARK;
     rt = cx->runtime;
     METER(if (++rt->gcStats.depth > rt->gcStats.maxdepth)
-	      rt->gcStats.maxdepth = rt->gcStats.depth);
+              rt->gcStats.maxdepth = rt->gcStats.depth);
 
 #ifdef GC_MARK_DEBUG
     if (js_DumpGCHeap)
@@ -832,8 +853,8 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
 
     switch (flags & GCF_TYPEMASK) {
       case GCX_OBJECT:
-	obj = (JSObject *) thing;
-	vp = obj->slots;
+        obj = (JSObject *) thing;
+        vp = obj->slots;
         if (!vp) {
             /* If obj->slots is null, obj must be a newborn. */
             JS_ASSERT(!obj->map);
@@ -856,7 +877,8 @@ js_MarkGCThing(JSContext *cx, void *thing, void *arg)
                     jsval nval;
 
                     slot = vp - obj->slots;
-                    for (sprop = scope->lastProp; ; sprop = sprop->parent) {
+                    for (sprop = SCOPE_LAST_PROP(scope); ;
+                         sprop = sprop->parent) {
                         if (!sprop) {
                             switch (slot) {
                               case JSSLOT_PROTO:
@@ -931,11 +953,11 @@ gc_root_marker(JSDHashTable *table, JSDHashEntryHdr *hdr, uint32 num, void *arg)
         JSArena *a;
         jsuword firstpage;
         JSBool root_points_to_gcArenaPool = JS_FALSE;
-	void *thing = JSVAL_TO_GCTHING(v);
+        void *thing = JSVAL_TO_GCTHING(v);
 
         for (a = cx->runtime->gcArenaPool.first.next; a; a = a->next) {
             firstpage = FIRST_THING_PAGE(a);
-	    if (JS_UPTRDIFF(thing, firstpage) < a->avail - firstpage) {
+            if (JS_UPTRDIFF(thing, firstpage) < a->avail - firstpage) {
                 root_points_to_gcArenaPool = JS_TRUE;
                 break;
             }
@@ -966,15 +988,16 @@ gc_lock_marker(JSDHashTable *table, JSDHashEntryHdr *hdr, uint32 num, void *arg)
     return JS_DHASH_NEXT;
 }
 
-JS_FRIEND_API(void)
-js_ForceGC(JSContext *cx)
+void
+js_ForceGC(JSContext *cx, uintN gcflags)
 {
     uintN i;
 
     for (i = 0; i < GCX_NTYPES; i++)
         cx->newborn[i] = NULL;
+    cx->lastAtom = NULL;
     cx->runtime->gcPoke = JS_TRUE;
-    js_GC(cx, 0);
+    js_GC(cx, gcflags);
     JS_ArenaFinish();
 }
 
@@ -995,7 +1018,7 @@ js_GC(JSContext *cx, uintN gcflags)
     JSRuntime *rt;
     JSContext *iter, *acx;
     JSStackFrame *fp, *chain;
-    uintN i, depth, nslots;
+    uintN i, depth, nslots, type;
     JSStackHeader *sh;
     JSArena *a, **ap;
     uint8 flags, *flagp, *split;
@@ -1013,13 +1036,18 @@ js_GC(JSContext *cx, uintN gcflags)
     JS_ASSERT(!JS_IS_RUNTIME_LOCKED(rt));
 #endif
 
-    /* Don't run gc if it is disabled (unless this is the last context). */
-    if (rt->gcDisabled && !(gcflags & GC_LAST_CONTEXT))
+    /*
+     * Don't collect garbage if the runtime isn't up, and cx is not the last
+     * context in the runtime.  The last context must force a GC, and nothing
+     * should suppress that final collection or there may be shutdown leaks,
+     * or runtime bloat until the next context is created.
+     */
+    if (rt->state != JSRTS_UP && !(gcflags & GC_LAST_CONTEXT))
         return;
 
     /*
      * Let the API user decide to defer a GC if it wants to (unless this
-     * is the last context). Call the callback regardless.
+     * is the last context).  Invoke the callback regardless.
      */
     if (rt->gcCallback) {
         if (!rt->gcCallback(cx, JSGC_BEGIN) && !(gcflags & GC_LAST_CONTEXT))
@@ -1027,26 +1055,28 @@ js_GC(JSContext *cx, uintN gcflags)
     }
 
     /* Lock out other GC allocator and collector invocations. */
-    JS_LOCK_GC(rt);
+    if (!(gcflags & GC_ALREADY_LOCKED))
+        JS_LOCK_GC(rt);
 
     /* Do nothing if no assignment has executed since the last GC. */
     if (!rt->gcPoke) {
-	METER(rt->gcStats.nopoke++);
-	JS_UNLOCK_GC(rt);
-	return;
+        METER(rt->gcStats.nopoke++);
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
+        return;
     }
-    rt->gcPoke = JS_FALSE;
     METER(rt->gcStats.poke++);
 
 #ifdef JS_THREADSAFE
     /* Bump gcLevel and return rather than nest on this thread. */
     currentThread = js_CurrentThreadId();
     if (rt->gcThread == currentThread) {
-	JS_ASSERT(rt->gcLevel > 0);
-	rt->gcLevel++;
-	METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-		  rt->gcStats.maxlevel = rt->gcLevel);
-        JS_UNLOCK_GC(rt);
+        JS_ASSERT(rt->gcLevel > 0);
+        rt->gcLevel++;
+        METER(if (rt->gcLevel > rt->gcStats.maxlevel)
+                  rt->gcStats.maxlevel = rt->gcLevel);
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
         return;
     }
 
@@ -1063,7 +1093,7 @@ js_GC(JSContext *cx, uintN gcflags)
          * keep a sub-list of contexts having the same id?
          */
         iter = NULL;
-        while ((acx = js_ContextIterator(rt, &iter)) != NULL) {
+        while ((acx = js_ContextIterator(rt, JS_FALSE, &iter)) != NULL) {
             if (acx->thread == cx->thread && acx->requestDepth)
                 requestDebit++;
         }
@@ -1089,17 +1119,18 @@ js_GC(JSContext *cx, uintN gcflags)
     /* If another thread is already in GC, don't attempt GC; wait instead. */
     if (rt->gcLevel > 0) {
         /* Bump gcLevel to restart the current GC, so it finds new garbage. */
-	rt->gcLevel++;
-	METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-		  rt->gcStats.maxlevel = rt->gcLevel);
+        rt->gcLevel++;
+        METER(if (rt->gcLevel > rt->gcStats.maxlevel)
+                  rt->gcStats.maxlevel = rt->gcLevel);
 
         /* Wait for the other thread to finish, then resume our request. */
         while (rt->gcLevel > 0)
             JS_AWAIT_GC_DONE(rt);
         if (requestDebit)
             rt->requestCount += requestDebit;
-	JS_UNLOCK_GC(rt);
-	return;
+        if (!(gcflags & GC_ALREADY_LOCKED))
+            JS_UNLOCK_GC(rt);
+        return;
     }
 
     /* No other thread is in GC, so indicate that we're now in GC. */
@@ -1108,16 +1139,16 @@ js_GC(JSContext *cx, uintN gcflags)
 
     /* Wait for all other requests to finish. */
     while (rt->requestCount > 0)
-	JS_AWAIT_REQUEST_DONE(rt);
+        JS_AWAIT_REQUEST_DONE(rt);
 
 #else  /* !JS_THREADSAFE */
 
     /* Bump gcLevel and return rather than nest; the outer gc will restart. */
     rt->gcLevel++;
     METER(if (rt->gcLevel > rt->gcStats.maxlevel)
-	      rt->gcStats.maxlevel = rt->gcLevel);
+              rt->gcStats.maxlevel = rt->gcLevel);
     if (rt->gcLevel > 1)
-	return;
+        return;
 
 #endif /* !JS_THREADSAFE */
 
@@ -1132,7 +1163,11 @@ js_GC(JSContext *cx, uintN gcflags)
     rt->gcRunning = JS_TRUE;
     JS_UNLOCK_GC(rt);
 
-    /* Reset malloc counter */
+    /* If a suspended compile is running on another context, keep atoms. */
+    if (rt->gcKeepAtoms)
+        gcflags |= GC_KEEP_ATOMS;
+
+    /* Reset malloc counter. */
     rt->gcMallocBytes = 0;
 
     /* Drop atoms held by the property cache, and clear property weak links. */
@@ -1154,23 +1189,24 @@ restart:
     if (rt->gcLocksHash)
         JS_DHashTableEnumerate(rt->gcLocksHash, gc_lock_marker, cx);
     js_MarkAtomState(&rt->atomState, gcflags, gc_mark_atom_key_thing, cx);
+    js_MarkWatchPoints(rt);
     iter = NULL;
-    while ((acx = js_ContextIterator(rt, &iter)) != NULL) {
-	/*
-	 * Iterate frame chain and dormant chains. Temporarily tack current
-	 * frame onto the head of the dormant list to ease iteration.
-	 *
-	 * (NB: see comment on this whole "dormant" thing in js_Execute.)
-	 */
-	chain = acx->fp;
-	if (chain) {
-	    JS_ASSERT(!chain->dormantNext);
-	    chain->dormantNext = acx->dormantFrameChain;
-	} else {
-	    chain = acx->dormantFrameChain;
-	}
+    while ((acx = js_ContextIterator(rt, JS_TRUE, &iter)) != NULL) {
+        /*
+         * Iterate frame chain and dormant chains. Temporarily tack current
+         * frame onto the head of the dormant list to ease iteration.
+         *
+         * (NB: see comment on this whole "dormant" thing in js_Execute.)
+         */
+        chain = acx->fp;
+        if (chain) {
+            JS_ASSERT(!chain->dormantNext);
+            chain->dormantNext = acx->dormantFrameChain;
+        } else {
+            chain = acx->dormantFrameChain;
+        }
 
-	for (fp = chain; fp; fp = chain = chain->dormantNext) {
+        for (fp = chain; fp; fp = chain = chain->dormantNext) {
             do {
                 if (fp->callobj)
                     GC_MARK(cx, fp->callobj, "call object", NULL);
@@ -1207,25 +1243,39 @@ restart:
                 GC_MARK(cx, fp->scopeChain, "scope chain", NULL);
                 if (fp->sharpArray)
                     GC_MARK(cx, fp->sharpArray, "sharp array", NULL);
-            } while ((fp = fp->down) != NULL);
-	}
 
-	/* Cleanup temporary "dormant" linkage. */
-	if (acx->fp)
-	    acx->fp->dormantNext = NULL;
+                if (fp->objAtomMap) {
+                    JSAtom **vector, *atom;
+
+                    nslots = fp->objAtomMap->length;
+                    vector = fp->objAtomMap->vector;
+                    for (i = 0; i < nslots; i++) {
+                        atom = vector[i];
+                        if (atom)
+                            GC_MARK_ATOM(cx, atom, NULL);
+                    }
+                }
+            } while ((fp = fp->down) != NULL);
+        }
+
+        /* Cleanup temporary "dormant" linkage. */
+        if (acx->fp)
+            acx->fp->dormantNext = NULL;
 
         /* Mark other roots-by-definition in acx. */
-	GC_MARK(cx, acx->globalObject, "global object", NULL);
-	GC_MARK(cx, acx->newborn[GCX_OBJECT], "newborn object", NULL);
-	GC_MARK(cx, acx->newborn[GCX_STRING], "newborn string", NULL);
-	GC_MARK(cx, acx->newborn[GCX_DOUBLE], "newborn double", NULL);
+        GC_MARK(cx, acx->globalObject, "global object", NULL);
+        GC_MARK(cx, acx->newborn[GCX_OBJECT], "newborn object", NULL);
+        GC_MARK(cx, acx->newborn[GCX_STRING], "newborn string", NULL);
+        GC_MARK(cx, acx->newborn[GCX_DOUBLE], "newborn double", NULL);
         GC_MARK(cx, acx->newborn[GCX_MUTABLE_STRING], "newborn mutable string",
                 NULL);
-	for (i = GCX_EXTERNAL_STRING; i < GCX_NTYPES; i++)
+        for (i = GCX_EXTERNAL_STRING; i < GCX_NTYPES; i++)
             GC_MARK(cx, acx->newborn[i], "newborn external string", NULL);
+        if (acx->lastAtom)
+            GC_MARK_ATOM(cx, acx->lastAtom, NULL);
 #if JS_HAS_EXCEPTIONS
-	if (acx->throwing && JSVAL_IS_GCTHING(acx->exception))
-	    GC_MARK(cx, JSVAL_TO_GCTHING(acx->exception), "exception", NULL);
+        if (acx->throwing && JSVAL_IS_GCTHING(acx->exception))
+            GC_MARK(cx, JSVAL_TO_GCTHING(acx->exception), "exception", NULL);
 #endif
 #if JS_HAS_LVALUE_RETURN
         if (acx->rval2set && JSVAL_IS_GCTHING(acx->rval2))
@@ -1250,6 +1300,7 @@ restart:
      */
     js_SweepAtomState(&rt->atomState);
     js_SweepScopeProperties(rt);
+    js_SweepScriptFilenames(rt);
     for (a = rt->gcArenaPool.first.next; a; a = a->next) {
         flagp = (uint8 *) a->base;
         split = (uint8 *) FIRST_THING_PAGE(a);
@@ -1264,9 +1315,12 @@ restart:
                 *flagp &= ~GCF_MARK;
             } else if (!(flags & (GCF_LOCKMASK | GCF_FINAL))) {
                 /* Call the finalizer with GCF_FINAL ORed into flags. */
-                finalizer = gc_finalizers[flags & GCF_TYPEMASK];
+                type = flags & GCF_TYPEMASK;
+                finalizer = gc_finalizers[type];
                 if (finalizer) {
                     *flagp = (uint8)(flags | GCF_FINAL);
+                    if (type >= GCX_EXTERNAL_STRING)
+                        js_PurgeDeflatedStringCache((JSString *)thing);
                     finalizer(cx, thing);
                 }
 
@@ -1288,7 +1342,7 @@ restart:
     ap = &rt->gcArenaPool.first.next;
     a = *ap;
     if (!a)
-	goto out;
+        goto out;
     all_clear = JS_TRUE;
     flp = oflp = &rt->gcFreeList;
     *flp = NULL;
@@ -1331,6 +1385,11 @@ restart:
 
     if (rt->gcCallback)
         (void) rt->gcCallback(cx, JSGC_FINALIZE_END);
+#ifdef DEBUG_brendan
+  { extern void DumpSrcNoteSizeHist();
+    DumpSrcNoteSizeHist();
+  }
+#endif
 
 out:
     JS_LOCK_GC(rt);
@@ -1342,17 +1401,23 @@ out:
     js_EnablePropertyCache(cx);
     rt->gcLevel = 0;
     rt->gcLastBytes = rt->gcBytes;
-    rt->gcRunning = JS_FALSE;
+    rt->gcPoke = rt->gcRunning = JS_FALSE;
 
 #ifdef JS_THREADSAFE
     /* If we were invoked during a request, pay back the temporary debit. */
     if (requestDebit)
-	rt->requestCount += requestDebit;
+        rt->requestCount += requestDebit;
     rt->gcThread = 0;
     JS_NOTIFY_GC_DONE(rt);
-    JS_UNLOCK_GC(rt);
+    if (!(gcflags & GC_ALREADY_LOCKED))
+        JS_UNLOCK_GC(rt);
 #endif
 
-    if (rt->gcCallback)
+    if (rt->gcCallback) {
+        if (gcflags & GC_ALREADY_LOCKED)
+            JS_UNLOCK_GC(rt);
         (void) rt->gcCallback(cx, JSGC_END);
+        if (gcflags & GC_ALREADY_LOCKED)
+            JS_LOCK_GC(rt);
+    }
 }
