@@ -47,7 +47,7 @@
 #include "nsIFontMetrics.h"
 #include "nsHTMLAtoms.h"
 #include "nsXULAtoms.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsCOMPtr.h"
 #include "nsIPresContext.h"
@@ -60,14 +60,11 @@
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
 #include "nsIPresShell.h"
-#include "nsHTMLIIDs.h"
 #include "nsIImage.h"
 #include "nsIWidget.h"
 #include "nsHTMLAtoms.h"
-#include "nsIHTMLAttributes.h"
 #include "nsIDocument.h"
 #include "nsIHTMLDocument.h"
-#include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsImageMap.h"
 #include "nsILinkHandler.h"
@@ -77,7 +74,6 @@
 #include "nsIViewManager.h"
 #include "nsHTMLContainerFrame.h"
 #include "prprf.h"
-#include "nsISizeOfHandler.h"
 #include "nsIFontMetrics.h"
 #include "nsCSSRendering.h"
 #include "nsIDOMHTMLImageElement.h"
@@ -85,8 +81,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsTextFragment.h"
 #include "nsIDOMHTMLMapElement.h"
-#include "nsIStyleSet.h"
-#include "nsIStyleContext.h"
 #include "nsBoxLayoutState.h"
 #include "nsIDOMDocument.h"
 #include "nsIEventQueueService.h"
@@ -122,9 +116,7 @@ HandleImagePLEvent(nsIContent *aContent, PRUint32 aMessage, PRUint32 aFlags)
     return;
   }
 
-  nsCOMPtr<nsIPresShell> pres_shell;
-  doc->GetShellAt(0, getter_AddRefs(pres_shell));
-
+  nsIPresShell *pres_shell = doc->GetShellAt(0);
   if (!pres_shell) {
     return;
   }
@@ -137,9 +129,7 @@ HandleImagePLEvent(nsIContent *aContent, PRUint32 aMessage, PRUint32 aFlags)
   }
 
   nsEventStatus status = nsEventStatus_eIgnore;
-  nsEvent event;
-  event.eventStructType = NS_EVENT;
-  event.message = aMessage;
+  nsEvent event(aMessage);
 
   aContent->HandleDOMEvent(pres_context, &event, nsnull, aFlags, &status);
 }
@@ -266,33 +256,29 @@ nsImageBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
                                nsIContent* aChild,
                                PRInt32 aNameSpaceID,
                                nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                               PRInt32 aModType)
 {
-  nsresult rv = nsLeafBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType, aHint);
+  nsresult rv = nsLeafBoxFrame::AttributeChanged(aPresContext, aChild, aNameSpaceID, aAttribute, aModType);
 
-  PRBool aResize;
-  PRBool aRedraw;
-  UpdateAttributes(aPresContext, aAttribute, aResize, aRedraw);
-
-  nsBoxLayoutState state(aPresContext);
+  PRBool aResize = UpdateAttributes(aAttribute);
 
   if (aResize) {
+    nsBoxLayoutState state(aPresContext);
     MarkDirty(state);
-  } else if (aRedraw) {
-    Redraw(state);
   }
 
-
-  return NS_OK;
+  return rv;
 }
 
-nsImageBoxFrame::nsImageBoxFrame(nsIPresShell* aShell):nsLeafBoxFrame(aShell), mIntrinsicSize(0,0),
-  mLoadFlags(nsIRequest::LOAD_NORMAL), mSuppressStyleCheck(PR_FALSE)
+nsImageBoxFrame::nsImageBoxFrame(nsIPresShell* aShell) :
+  nsLeafBoxFrame(aShell),
+  mUseSrcAttr(PR_FALSE),
+  mSizeFrozen(PR_FALSE),
+  mHasImage(PR_FALSE),
+  mSuppressStyleCheck(PR_FALSE),
+  mIntrinsicSize(0,0),
+  mLoadFlags(nsIRequest::LOAD_NORMAL)
 {
-  mSizeFrozen = PR_FALSE;
-  mHasImage = PR_FALSE;
-  mUseSrcAttr = PR_FALSE;
   NeedsRecalc();
 }
 
@@ -326,7 +312,7 @@ NS_IMETHODIMP
 nsImageBoxFrame::Init(nsIPresContext*  aPresContext,
                           nsIContent*      aContent,
                           nsIFrame*        aParent,
-                          nsIStyleContext* aContext,
+                          nsStyleContext*  aContext,
                           nsIFrame*        aPrevInFlow)
 {
   if (!mListener) {
@@ -345,8 +331,7 @@ nsImageBoxFrame::Init(nsIPresContext*  aPresContext,
   GetImageSource();
   UpdateLoadFlags();
 
-  PRBool aResize;
-  UpdateImage(aPresContext, aResize);
+  UpdateImage();
 
   return rv;
 }
@@ -355,44 +340,40 @@ void
 nsImageBoxFrame::GetImageSource()
 {
   // get the new image src
-  mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, mSrc);
-
-  // if the new image is empty
-  if (mSrc.IsEmpty()) {
-    mUseSrcAttr = PR_FALSE;
-
+  nsAutoString src;
+  mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::src, src);
+  mUseSrcAttr = !src.IsEmpty();
+  if (mUseSrcAttr) {
+    nsCOMPtr<nsIURI> baseURI;
+    if (mContent) {
+      baseURI = mContent->GetBaseURI();
+    }
+    // XXX origin charset needed
+    NS_NewURI(getter_AddRefs(mURI), src, nsnull, baseURI);
+  } else {
     // Only get the list-style-image if we aren't being drawn
     // by a native theme.
-    const nsStyleDisplay* disp =
-      (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
+    const nsStyleDisplay* disp = GetStyleDisplay();
     if (disp->mAppearance && nsBox::gTheme && 
-        nsBox::gTheme->ThemeSupportsWidget(nsnull, disp->mAppearance))
+        nsBox::gTheme->ThemeSupportsWidget(nsnull, this, disp->mAppearance))
       return;
 
     // get the list-style-image
-    const nsStyleList* myList =
-      (const nsStyleList*)mStyleContext->GetStyleData(eStyleStruct_List);
-  
-    if (myList->mListStyleImage.Length() > 0) {
-      mSrc = myList->mListStyleImage;
-    }
+    mURI = GetStyleList()->mListStyleImage;
   }
-  else
-    mUseSrcAttr = PR_TRUE;
 }
 
-void
-nsImageBoxFrame::UpdateAttributes(nsIPresContext*  aPresContext, nsIAtom* aAttribute, PRBool& aResize, PRBool& aRedraw)
+PRBool
+nsImageBoxFrame::UpdateAttributes(nsIAtom* aAttribute)
 {
-  aResize = PR_FALSE;
-  aRedraw = PR_FALSE;
-
   if (aAttribute == nsnull || aAttribute == nsHTMLAtoms::src) {
     GetImageSource();
-    UpdateImage(aPresContext, aResize);
+    return UpdateImage();
   }
   else if (aAttribute == nsXULAtoms::validate)
     UpdateLoadFlags();
+
+  return PR_FALSE;
 }
 
 void
@@ -400,46 +381,41 @@ nsImageBoxFrame::UpdateLoadFlags()
 {
   nsAutoString loadPolicy;
   mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::validate, loadPolicy);
-  if (loadPolicy.EqualsIgnoreCase("always"))
+  if (loadPolicy.Equals(NS_LITERAL_STRING("always")))
     mLoadFlags = nsIRequest::VALIDATE_ALWAYS;
-  else if (loadPolicy.EqualsIgnoreCase("never"))
+  else if (loadPolicy.Equals(NS_LITERAL_STRING("never")))
     mLoadFlags = nsIRequest::VALIDATE_NEVER|nsIRequest::LOAD_FROM_CACHE; 
   else
     mLoadFlags = nsIRequest::LOAD_NORMAL;
 }
 
-void
-nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
+PRBool
+nsImageBoxFrame::UpdateImage()
 {
-  aResize = PR_FALSE;
-
   // get the new image src
-  if (mSrc.IsEmpty()) {
+  if (!mURI) {
     mSizeFrozen = PR_TRUE;
     mHasImage = PR_FALSE;
-    aResize = PR_TRUE;
 
-    mImageRequest = nsnull;
-    
-    return;
+    if (mImageRequest) {
+      mImageRequest->Cancel(NS_ERROR_FAILURE);
+      mImageRequest = nsnull;
+    }
+
+    return PR_TRUE;
   }
 
-  nsCOMPtr<nsIURI> baseURI;
-  GetBaseURI(getter_AddRefs(baseURI));
-  nsCOMPtr<nsIURI> srcURI;
-  NS_NewURI(getter_AddRefs(srcURI), mSrc, nsnull, baseURI);
-
+  nsresult rv;
   if (mImageRequest) {
     nsCOMPtr<nsIURI> requestURI;
-    nsresult rv = mImageRequest->GetURI(getter_AddRefs(requestURI));
+    rv = mImageRequest->GetURI(getter_AddRefs(requestURI));
     NS_ASSERTION(NS_SUCCEEDED(rv) && requestURI,"no request URI");
-    if (NS_FAILED(rv) || !requestURI) return;
+    if (NS_FAILED(rv) || !requestURI) return PR_FALSE;
 
     PRBool eq;
-    requestURI->Equals(srcURI, &eq);
     // if the source uri and the current one are the same, return
-    if (eq)
-      return;
+    if (NS_SUCCEEDED(requestURI->Equals(mURI, &eq)) && eq)
+      return PR_FALSE;
   }
 
   mSizeFrozen = PR_FALSE;
@@ -451,15 +427,26 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
     mImageRequest = nsnull;
   }
 
-  nsresult rv;
   nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1", &rv));
+  if (NS_FAILED(rv)) return PR_FALSE;
 
-  nsCOMPtr<nsILoadGroup> loadGroup;
-  GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
+  nsCOMPtr<nsILoadGroup> loadGroup = GetLoadGroup();
 
-  il->LoadImage(srcURI, loadGroup, mListener, aPresContext, mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
+  // Get the document URI for the referrer...
+  nsIURI *documentURI = nsnull;
+  nsCOMPtr<nsIDocument> doc;
+  if (mContent) {
+    doc = mContent->GetDocument();
+    if (doc) {
+      documentURI = doc->GetDocumentURI();
+    }
+  }
 
-  aResize = PR_TRUE;
+  // XXX: initialDocumentURI is NULL!
+  il->LoadImage(mURI, nsnull, documentURI, loadGroup, mListener, doc,
+                mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
+
+  return PR_TRUE;
 }
 
 NS_IMETHODIMP
@@ -469,49 +456,47 @@ nsImageBoxFrame::Paint(nsIPresContext*      aPresContext,
                        nsFramePaintLayer    aWhichLayer,
                        PRUint32             aFlags)
 {	
-	const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-	if (!vis->IsVisible())
-		return NS_OK;
+  if (!GetStyleVisibility()->IsVisible())
+    return NS_OK;
 
   nsresult rv = nsLeafBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 
-  PaintImage(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  PaintImage(aRenderingContext, aDirtyRect, aWhichLayer);
 
   return rv;
 }
 
 
-NS_IMETHODIMP
-nsImageBoxFrame::PaintImage(nsIPresContext* aPresContext,
-                                nsIRenderingContext& aRenderingContext,
-                                const nsRect& aDirtyRect,
-                                nsFramePaintLayer aWhichLayer)
+void
+nsImageBoxFrame::PaintImage(nsIRenderingContext& aRenderingContext,
+                            const nsRect& aDirtyRect,
+                            nsFramePaintLayer aWhichLayer)
 {
   if ((0 == mRect.width) || (0 == mRect.height)) {
     // Do not render when given a zero area. This avoids some useless
     // scaling work while we wait for our image dimensions to arrive
     // asynchronously.
-    return NS_OK;
+    return;
   }
 
   nsRect rect;
   GetClientRect(rect);
 
+  if (NS_FRAME_PAINT_LAYER_FOREGROUND != aWhichLayer)
+    return;
+
+  if (!mImageRequest)
+    return;
+
   // don't draw if the image is not dirty
   if (!mHasImage || !aDirtyRect.Intersects(rect))
-    return NS_OK;
-
-  if (NS_FRAME_PAINT_LAYER_FOREGROUND != aWhichLayer)
-    return NS_OK;
-
-  if (!mImageRequest) return NS_ERROR_UNEXPECTED;
+    return;
 
   nsCOMPtr<imgIContainer> imgCon;
   mImageRequest->GetImage(getter_AddRefs(imgCon));
 
   if (imgCon) {
-    PRBool hasSubRect = mSubRect.width > 0 || mSubRect.height > 0;
+    PRBool hasSubRect = !mUseSrcAttr && (mSubRect.width > 0 || mSubRect.height > 0);
     PRBool sizeMatch = hasSubRect ? 
                        mSubRect.width == rect.width && mSubRect.height == rect.height :
                        mImageSize.width == rect.width && mImageSize.height == rect.height;
@@ -537,8 +522,6 @@ nsImageBoxFrame::PaintImage(nsIPresContext* aPresContext,
       aRenderingContext.DrawScaledImage(imgCon, &src, &rect);
     }
   }
-
-  return NS_OK;
 }
 
 
@@ -551,40 +534,39 @@ NS_IMETHODIMP
 nsImageBoxFrame::DidSetStyleContext( nsIPresContext* aPresContext )
 {
   // Fetch our subrect.
-  const nsStyleList* myList =
-    (const nsStyleList*)mStyleContext->GetStyleData(eStyleStruct_List);
+  const nsStyleList* myList = GetStyleList();
   mSubRect = myList->mImageRegion;
 
   if (mUseSrcAttr || mSuppressStyleCheck)
     return NS_OK; // No more work required, since the image isn't specified by style.
 
   // If we're using a native theme implementation, we shouldn't draw anything.
-  const nsStyleDisplay* disp =
-    (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
+  const nsStyleDisplay* disp = GetStyleDisplay();
   if (disp->mAppearance && nsBox::gTheme && 
-      nsBox::gTheme->ThemeSupportsWidget(nsnull, disp->mAppearance))
+      nsBox::gTheme->ThemeSupportsWidget(nsnull, this, disp->mAppearance))
     return NS_OK;
 
   // If list-style-image changes, we have a new image.
-  nsAutoString newSrc;
-  if (myList->mListStyleImage.Equals(mSrc))
+  nsIURI *newURI = myList->mListStyleImage;
+  PRBool equal;
+  if (newURI == mURI ||   // handles null==null
+      (newURI && mURI && NS_SUCCEEDED(newURI->Equals(mURI, &equal)) && equal))
     return NS_OK;
 
-  mSrc = myList->mListStyleImage;
+  mURI = newURI;
 
-  PRBool aResize;
-  UpdateImage(aPresContext, aResize);
+  UpdateImage();
   return NS_OK;
 } // DidSetStyleContext
 
 void
-nsImageBoxFrame::GetImageSize(nsIPresContext* aPresContext)
+nsImageBoxFrame::GetImageSize()
 {
-  nsSize s(0,0);
-  nsHTMLReflowMetrics desiredSize(&s);
+  nsHTMLReflowMetrics desiredSize(PR_TRUE);
   const PRInt32 kDefaultSize = 0;
   float p2t;
-  aPresContext->GetScaledPixelsToTwips(&p2t);
+  GetPresContext()->GetScaledPixelsToTwips(&p2t);
+  // XXX constant zero?
   const PRInt32 kDefaultSizeInTwips = NSIntPixelsToTwips(kDefaultSize, p2t);
 
 // not calculated? Get the intrinsic size
@@ -608,20 +590,12 @@ nsImageBoxFrame::GetImageSize(nsIPresContext* aPresContext)
 	  }
 	}
 
+  // XXX constant zero?
   mImageSize.width = desiredSize.width;
   mImageSize.height = desiredSize.height;
 }
 
 
-/**
- * Ok return our dimensions
- */
-NS_IMETHODIMP
-nsImageBoxFrame::DoLayout(nsBoxLayoutState& aState)
-{
-  return nsLeafBoxFrame::DoLayout(aState);
-}
-
 /**
  * Ok return our dimensions
  */
@@ -629,10 +603,10 @@ NS_IMETHODIMP
 nsImageBoxFrame::GetPrefSize(nsBoxLayoutState& aState, nsSize& aSize)
 {
   if (DoesNeedRecalc(mImageSize)) {
-     CacheImageSize(aState);
+     GetImageSize();
   }
 
-  if (mSubRect.width > 0 || mSubRect.height > 0)
+  if (!mUseSrcAttr && (mSubRect.width > 0 || mSubRect.height > 0))
     aSize = nsSize(mSubRect.width, mSubRect.height);
   else
     aSize = mImageSize;
@@ -670,16 +644,6 @@ nsImageBoxFrame::GetAscent(nsBoxLayoutState& aState, nscoord& aCoord)
   return NS_OK;
 }
 
-/**
- * Ok return our dimensions
- */
-void
-nsImageBoxFrame::CacheImageSize(nsBoxLayoutState& aState)
-{
-  nsIPresContext* presContext = aState.GetPresContext();
-  GetImageSize(presContext);
-}
-
 #ifdef DEBUG
 NS_IMETHODIMP
 nsImageBoxFrame::GetFrameName(nsAString& aResult) const
@@ -689,56 +653,29 @@ nsImageBoxFrame::GetFrameName(nsAString& aResult) const
 #endif
 
 
-void
-nsImageBoxFrame::GetBaseURI(nsIURI **uri)
+already_AddRefed<nsILoadGroup>
+nsImageBoxFrame::GetLoadGroup()
 {
-  nsresult rv;
-  nsCOMPtr<nsIURI> baseURI;
-  nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(mContent, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    htmlContent->GetBaseURL(*getter_AddRefs(baseURI));
-  }
-  else {
-    nsCOMPtr<nsIDocument> doc;
-    mContent->GetDocument(*getter_AddRefs(doc));
-    if (doc) {
-      doc->GetBaseURL(*getter_AddRefs(baseURI));
-    }
-  }
-  *uri = baseURI;
-  NS_IF_ADDREF(*uri);
-}
-
-void
-nsImageBoxFrame::GetLoadGroup(nsIPresContext *aPresContext, nsILoadGroup **aLoadGroup)
-{
-  nsCOMPtr<nsIPresShell> shell;
-  aPresContext->GetShell(getter_AddRefs(shell));
-
+  nsIPresShell *shell = GetPresContext()->GetPresShell();
   if (!shell)
-    return;
+    return nsnull;
 
   nsCOMPtr<nsIDocument> doc;
   shell->GetDocument(getter_AddRefs(doc));
   if (!doc)
-    return;
+    return nsnull;
 
-  doc->GetDocumentLoadGroup(aLoadGroup);
+  return doc->GetDocumentLoadGroup(); // already_AddRefed
 }
 
 
-NS_IMETHODIMP nsImageBoxFrame::OnStartDecode(imgIRequest *request, nsIPresContext *aPresContext)
+NS_IMETHODIMP nsImageBoxFrame::OnStartContainer(imgIRequest *request,
+                                                imgIContainer *image)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
+  NS_ENSURE_ARG_POINTER(image);
 
-NS_IMETHODIMP nsImageBoxFrame::OnStartContainer(imgIRequest *request, nsIPresContext *aPresContext, imgIContainer *image)
-{
-#ifdef DEBUG_pavlov
-  NS_ENSURE_ARG(image);
-#else
-  if (!image) return NS_ERROR_INVALID_ARG; 
-#endif  
+  // Ensure the animation (if any) is started
+  image->StartAnimation();
 
   mHasImage = PR_TRUE;
   mSizeFrozen = PR_FALSE;
@@ -747,42 +684,29 @@ NS_IMETHODIMP nsImageBoxFrame::OnStartContainer(imgIRequest *request, nsIPresCon
   image->GetWidth(&w);
   image->GetHeight(&h);
 
-  float p2t;
-  aPresContext->GetPixelsToTwips(&p2t);
+  nsIPresContext* presContext = GetPresContext();
+  float p2t = presContext->PixelsToTwips();
 
   mIntrinsicSize.SizeTo(NSIntPixelsToTwips(w, p2t), NSIntPixelsToTwips(h, p2t));
 
-  nsBoxLayoutState state(aPresContext);
-
+  nsBoxLayoutState state(presContext);
   this->MarkDirty(state);
 
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImageBoxFrame::OnStartFrame(imgIRequest *request, nsIPresContext *aPresContext, gfxIImageFrame *frame)
+NS_IMETHODIMP nsImageBoxFrame::OnStopContainer(imgIRequest *request,
+                                               imgIContainer *image)
 {
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsImageBoxFrame::OnDataAvailable(imgIRequest *request, nsIPresContext *aPresContext, gfxIImageFrame *frame, const nsRect * rect)
-{
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsImageBoxFrame::OnStopFrame(imgIRequest *request, nsIPresContext *aPresContext, gfxIImageFrame *frame)
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsImageBoxFrame::OnStopContainer(imgIRequest *request, nsIPresContext *aPresContext, imgIContainer *image)
-{
-  nsBoxLayoutState state(aPresContext);
+  nsBoxLayoutState state(GetPresContext());
   this->Redraw(state);
 
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImageBoxFrame::OnStopDecode(imgIRequest *request, nsIPresContext *aPresContext, nsresult aStatus, const PRUnichar *statusArg)
+NS_IMETHODIMP nsImageBoxFrame::OnStopDecode(imgIRequest *request,
+                                            nsresult aStatus,
+                                            const PRUnichar *statusArg)
 {
   if (NS_SUCCEEDED(aStatus))
     // Fire an onerror DOM event.
@@ -793,9 +717,11 @@ NS_IMETHODIMP nsImageBoxFrame::OnStopDecode(imgIRequest *request, nsIPresContext
   return NS_OK;
 }
 
-NS_IMETHODIMP nsImageBoxFrame::FrameChanged(imgIContainer *container, nsIPresContext *aPresContext, gfxIImageFrame *newframe, nsRect * dirtyRect)
+NS_IMETHODIMP nsImageBoxFrame::FrameChanged(imgIContainer *container,
+                                            gfxIImageFrame *newframe,
+                                            nsRect * dirtyRect)
 {
-  nsBoxLayoutState state(aPresContext);
+  nsBoxLayoutState state(GetPresContext());
   this->Redraw(state);
 
   return NS_OK;
@@ -805,84 +731,71 @@ NS_IMPL_ISUPPORTS2(nsImageBoxListener, imgIDecoderObserver, imgIContainerObserve
 
 nsImageBoxListener::nsImageBoxListener()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsImageBoxListener::~nsImageBoxListener()
 {
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnStartDecode(imgIRequest *request, nsISupports *cx)
+NS_IMETHODIMP nsImageBoxListener::OnStartDecode(imgIRequest *request)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsImageBoxListener::OnStartContainer(imgIRequest *request,
+                                                   imgIContainer *image)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnStartDecode(request, pc);
+  return mFrame->OnStartContainer(request, image);
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnStartContainer(imgIRequest *request, nsISupports *cx, imgIContainer *image)
+NS_IMETHODIMP nsImageBoxListener::OnStartFrame(imgIRequest *request,
+                                               gfxIImageFrame *frame)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsImageBoxListener::OnDataAvailable(imgIRequest *request,
+                                                  gfxIImageFrame *frame,
+                                                  const nsRect * rect)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsImageBoxListener::OnStopFrame(imgIRequest *request,
+                                              gfxIImageFrame *frame)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsImageBoxListener::OnStopContainer(imgIRequest *request,
+                                                  imgIContainer *image)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnStartContainer(request, pc, image);
+  return mFrame->OnStopContainer(request, image);
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnStartFrame(imgIRequest *request, nsISupports *cx, gfxIImageFrame *frame)
+NS_IMETHODIMP nsImageBoxListener::OnStopDecode(imgIRequest *request,
+                                               nsresult status,
+                                               const PRUnichar *statusArg)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnStartFrame(request, pc, frame);
+  return mFrame->OnStopDecode(request, status, statusArg);
 }
 
-NS_IMETHODIMP nsImageBoxListener::OnDataAvailable(imgIRequest *request, nsISupports *cx, gfxIImageFrame *frame, const nsRect * rect)
+NS_IMETHODIMP nsImageBoxListener::FrameChanged(imgIContainer *container,
+                                               gfxIImageFrame *newframe,
+                                               nsRect * dirtyRect)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnDataAvailable(request, pc, frame, rect);
-}
-
-NS_IMETHODIMP nsImageBoxListener::OnStopFrame(imgIRequest *request, nsISupports *cx, gfxIImageFrame *frame)
-{
-  if (!mFrame)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnStopFrame(request, pc, frame);
-}
-
-NS_IMETHODIMP nsImageBoxListener::OnStopContainer(imgIRequest *request, nsISupports *cx, imgIContainer *image)
-{
-  if (!mFrame)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->OnStopContainer(request, pc, image);
-}
-
-NS_IMETHODIMP nsImageBoxListener::OnStopDecode(imgIRequest *request, nsISupports *cx, nsresult status, const PRUnichar *statusArg)
-{
-  if (!mFrame)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  nsresult rv = mFrame->OnStopDecode(request, pc, status, statusArg);
-
-  return rv;
-}
-
-NS_IMETHODIMP nsImageBoxListener::FrameChanged(imgIContainer *container, nsISupports *cx, gfxIImageFrame *newframe, nsRect * dirtyRect)
-{
-  if (!mFrame)
-    return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIPresContext> pc(do_QueryInterface(cx));
-  return mFrame->FrameChanged(container, pc, newframe, dirtyRect);
+  return mFrame->FrameChanged(container, newframe, dirtyRect);
 }
 
