@@ -51,7 +51,7 @@
 #include "nsXULAtoms.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIContent.h"
 #include "nsINameSpaceManager.h"
 #include "nsBoxLayoutState.h"
@@ -87,13 +87,13 @@ nsLeafBoxFrame::nsLeafBoxFrame(nsIPresShell* aShell):nsBox(aShell)
 {
 }
 
+#ifdef DEBUG_LAYOUT
 void
 nsLeafBoxFrame::GetBoxName(nsAutoString& aName)
 {
-#ifdef DEBUG
    GetFrameName(aName);
-#endif
 }
+#endif
 
 
 /**
@@ -103,29 +103,21 @@ NS_IMETHODIMP
 nsLeafBoxFrame::Init(nsIPresContext*  aPresContext,
               nsIContent*      aContent,
               nsIFrame*        aParent,
-              nsIStyleContext* aContext,
+              nsStyleContext*  aContext,
               nsIFrame*        aPrevInFlow)
 {
   nsresult  rv = nsLeafFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
    // see if we need a widget
-  nsCOMPtr<nsIBox> parent (do_QueryInterface(aParent));
-  if (parent) {
+  nsIBox *parent;
+  if (aParent && NS_SUCCEEDED(CallQueryInterface(aParent, &parent))) {
     PRBool needsWidget = PR_FALSE;
     parent->ChildrenMustHaveWidgets(needsWidget);
     if (needsWidget) {
-        nsIView* view = nsnull;
-        GetView(aPresContext, &view);
+        nsHTMLContainerFrame::CreateViewForFrame(this, nsnull, PR_TRUE); 
+        nsIView* view = GetView();
 
-        if (!view) {
-           nsHTMLContainerFrame::CreateViewForFrame(aPresContext,this,mStyleContext,nsnull,PR_TRUE); 
-           GetView(aPresContext, &view);
-        }
-
-        nsCOMPtr<nsIWidget> widget;
-        view->GetWidget(*getter_AddRefs(widget));
-
-        if (!widget)
+        if (!view->HasWidget())
            view->CreateWidget(kWidgetCID);   
     }
   }
@@ -139,17 +131,17 @@ nsLeafBoxFrame::Init(nsIPresContext*  aPresContext,
 
 NS_IMETHODIMP
 nsLeafBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
-                               nsIContent* aChild,
-                               PRInt32 aNameSpaceID,
-                               nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                                 nsIContent* aChild,
+                                 PRInt32 aNameSpaceID,
+                                 nsIAtom* aAttribute,
+                                 PRInt32 aModType)
 {
-    nsresult rv = nsLeafFrame::AttributeChanged(aPresContext, aChild,
-                                              aNameSpaceID, aAttribute, aModType, aHint);
+  nsresult rv = nsLeafFrame::AttributeChanged(aPresContext, aChild,
+                                              aNameSpaceID, aAttribute,
+                                              aModType);
 
-    if (aAttribute == nsXULAtoms::mousethrough) 
-       UpdateMouseThrough();
+  if (aAttribute == nsXULAtoms::mousethrough) 
+    UpdateMouseThrough();
 
   return rv;
 }
@@ -159,9 +151,9 @@ void nsLeafBoxFrame::UpdateMouseThrough()
   if (mContent) {
     nsAutoString value;
     if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::mousethrough, value)) {
-        if (value.EqualsIgnoreCase("never")) 
+        if (value.Equals(NS_LITERAL_STRING("never")))
           mMouseThrough = never;
-        else if (value.EqualsIgnoreCase("always")) 
+        else if (value.Equals(NS_LITERAL_STRING("always")))
           mMouseThrough = always;
       
     }
@@ -345,41 +337,30 @@ nsLeafBoxFrame::Reflow(nsIPresContext*   aPresContext,
 
   // max sure the max element size reflects
   // our min width
-  nsSize* maxElementSize = nsnull;
-  state.GetMaxElementSize(&maxElementSize);
-  if (maxElementSize)
+  nscoord* maxElementWidth = state.GetMaxElementWidth();
+  if (maxElementWidth)
   {
      nsSize minSize(0,0);
      GetMinSize(state,  minSize);
 
      if (mRect.width > minSize.width) {
        if (aReflowState.mComputedWidth == NS_INTRINSICSIZE) {
-         maxElementSize->width = minSize.width;
+         *maxElementWidth = minSize.width;
        } else {
-         maxElementSize->width = mRect.width;
+         *maxElementWidth = mRect.width;
        }
      } else {
-        maxElementSize->width = mRect.width;
-     }
-
-     if (mRect.height > minSize.height) {
-       if (aReflowState.mComputedHeight == NS_INTRINSICSIZE) {
-         maxElementSize->height = minSize.height;
-       } else {
-         maxElementSize->height = mRect.height;
-       }
-     } else {
-        maxElementSize->height = mRect.height;
+        *maxElementWidth = mRect.width;
      }
   }
 #ifdef DO_NOISY_REFLOW
   {
     printf("%p ** nsLBF(done) W:%d H:%d  ", this, aDesiredSize.width, aDesiredSize.height);
 
-    if (maxElementSize) {
-      printf("MW:%d MH:%d\n", maxElementSize->width, maxElementSize->height); 
+    if (maxElementWidth) {
+      printf("MW:%d\n", *maxElementWidth); 
     } else {
-      printf("MW:? MH:?\n"); 
+      printf("MW:?\n"); 
     }
 
   }
@@ -409,12 +390,12 @@ nsLeafBoxFrame::Release(void)
 }
 
 NS_IMETHODIMP
-nsLeafBoxFrame::ContentChanged(nsIPresContext* aPresContext,
-                            nsIContent*     aChild,
-                            nsISupports*    aSubContent)
+nsLeafBoxFrame::CharacterDataChanged(nsIPresContext* aPresContext,
+                                     nsIContent*     aChild,
+                                     PRBool          aAppend)
 {
   NeedsRecalc();
-  return nsLeafFrame::ContentChanged(aPresContext, aChild, aSubContent);
+  return nsLeafFrame::CharacterDataChanged(aPresContext, aChild, aAppend);
 }
 
 
