@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -16,10 +16,12 @@
  *
  * The Initial Developer of the Original Code is 
  * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
+ * Portions created by the Initial Developer are Copyright (C) 2001-2003
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *  Seth Spitzer <sspitzer@netscape.com>
+ *  Dan Mosedale <dmose@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -57,19 +59,17 @@
 #include "nsIDOMElement.h"
 #include "nsIAtom.h"
 #include "nsIImapIncomingServer.h"
-#include "nsIMsgSearchNotify.h"
 #include "nsIPref.h"
 #include "nsIWeakReference.h"
 #include "nsIObserver.h"
-
+#include "nsIMsgFilterPlugin.h"
 #include "nsIStringBundle.h"
 
 #define MESSENGER_STRING_URL       "chrome://messenger/locale/messenger.properties"
 
 enum eFieldType {
     kCollationKey,
-    kU32,
-    kPRTime
+    kU32
 };
 
 // reserve the top 8 bits in the msg flags for the view-only flags.
@@ -88,7 +88,9 @@ enum eFieldType {
 // I think this will be an abstract implementation class.
 // The classes that implement the tree support will probably
 // inherit from this class.
-class nsMsgDBView : public nsIMsgDBView, public nsIDBChangeListener, public nsITreeView, public nsIMsgSearchNotify, public nsIObserver
+class nsMsgDBView : public nsIMsgDBView, public nsIDBChangeListener,
+                    public nsITreeView, public nsIObserver,
+                    public nsIJunkMailClassificationListener
 {
 public:
   nsMsgDBView();
@@ -98,8 +100,8 @@ public:
   NS_DECL_NSIMSGDBVIEW
   NS_DECL_NSIDBCHANGELISTENER
   NS_DECL_NSITREEVIEW
-  NS_DECL_NSIMSGSEARCHNOTIFY
   NS_DECL_NSIOBSERVER
+  NS_DECL_NSIJUNKMAILCLASSIFICATIONLISTENER
 
 protected:
   static nsrefcnt gInstanceCount;
@@ -108,6 +110,8 @@ protected:
   static nsIAtom* kUnreadMsgAtom;
   static nsIAtom* kNewMsgAtom;
   static nsIAtom* kReadMsgAtom;
+  static nsIAtom* kRepliedMsgAtom;
+  static nsIAtom* kForwardedMsgAtom;
   static nsIAtom* kOfflineMsgAtom;
   static nsIAtom* kFlaggedMsgAtom;
   static nsIAtom* kNewsMsgAtom;
@@ -116,6 +120,7 @@ protected:
   static nsIAtom* kHasUnreadAtom;
   static nsIAtom* kWatchThreadAtom;
   static nsIAtom* kIgnoreThreadAtom;
+  static nsIAtom* kHasImageAtom;
 
 #ifdef SUPPORT_PRIORITY_COLORS
   static nsIAtom* kHighestPriorityAtom;
@@ -133,6 +138,9 @@ protected:
   static nsIAtom* kLabelColorWhiteAtom;
   static nsIAtom* kLabelColorBlackAtom;
 
+  static nsIAtom* kJunkMsgAtom;
+  static nsIAtom* kNotJunkMsgAtom;
+
   static PRUnichar* kReadString;
   static PRUnichar* kRepliedString;
   static PRUnichar* kForwardedString;
@@ -143,12 +151,13 @@ protected:
   PRUint32 mNumSelectedRows; // we cache this to determine when to push command status notifications.
   PRPackedBool   mSuppressMsgDisplay; // set when the message pane is collapsed
   PRPackedBool   mSuppressCommandUpdating;
-  PRPackedBool   mIsSearchView; // tells if the search view is loaded.
   PRPackedBool   mRemovingRow; // set when we're telling the outline a row is being removed. used to suppress msg loading.
                         // during delete/move operations.
-  PRPackedBool  mOfflineMsgSelected;
+  PRPackedBool  mCommandsNeedDisablingBecauseOffline;
+  PRPackedBool  mSuppressChangeNotification;
   virtual const char * GetViewName(void) {return "MsgDBView"; }
   nsresult FetchAuthor(nsIMsgHdr * aHdr, PRUnichar ** aAuthorString);
+  nsresult FetchRecipient(nsIMsgHdr * aHdr, PRUnichar ** aRecipientString);
   nsresult FetchSubject(nsIMsgHdr * aMsgHdr, PRUint32 aFlags, PRUnichar ** aValue);
   nsresult FetchDate(nsIMsgHdr * aHdr, PRUnichar ** aDateString);
   nsresult FetchStatus(PRUint32 aFlags, PRUnichar ** aStatusString);
@@ -177,7 +186,6 @@ protected:
   nsresult GetSelectedIndices(nsUInt32Array *selection);
   nsresult GenerateURIForMsgKey(nsMsgKey aMsgKey, nsIMsgFolder *folder, char ** aURI);
 // routines used in building up view
-  virtual nsresult AddKeys(nsMsgKey *pKeys, PRInt32 *pFlags, const char *pLevels, nsMsgViewSortTypeValue sortType, PRInt32 numKeysToAdd);
   virtual PRBool WantsThisThread(nsIMsgThread * thread);
   virtual nsresult	AddHdr(nsIMsgDBHdr *msgHdr);
   PRBool GetShowingIgnored() {return (m_viewFlags & nsMsgViewFlagsType::kShowIgnored) != 0;}
@@ -210,28 +218,29 @@ protected:
   nsresult ReverseThreads();
   nsresult SaveSortInfo(nsMsgViewSortTypeValue sortType, nsMsgViewSortOrderValue sortOrder);
 
-	nsMsgKey		GetAt(nsMsgViewIndex index) ;
-	nsMsgViewIndex	FindViewIndex(nsMsgKey  key) 
-						{return (nsMsgViewIndex) (m_keys.FindIndex(key));}
-	virtual nsMsgViewIndex	FindKey(nsMsgKey key, PRBool expand);
-	virtual nsresult GetDBForViewIndex(nsMsgViewIndex index, nsIMsgDatabase **db);
+  nsMsgKey		GetAt(nsMsgViewIndex index) ;
+  nsMsgViewIndex	FindViewIndex(nsMsgKey  key) 
+					  {return (nsMsgViewIndex) (m_keys.FindIndex(key));}
+  virtual nsMsgViewIndex	FindKey(nsMsgKey key, PRBool expand);
+  virtual nsresult GetDBForViewIndex(nsMsgViewIndex index, nsIMsgDatabase **db);
   virtual nsresult GetFolders(nsISupportsArray **folders);
+  virtual nsresult GetFolderFromMsgURI(const char *aMsgURI, nsIMsgFolder **aFolder);
 
   nsresult ListIdsInThread(nsIMsgThread *threadHdr, nsMsgViewIndex viewIndex, PRUint32 *pNumListed);
   nsresult ListUnreadIdsInThread(nsIMsgThread *threadHdr, nsMsgViewIndex startOfThreadViewIndex, PRUint32 *pNumListed);
   PRInt32  FindLevelInThread(nsIMsgDBHdr *msgHdr, nsMsgViewIndex startOfThreadViewIndex);
   nsresult ListIdsInThreadOrder(nsIMsgThread *threadHdr, nsMsgKey parentKey, PRInt32 level, nsMsgViewIndex *viewIndex, PRUint32 *pNumListed);
-	PRInt32	  GetSize(void) {return(m_keys.GetSize());}
+  PRInt32  GetSize(void) {return(m_keys.GetSize());}
 
   // notification api's
-	void	EnableChangeUpdates();
-	void	DisableChangeUpdates();
-	void	NoteChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
-							   nsMsgViewNotificationCodeValue changeType);
-	void	NoteStartChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
-							   nsMsgViewNotificationCodeValue changeType);
-	void	NoteEndChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
-							   nsMsgViewNotificationCodeValue changeType);
+  void	EnableChangeUpdates();
+  void	DisableChangeUpdates();
+  void	NoteChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
+                    nsMsgViewNotificationCodeValue changeType);
+  void	NoteStartChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
+                        nsMsgViewNotificationCodeValue changeType);
+  void	NoteEndChange(nsMsgViewIndex firstlineChanged, PRInt32 numChanged, 
+                        nsMsgViewNotificationCodeValue changeType);
 
   // for commands
   nsresult ApplyCommandToIndices(nsMsgViewCommandTypeValue command, nsMsgViewIndex* indices,
@@ -240,6 +249,11 @@ protected:
                     PRInt32 numIndices, nsIMsgFolder *destFolder);
   virtual nsresult CopyMessages(nsIMsgWindow *window, nsMsgViewIndex *indices, PRInt32 numIndices, PRBool isMove, nsIMsgFolder *destFolder);
   virtual nsresult DeleteMessages(nsIMsgWindow *window, nsMsgViewIndex *indices, PRInt32 numIndices, PRBool deleteStorage);
+  nsresult SetStringPropertyByIndex(nsMsgViewIndex index, const char *aProperty, const char *aValue);
+  nsresult SetJunkScoreByIndex(nsIJunkMailPlugin *aJunkPlugin, 
+                               nsMsgViewIndex aIndex,
+                               nsMsgJunkStatus aNewClassification,
+                               PRBool aIsLastInBatch);
   nsresult ToggleReadByIndex(nsMsgViewIndex index);
   nsresult SetReadByIndex(nsMsgViewIndex index, PRBool read);
   nsresult SetThreadOfMsgReadByIndex(nsMsgViewIndex index, nsMsgKeyArray &keysMarkedRead, PRBool read);
@@ -258,16 +272,12 @@ protected:
   nsresult SetThreadIgnored(nsIMsgThread *thread, nsMsgViewIndex threadIndex, PRBool ignored);
   nsresult DownloadForOffline(nsIMsgWindow *window, nsMsgViewIndex *indices, PRInt32 numIndices);
   nsresult DownloadFlaggedForOffline(nsIMsgWindow *window);
-
-
-  nsMsgViewIndex	GetThreadFromMsgIndex(nsMsgViewIndex index, 
-													 nsIMsgThread **threadHdr);
+  nsMsgViewIndex	GetThreadFromMsgIndex(nsMsgViewIndex index, nsIMsgThread **threadHdr);
 
   // for sorting
   nsresult GetFieldTypeAndLenForSort(nsMsgViewSortTypeValue sortType, PRUint16 *pMaxLen, eFieldType *pFieldType);
   nsresult GetCollationKey(nsIMsgHdr *msgHdr, nsMsgViewSortTypeValue sortType, PRUint8 **result, PRUint32 *len);
-  nsresult GetLongField(nsIMsgHdr *msgHdr, nsMsgViewSortTypeValue sortType, PRUint32 *result);
-  nsresult GetPRTimeField(nsIMsgHdr *msgHdr, nsMsgViewSortTypeValue sortType, PRTime *result);
+  nsresult GetLongField(nsIMsgDBHdr *msgHdr, nsMsgViewSortTypeValue sortType, PRUint32 *result);
   nsresult GetStatusSortValue(nsIMsgHdr *msgHdr, PRUint32 *result);
   nsresult GetLocationCollationKey(nsIMsgHdr *msgHdr, PRUint8 **result, PRUint32 *len);
 
@@ -284,7 +294,6 @@ protected:
   nsresult MarkThreadRead(nsIMsgThread *threadHdr, nsMsgViewIndex threadIndex, nsMsgKeyArray &idsMarkedRead, PRBool bRead);
   PRBool IsValidIndex(nsMsgViewIndex index);
   nsresult ToggleIgnored(nsMsgViewIndex * indices, PRInt32 numIndices, PRBool *resultToggleState);
-  nsresult GetKeyForFirstSelectedMessage(nsMsgKey *key);
   PRBool OfflineMsgSelected(nsMsgViewIndex * indices, PRInt32 numIndices);
   PRUnichar * GetString(const PRUnichar *aStringName);
   nsresult AddLabelPrefObservers();
@@ -294,10 +303,13 @@ protected:
   nsresult AppendLabelProperties(nsMsgLabelValue label, nsISupportsArray *aProperties);
   nsresult AppendSelectedTextColorProperties(nsMsgLabelValue label, nsISupportsArray *aProperties);
   nsresult InitLabelPrefs(void);
+  nsresult CopyDBView(nsMsgDBView *aNewMsgDBView, nsIMessenger *aMessengerInstance, nsIMsgWindow *aMsgWindow, nsIMsgDBViewCommandUpdater *aCmdUpdater);
   void InitializeAtomsAndLiterals();
   PRInt32 GetLevelInUnreadView(nsIMsgDBHdr *msgHdr, nsMsgViewIndex startOfThread, nsMsgViewIndex viewIndex);
   nsresult GetImapDeleteModel(nsIMsgFolder *folder);
   nsresult UpdateDisplayMessage(nsMsgKey aMsgKey);
+  nsresult LoadMessageByMsgKeyHelper(nsMsgKey aMsgKey, PRBool forceAllParts);
+  nsresult ReloadMessageHelper(PRBool forceAllParts);
 
   PRBool AdjustReadFlag(nsIMsgDBHdr *msgHdr, PRUint32 *msgFlags);
   void FreeAll(nsVoidArray *ptrs);
@@ -317,14 +329,17 @@ protected:
   // if we're deleting messages, we want to hold off loading messages on selection changed until the delete is done
   // and we want to batch notifications.
   PRPackedBool m_deletingRows;
-  PRPackedBool mIsSpecialFolder; // for special folders, the Sender column really shows recipients.
+  // for certain special folders 
+  // and decendents of those folders
+  // (like the "Sent" folder, "Sent/Old Sent")
+  // the Sender column really shows recipients.
   PRPackedBool mIsNews;          // we have special icons for news, and for news, we show lines instead of size
   PRPackedBool m_sortValid;
   PRUint8      m_saveRestoreSelectionDepth;
 
   nsCOMPtr <nsIMsgDatabase> m_db;
   nsCOMPtr <nsIMsgFolder> m_folder;
-  nsWeakPtr m_searchSession;
+  nsCOMPtr <nsIAtom> mRedirectorTypeAtom;
   nsMsgViewSortTypeValue  m_sortType;
   nsMsgViewSortOrderValue m_sortOrder;
   nsMsgViewFlagsTypeValue m_viewFlags;
@@ -344,6 +359,27 @@ protected:
   nsString mLabelPrefColors[PREF_LABELS_MAX];
   // used to cache the atoms created for each color to be displayed
   static nsIAtom* mLabelPrefColorAtoms[PREF_LABELS_MAX];
+
+  // used to know to finish out the junk mail classification batch when the 
+  // last classification callback happens
+  nsCString mLastJunkUriInBatch;
+  PRUint8 mOutstandingJunkBatches;
+  nsUInt32Array mIndicesToNoteChange;
+
+protected:
+  static nsresult   InitDisplayFormats();
+
+private:
+  static nsDateFormatSelector  m_dateFormatDefault;
+  static nsDateFormatSelector  m_dateFormatThisWeek;
+  static nsDateFormatSelector  m_dateFormatToday;
+  PRBool ServerSupportsFilterAfterTheFact();
+
+  nsMsgKeyArray	mJunkKeys;
+  nsCOMPtr <nsIMsgFolder> mJunkTargetFolder;
+  nsresult PerformActionOnJunkMsgs();
+  nsresult SaveJunkMsgForAction(nsIMsgIncomingServer *aServer, const char *aMsgURI, nsMsgJunkStatus aClassification);
+
 };
 
 #endif
