@@ -53,7 +53,7 @@
 #include "nsOutlookStringBundle.h"
 #include "nsABBaseCID.h"
 #include "nsIAbCard.h"
-#include "nsIAddrDatabase.h"
+#include "nsAddrDatabase.h"
 #include "OutlookDebugLog.h"
 #include "nsOutlookMail.h"
 #include "nsUnicharUtils.h"
@@ -98,8 +98,8 @@ static MAPIFields	gMapiFields[] = {
 	{ 20, kNoMultiLine, PR_STATE_OR_PROVINCE},
 	{ 17, 18, PR_STREET_ADDRESS},
 	{ 21, kNoMultiLine, PR_POSTAL_CODE},
-	{ 26, kNoMultiLine, PR_PERSONAL_HOME_PAGE},
-	{ 27, kNoMultiLine, PR_BUSINESS_HOME_PAGE},
+	{ 27, kNoMultiLine, PR_PERSONAL_HOME_PAGE},
+	{ 26, kNoMultiLine, PR_BUSINESS_HOME_PAGE},
 	{ 13, kNoMultiLine, PR_HOME_ADDRESS_CITY},
 	{ 16, kNoMultiLine, PR_HOME_ADDRESS_COUNTRY},
 	{ 15, kNoMultiLine, PR_HOME_ADDRESS_POSTAL_CODE},
@@ -350,6 +350,33 @@ void nsOutlookMail::OpenMessageStore( CMapiFolder *pNextFolder)
 	}
 }
 
+void nsOutlookMail::SetDefaultContentType(CMapiMessage &msg, nsCString &cType)
+{
+  cType.Truncate();
+
+  // MAPI doesn't seem to return the entire body data (ie, multiple parts) for
+  // content type "multipart/alternative", instead it only returns the body data 
+  // for a particular part. For this reason we'll need to set the content type
+  // here. Same thing when conten type is not being set at all.
+  if (msg.GetMimeContentLen())
+  {
+    // If content type is not multipart/alternative, return.
+    if (nsCRT::strcasecmp(msg.GetMimeContent(), "multipart/alternative"))
+      return;
+
+    // For multipart/alternative, if no body or boundary,
+    // or boundary is found in body then return.
+    const char *body = msg.GetBody();
+    const char *boundary = msg.GetMimeBoundary();
+    if (!body || !boundary || strstr(body, boundary))
+      return;
+  }
+  
+  // Now default the content type to text/html or text/plain
+  // depending whether or not the body data is html.
+  cType = msg.BodyIsHtml() ? "text/html" : "text/plain";
+}
+
 nsresult nsOutlookMail::ImportMailbox( PRUint32 *pDoneSoFar, PRBool *pAbort, PRInt32 index, const PRUnichar *pName, nsIFileSpec *pDest, PRInt32 *pMsgCount)
 {
 	if ((index < 0) || (index >= m_folderList.GetSize())) {
@@ -439,8 +466,23 @@ nsresult nsOutlookMail::ImportMailbox( PRUint32 *pDoneSoFar, PRBool *pAbort, PRI
 			}
 						
 			// --------------------------------------------------------------
-			compose.SetBody( msg.GetBody(), msg.GetBodyLen());
-			compose.SetHeaders( msg.GetHeaders(), msg.GetHeaderLen());
+			compose.SetBody( msg.GetBody());
+
+      // Need to convert all headers to unicode (for i18n).
+      // Init header here since 'composes' is used for all msgs.
+      compose.SetHeaders("");
+      nsCOMPtr<nsIImportService>	impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get import service");
+      if (NS_SUCCEEDED(rv))
+      {
+        nsAutoString newheader;
+        nsCAutoString tempCStr(msg.GetHeaders(), msg.GetHeaderLen());
+        rv = impSvc->SystemStringToUnicode(tempCStr.get(), newheader);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert headers to utf8");
+        if (NS_SUCCEEDED(rv))
+          compose.SetHeaders(NS_ConvertUCS2toUTF8(newheader).get());
+      }
+
 			compose.SetAttachments( &m_attachments);
 
       // See if it's a drafts folder. Outlook doesn't allow drafts
@@ -456,7 +498,9 @@ nsresult nsOutlookMail::ImportMailbox( PRUint32 *pDoneSoFar, PRBool *pAbort, PRI
 				I can't get no satisfaction
 			*/
 			if (msg.GetHeaderLen()) {
-				rv = compose.SendTheMessage( compositionFile, mode);
+        nsCAutoString cType;
+        SetDefaultContentType(msg, cType);
+				rv = compose.SendTheMessage( compositionFile, mode, cType);
 				if (NS_SUCCEEDED( rv)) {
 					rv = compose.CopyComposedMessage( fromLine, compositionFile, pDest, copy);
 					DeleteFile( compositionFile);
@@ -790,13 +834,31 @@ void nsOutlookMail::BuildAttachments( CMapiMessage& msg, int count)
 						// We have a file spec, now get the other info
 						OutlookAttachment *a = new OutlookAttachment;
 						a->mimeType = nsCRT::strdup( msg.GetMimeType());
-						a->description = nsCRT::strdup( msg.GetFileName());
-						if (!strlen( a->description)) {
-							nsCRT::free( a->description);
-							nsCString	str("Attachment ");
+            // Init description here so that we cacn tell
+            // if defaul tattacchment is needed later.
+            a->description = nsnull;  
+
+            const char *fileName = msg.GetFileName();
+            if (fileName && fileName[0]) {
+              // Convert description to unicode.
+              nsCOMPtr<nsIImportService>	impSvc = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+              NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get import service");
+              if (NS_SUCCEEDED(rv)) {
+                nsAutoString description;
+                rv = impSvc->SystemStringToUnicode(msg.GetFileName(), description);
+                NS_ASSERTION(NS_SUCCEEDED(rv), "failed to convert system string to unicode");
+                if (NS_SUCCEEDED(rv))
+                  a->description = ToNewUTF8String(description);
+              }
+            }
+
+            // If no description use "Attachment i" format.
+            if (!a->description) {
+              nsCAutoString	str("Attachment ");
 							str.AppendInt( (PRInt32) i);
 							a->description = ToNewCString( str);
 						}
+
 						a->pAttachment = pSpec;
 						m_attachments.AppendElement( a);
 					}
@@ -943,7 +1005,7 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
           pVal = m_mapi.GetMapiProperty( lpMsg, PR_SUBJECT);
           if (pVal)
             m_mapi.GetStringFromProp( pVal, subject);
-          CreateList(subject.get(), pDb);
+          CreateList(subject.get(), pDb, lpMsg, pFieldMap);
         }
 			}			
 
@@ -955,8 +1017,10 @@ nsresult nsOutlookMail::ImportAddresses( PRUint32 *pCount, PRUint32 *pTotal, con
   rv = pDb->Commit(nsAddrDBCommitType::kLargeCommit);
 	return rv;
 }
-
-nsresult nsOutlookMail::CreateList( const PRUnichar * pName, nsIAddrDatabase *pDb)
+nsresult nsOutlookMail::CreateList( const PRUnichar * pName,
+                                   nsIAddrDatabase *pDb,
+                                   LPMAPIPROP pUserList,
+                                   nsIImportFieldMap *pFieldMap)
 {
   // If no name provided then we're done.
   if (!pName || !(*pName))
@@ -967,20 +1031,103 @@ nsresult nsOutlookMail::CreateList( const PRUnichar * pName, nsIAddrDatabase *pD
   if (!pDb)
     return rv;
 
-  nsCOMPtr <nsIMdbRow> newRow;
-  rv = pDb->GetNewListRow(getter_AddRefs(newRow));
+  nsCOMPtr <nsIMdbRow> newListRow;
+  rv = pDb->GetNewListRow(getter_AddRefs(newListRow));
   NS_ENSURE_SUCCESS(rv, rv);
-
   nsCAutoString column;
   column.AssignWithConversion(pName);
-  rv = pDb->AddListName(newRow, column.get());
+  rv = pDb->AddListName(newListRow, column.get());
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = pDb->AddCardRowToDB(newRow);
+
+    HRESULT             hr;
+    LPSPropValue aValue = NULL ;
+    ULONG aValueCount = 0 ;
+
+    LPSPropTagArray properties = NULL ;
+    m_mapi.MAPIAllocateBuffer(CbNewSPropTagArray(1),
+                       (void **)&properties) ;
+    properties->cValues = 1;
+    properties->aulPropTag [0] = m_mapi.GetEmailPropertyTag(pUserList, 0x8054);
+    hr = pUserList->GetProps(properties, 0, &aValueCount, &aValue) ;
+
+    SBinaryArray *sa=(SBinaryArray *)&aValue->Value.bin;
+  if (!sa || !sa->lpbin)
+    return NS_ERROR_NULL_POINTER;
+
+    LPENTRYID    lpEid;
+    ULONG        cbEid;
+    PRInt32        idx;
+    LPMESSAGE        lpMsg;
+    nsCString        type;
+    LPSPropValue    pVal;
+    nsString        subject;
+    PRUint32 total;
+    
+
+    total=sa->cValues;
+    for (idx=0;idx<sa->cValues ;idx++)
+    {
+        lpEid= (LPENTRYID) sa->lpbin[idx].lpb;
+        cbEid = sa->lpbin[idx].cb;
+
+
+        if (!m_mapi.OpenEntry(cbEid, lpEid, (LPUNKNOWN *) &lpMsg))
+        {
+            
+            IMPORT_LOG1( "*** Error opening messages in mailbox: %S\n", pName);
+            return( NS_ERROR_FAILURE);
+        }
+                // This is a contact, add it to the address book!
+                subject.Truncate( 0);
+                pVal = m_mapi.GetMapiProperty( lpMsg, PR_SUBJECT);
+                if (pVal)
+                    m_mapi.GetStringFromProp( pVal, subject);
+                
+                    nsCOMPtr <nsIMdbRow> newRow;
+                    nsCOMPtr <nsIMdbRow> oldRow;
+                    pDb->GetNewRow( getter_AddRefs(newRow));
+                    if (newRow) {
+                        if (BuildCard( subject.get(), pDb, newRow, lpMsg, pFieldMap)) 
+                        {
+                            nsCOMPtr <nsIAbCard> userCard;
+                            nsCOMPtr <nsIAbCard> newCard;
+                            userCard = do_CreateInstance(NS_ABMDBCARD_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = pDb->AddListDirNode(newRow);
+                            pDb->InitCardFromRow(userCard,newRow);
+                            
+                            //add card to db
+                            PRBool bl=PR_FALSE;
+                            pDb->FindRowByCard(userCard,getter_AddRefs(oldRow));
+                            if (oldRow)
+                            {
+                                newRow = oldRow;
+                            }
+                            else
+                            {
+                                pDb->AddCardRowToDB( newRow);
+                            }
+                            
+                            //add card list
+                            pDb->AddListCardColumnsToRow(userCard,
+                                              newListRow,idx+1,
+                                              getter_AddRefs(newCard),PR_TRUE);
+
+                            
+                        }
+                    }
+
+
+            }
+  
+  rv = pDb->AddCardRowToDB(newListRow);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = pDb->SetListAddressTotal(newListRow, total);
+  rv = pDb->AddListDirNode(newListRow);
   return rv;
 }
 
+
 void nsOutlookMail::SanitizeValue( nsString& val)
 {
 	val.ReplaceSubstring(NS_LITERAL_STRING("\x0D\x0A").get(),
