@@ -21,12 +21,12 @@
 
 #include "nsIServiceManager.h"
 #include "nsIImportService.h"
-#include "nsIAddrDatabase.h"
+#include "nsAddrDatabase.h"
 #include "nsAbBaseCID.h"
 #include "nsIAbCard.h"
 #include "nsReadableUtils.h"
-
-static NS_DEFINE_CID(kAbCardPropertyCID, NS_ABCARDPROPERTY_CID);
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 
 #include "TextDebugLog.h"
 
@@ -37,7 +37,6 @@ static NS_DEFINE_CID(kAbCardPropertyCID, NS_ABCARDPROPERTY_CID);
 
 // ONLY included for the ldif code that was copied over from nsAddressBook.cpp
 // Maybe re-write that code and get rid of this? (isspace is what is needed)
-#include "xp_core.h"
 #include "plstr.h"
 #include "prmem.h"
 #include "prprf.h"
@@ -47,6 +46,8 @@ nsTextAddress::nsTextAddress()
 {
     m_database = nsnull;
     m_fieldMap = nsnull;
+    m_LFCount = 0;
+    m_CRCount = 0;
 }
 
 nsTextAddress::~nsTextAddress()
@@ -316,16 +317,15 @@ PRBool nsTextAddress::GetField( const char *pLine, PRInt32 maxLen, PRInt32 index
         if (len >= maxLen)
             break;
         if (*pChar == '"') {
-            pChar++;
-            len++;
-            while ((len < maxLen) && (*pChar != '"')) {
+            len = -1;
+            do {
                 len++;
                 pChar++;
                 if (((len + 1) < maxLen) && (*pChar == '"') && (*(pChar + 1) == '"')) {
                     len += 2;
                     pChar += 2;
                 }
-            }
+            } while ((len < maxLen) && (*pChar != '"'));
             if (len < maxLen) {
                 pChar++;
                 len++;
@@ -362,20 +362,19 @@ PRBool nsTextAddress::GetField( const char *pLine, PRInt32 maxLen, PRInt32 index
     PRInt32        fLen = 0;
     PRBool        quoted = PR_FALSE;
     if (*pChar == '"') {
-        quoted = PR_TRUE;
         pStart++;
-        pChar++;
-        len++;
-        while ((len < maxLen) && (*pChar != '"')) {
+        fLen = -1;
+        do {
             pChar++;
             len++;
             fLen++;
             if (((len + 1) < maxLen) && (*pChar == '"') && (*(pChar + 1) == '"')) {
+                quoted = PR_TRUE;
                 len += 2;
                 pChar += 2;
                 fLen += 2;
             }
-        }
+        } while ((len < maxLen) && (*pChar != '"'));
     }
     else {
         while ((len < maxLen) && (*pChar != delim)) {
@@ -495,7 +494,7 @@ nsresult nsTextAddress::ProcessLine( const char *pLine, PRInt32 len, nsString& e
             rv = m_fieldMap->GetFieldActive( i, &active);
         if (NS_SUCCEEDED( rv) && active) {
             if (GetField( pLine, len, i, fieldVal, m_delim)) {
-                if (fieldVal.Length()) {
+                if (!fieldVal.IsEmpty()) {
                     if (!newRow) {
                         rv = m_database->GetNewRow( &newRow);
                         if (NS_FAILED( rv)) {
@@ -546,6 +545,8 @@ static const char *const sLDIFFields[] = {
 };
 #define kMaxLDIFLen        14
 
+// Count total number of legal ldif fields and records in the first 100 lines of the 
+// file and if the average legal ldif field is 3 or higher than it's a valid ldif file.
 nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
 {
     *pIsLDIF = PR_FALSE;
@@ -568,15 +569,13 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
     PRBool    wasTruncated = PR_FALSE;
     PRInt32    lineLen = 0;
     PRInt32    lineCount = 0;
-    PRInt32    ldifFields = 0;
+    PRInt32    ldifFields = 0;  // total number of legal ldif fields.
     char    field[kMaxLDIFLen];
     PRInt32    fLen = 0;
     char *    pChar;
-    PRInt32    rCount = 1;
+    PRInt32    recCount = 0;  // total number of records.
     PRInt32    i;
     PRBool    gotLDIF = PR_FALSE;
-    PRInt32    commaCount = 0;
-    PRInt32    tabCount = 0;
 
     while (!eof && NS_SUCCEEDED( rv) && (lineCount < 100)) {
         wasTruncated = PR_FALSE;
@@ -585,24 +584,12 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
             pLine[kTextAddressBufferSz - 1] = 0;
         if (NS_SUCCEEDED( rv)) {
             lineLen = strlen( pLine);            
-            tabCount += CountFields( pLine, lineLen, 9);
-            commaCount += CountFields( pLine, lineLen, ',');
             pChar = pLine;
             if (!lineLen && gotLDIF) {
-                rCount++;
+                recCount++;
                 gotLDIF = PR_FALSE;
             }
-            
-            /* 
-                There should be no leading whitespace in front
-                of an ldif field!
-            *//*
-            while (lineLen && ((*pChar == ' ') || (*pChar == 9))) {
-                lineLen--;
-                pChar++;
-            }
-            */
-            
+                    
             if (lineLen && (*pChar != ' ') && (*pChar != 9)) {
                 fLen = 0;
                 while (lineLen && (fLen < (kMaxLDIFLen - 1)) && (*pChar != ':')) {
@@ -614,10 +601,10 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
                 
                 field[fLen] = 0;
                 if (lineLen && (*pChar == ':') && (fLen < (kMaxLDIFLen - 1))) {
-                    // see if this is an ldif field?
+                    // see if this is an ldif field (case insensitive)?
                     i = 0;
                     while (sLDIFFields[i]) {
-                        if (!nsCRT::strcmp( sLDIFFields[i], field)) {
+                        if (!nsCRT::strcasecmp( sLDIFFields[i], field)) {
                             ldifFields++;
                             gotLDIF = PR_TRUE;
                             break;
@@ -632,28 +619,21 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
         }
         lineCount++;
     }
+
+    // If we just saw ldif address, increment recCount.
+    if (gotLDIF)
+      recCount++;
     
     rv = pSrc->CloseStream();
     
     delete [] pLine;
     
-    ldifFields /= rCount;
-    tabCount /= lineCount;
-    commaCount /= lineCount;
+    if (recCount > 1)
+      ldifFields /= recCount;
 
-    /*
-    if ((tabCount <= ldifFields) && (commaCount <= ldifFields) && (ldifFields > 1)) {
-        *pIsLDIF = PR_TRUE;
-    }
-    */
-    if (rCount == 1) {
-        if ((ldifFields >= 3) && (lineCount < 500))
-            *pIsLDIF = PR_TRUE;
-    }
-    else {
-        if (ldifFields >= 3)
-            *pIsLDIF = PR_TRUE;
-    }
+    // If the average field number >= 3 then it's a good ldif file.
+    if (ldifFields >= 3)
+      *pIsLDIF = PR_TRUE;
 
     return( NS_OK);
 }
@@ -674,6 +654,9 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
 #define IS_SPACE(VAL)                \
     (((((intn)(VAL)) & 0x7f) == ((intn)(VAL))) && isspace((intn)(VAL)) )
 
+// XXX TODO fix me
+// use the NSPR base64 library.  see plbase64.h
+// see bug #145367
 static unsigned char b642nib[0x80] = {
     0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
     0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
@@ -698,6 +681,7 @@ static unsigned char b642nib[0x80] = {
  * into components "type" and "value".  if a double colon separates type from
  * value, then value is encoded in base 64, and parse_line un-decodes it
  * (in place) before returning.
+ * in LDIF, non-ASCII data is treated as base64 encoded UTF-8 
  */
 
 nsresult nsTextAddress::str_parse_line(
@@ -845,43 +829,39 @@ char * nsTextAddress::str_getline( char **next )
  */
 nsresult nsTextAddress::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos)
 {
-    PRInt32 LFCount = 0;
-    PRInt32 CRCount = 0;
-
     for (; stopPos < len; stopPos++) 
     {
         char c = buf[stopPos];
 
         if (c == 0xA)
         {
-            LFCount++;
+            m_LFCount++;
         }
         else if (c == 0xD)
         {
-            CRCount++;
+            m_CRCount++;
         }
         else if ( c != 0xA && c != 0xD)
         {
-            if (LFCount == 0 && CRCount == 0)
+            if (m_LFCount == 0 && m_CRCount == 0)
                  m_ldifLine.Append(c);
-            else if (( LFCount > 1) || ( CRCount > 2 && LFCount ) ||
-                ( !LFCount && CRCount > 1 ))
+            else if (( m_LFCount > 1) || ( m_CRCount > 2 && m_LFCount ) ||
+                ( !m_LFCount && m_CRCount > 1 ))
             {
                 return NS_OK;
             }
-            else if ((LFCount == 1 || CRCount == 1))
+            else if ((m_LFCount == 1 || m_CRCount == 1))
             {
-                 m_ldifLine.Append('\n');
-                 m_ldifLine.Append(c);
-                LFCount = 0;
-                CRCount = 0;
+                m_ldifLine.Append('\n');
+                m_ldifLine.Append(c);
+                m_LFCount = 0;
+                m_CRCount = 0;
             }
         }
     }
-    if ((LFCount == 1 || CRCount == 1) && stopPos == len)
-        m_ldifLine.Append('\n');
-    if ((stopPos == len) && (LFCount > 1) || (CRCount > 2 && LFCount) ||
-        (!LFCount && CRCount > 1))
+
+    if ((stopPos == len) && (m_LFCount > 1) || (m_CRCount > 2 && m_LFCount) ||
+        (!m_LFCount && m_CRCount > 1))
         return NS_OK;
     else
         return NS_ERROR_FAILURE;
@@ -894,7 +874,8 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
     PRInt32 startPos = 0;
     PRInt32 len = 0;
     PRBool bEof = PR_FALSE;
-    nsVoidArray listPosArray;
+    nsVoidArray listPosArray;   // where each list/group starts in ldif file
+    nsVoidArray listSizeArray;  // size of the list/group info
     PRInt32 savedStartPos = 0;
     PRInt32 filePos = 0;
 
@@ -906,14 +887,14 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
 
             while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
             {
-                if (m_ldifLine.Find("groupOfNames") == -1)
+                 if (m_ldifLine.Find("groupOfNames") == kNotFound)
                     AddLdifRowToDatabase(PR_FALSE);
                 else
                 {
                     //keep file position for mailing list
                     listPosArray.AppendElement((void*)savedStartPos);
-                    if (m_ldifLine.Length() > 0)
-                        m_ldifLine.Truncate();
+                    listSizeArray.AppendElement((void*)(filePos + startPos-savedStartPos));
+                    ClearLdifRecordBuffer();
                 }
                 savedStartPos = filePos + startPos;
             }
@@ -922,23 +903,31 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
         }
     }
     //last row
-    if (m_ldifLine.Length() > 0 && m_ldifLine.Find("groupOfNames") == -1)
+    if (!m_ldifLine.IsEmpty() && m_ldifLine.Find("groupOfNames") == kNotFound)
         AddLdifRowToDatabase(PR_FALSE); 
+
     // mail Lists
-    PRInt32 i;
+    PRInt32 i, pos, size;
     PRInt32 listTotal = listPosArray.Count();
+    char *listBuf;
+    ClearLdifRecordBuffer();  // make sure the buffer is clean
     for (i = 0; i < listTotal; i++)
     {
-        PRInt32 pos = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        pos  = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        size = NS_PTR_TO_INT32(listSizeArray.ElementAt(i));
         if (NS_SUCCEEDED(pSrc->Seek(pos)))
         {
-            if (NS_SUCCEEDED(pSrc->Read(&pBuf, (PRInt32)sizeof(buf), &len)) && len > 0)
+            // Allocate enough space for the lists/groups as the size varies.
+            listBuf = (char *) PR_Malloc(size);
+            if (!listBuf)
+              continue;
+            if (NS_SUCCEEDED(pSrc->Read(&listBuf, size, &len)) && len > 0)
             {
                 startPos = 0;
 
-                while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
+                while (NS_SUCCEEDED(GetLdifStringRecord(listBuf, len, startPos)))
                 {
-                    if (m_ldifLine.Find("groupOfNames") != -1)
+                    if (m_ldifLine.Find("groupOfNames") != kNotFound)
                     {
                         AddLdifRowToDatabase(PR_TRUE);
                         if (NS_SUCCEEDED(pSrc->Seek(0)))
@@ -946,6 +935,7 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
                     }
                 }
             }
+            PR_FREEIF(listBuf);
         }
     }
     return NS_OK;
@@ -953,13 +943,21 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
 
 void nsTextAddress::AddLdifRowToDatabase(PRBool bIsList)
 {
-    nsIMdbRow* newRow = nsnull;
+    // If no data to process then reset CR/LF counters and return.
+    if (m_ldifLine.IsEmpty())
+    {
+      m_LFCount = 0;
+      m_CRCount = 0;
+      return;
+    }
+
+    nsCOMPtr <nsIMdbRow> newRow;
     if (m_database)
     {
         if (bIsList)
-            m_database->GetNewListRow(&newRow); 
+            m_database->GetNewListRow(getter_AddRefs(newRow)); 
         else
-            m_database->GetNewRow(&newRow); 
+            m_database->GetNewRow(getter_AddRefs(newRow)); 
 
         if (!newRow)
             return;
@@ -981,14 +979,36 @@ void nsTextAddress::AddLdifRowToDatabase(PRBool bIsList)
         else
             continue; // parse error: continue with next loop iteration
     }
-    delete [] saveCursor;
+    nsMemory::Free(saveCursor);
     m_database->AddCardRowToDB(newRow);    
 
     if (bIsList)
         m_database->AddListDirNode(newRow);
         
-    if (m_ldifLine.Length() > 0)
-        m_ldifLine.Truncate();
+    // Clear buffer for next record
+    ClearLdifRecordBuffer();
+}
+
+void nsTextAddress::ClearLdifRecordBuffer()
+{
+  if (!m_ldifLine.IsEmpty())
+  {
+      m_ldifLine.Truncate();
+      m_LFCount = 0;
+      m_CRCount = 0;
+  }
+}
+
+static void SplitCRLFAddressField(nsCString &inputAddress, nsCString &outputLine1, nsCString &outputLine2)
+{
+  PRInt32 crlfPos = inputAddress.Find("\r\n");
+  if (crlfPos != kNotFound)
+  {
+    inputAddress.Left(outputLine1, crlfPos);
+    inputAddress.Right(outputLine2, inputAddress.Length() - (crlfPos + 2));
+  }
+  else
+    outputLine1.Assign(inputAddress);
 }
 
 // We have two copies of this function in the code, one here for import and 
@@ -1000,7 +1020,8 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
   nsCAutoString colType(typeSlot);
   nsCAutoString column(valueSlot);
 
-  // 4.x exports attributes like "givenname", mozilla does "givenName" to be compliant with RFC 2798
+  // 4.x exports attributes like "givenname", 
+  // mozilla does "givenName" to be compliant with RFC 2798
   ToLowerCase(colType);
 
     mdb_u1 firstByte = (mdb_u1)(colType.get())[0];
@@ -1082,9 +1103,14 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
       if (colType.Equals("homephone") )
         m_database->AddHomePhone(newRow, column.get());
         else if (colType.Equals("homepostaladdress") )
-    m_database->AddHomeAddress(newRow, column.get());
+        {
+          nsCAutoString homeAddr1, homeAddr2;
+          SplitCRLFAddressField(column, homeAddr1, homeAddr2);
+          m_database->AddHomeAddress(newRow, homeAddr1.get());
+          m_database->AddHomeAddress2(newRow, homeAddr2.get());
+        }
       else if (colType.Equals("homeurl") )
-        m_database->AddWebPage1(newRow, column.get());
+        m_database->AddWebPage2(newRow, column.get());
       break; // 'h'
 
     case 'i':
@@ -1106,10 +1132,12 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
       if (colType.Equals("mail") )
         m_database->AddPrimaryEmail(newRow, column.get());
       else if ( colType.Equals("mobile") )
-    m_database->AddCellularNumber(newRow, column.get());
+        m_database->AddCellularNumber(newRow, column.get());
       else if (colType.Equals("member") && bIsList )
         m_database->AddLdifListMember(newRow, column.get());
-
+      // check if it starts with our magic prefix
+      else if (colType.Find(MOZ_AB_LDIF_PREFIX) == 0)
+        m_database->AddRowValue(newRow, colType, NS_ConvertUTF8toUCS2(column));
       break; // 'm'
 
     case 'n':
@@ -1134,12 +1162,13 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
       if (colType.Equals("postalcode") )
         m_database->AddWorkZipCode(newRow, column.get());
 
-      else if (colType.Equals("postofficebox") )  // is this right?
-        m_database->AddWorkAddress(newRow, column.get());
-
-    else if (colType.Equals("postaladdress") )
-        m_database->AddWorkAddress(newRow, column.get());
-
+      else if (colType.Equals("postofficebox") || colType.Equals("postaladdress")) 
+      { 
+          nsCAutoString workAddr1, workAddr2;
+          SplitCRLFAddressField(column, workAddr1, workAddr2);
+          m_database->AddWorkAddress(newRow, workAddr1.get());
+          m_database->AddWorkAddress2(newRow, workAddr2.get());
+      }
       else if (colType.Equals("pager") ||
         colType.Equals("pagerphone") )
         m_database->AddPagerNumber(newRow, column.get());
@@ -1157,8 +1186,12 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
         m_database->AddLastName(newRow, column.get());
 
       else if ( colType.Equals("streetaddress") )
-        m_database->AddWorkAddress(newRow, column.get());
-
+      {
+        nsCAutoString workAddr1, workAddr2;
+        SplitCRLFAddressField(column, workAddr1, workAddr2);
+        m_database->AddWorkAddress(newRow, workAddr1.get());
+        m_database->AddWorkAddress2(newRow, workAddr2.get());
+      }
       else if ( colType.Equals("st") )
         m_database->AddWorkState(newRow, column.get());
 
@@ -1186,7 +1219,7 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
 
     case 'w':
       if ( colType.Equals("workurl") )
-        m_database->AddWebPage2(newRow, column.get());
+        m_database->AddWebPage1(newRow, column.get());
 
       break; // 'w'
 
