@@ -41,21 +41,15 @@
 
 class nsSafeSaveFile {
     public:
-    	nsSafeSaveFile(nsIFile *aTargetFile, PRInt32 aNumBackupCopies = 0);
-    	virtual ~nsSafeSaveFile(void) {};
+    	nsSafeSaveFile() {}
+    	~nsSafeSaveFile() {}
 
-        void     CleanupFailedSave(void);
-        nsresult GetSaveFile(nsIFile **_retval);
-        nsresult PostProcessSave(void);
-        nsresult PurgeOldestBackup(void);
-
-    protected:
-    	nsSafeSaveFile(void) {};
+        nsresult Init(nsIFile *aTargetFile, nsIFile **aTempFile);
+        nsresult OnSaveFinished(PRBool aSaveSucceeded, PRBool aBackupTarget);
 
     private:
+        nsCOMPtr<nsIFile> mTargetFile;
+        PRBool            mTargetFileExists;
         nsCOMPtr<nsIFile> mTempFile;
-        nsCString         mTargetFileName; // native charset
-        PRInt32           mTargetNameLen;
-        PRInt32           mBackupCount;
 };
 
