@@ -118,85 +118,28 @@ PR_Now(void)
 static int assembleCmdLine(char *const *argv, char **cmdLine)
 {
     char *const *arg;
-    char *p, *q;
     int cmdLineSize;
-    int numBackslashes;
-    int i;
 
     /*
      * Find out how large the command line buffer should be.
      */
-    cmdLineSize = 0;
+    cmdLineSize = 1; /* final null */
     for (arg = argv+1; *arg; arg++) {
-        /*
-         * \ and " need to be escaped by a \.  In the worst case,
-         * every character is a \ or ", so the string of length
-         * may double.  If we quote an argument, that needs two ".
-         * Finally, we need a space between arguments, a null between
-         * the EXE name and the arguments, and 2 nulls at the end 
-         * of command line.
-         */
-        cmdLineSize += 2 * strlen(*arg)  /* \ and " need to be escaped */
-                + 4;                     /* space in between, or final nulls */
+        cmdLineSize += strlen(*arg) + 1; /* space in between */
     }
-    p = *cmdLine = PR_MALLOC(cmdLineSize);
-    if (p == NULL) {
+    *cmdLine = PR_MALLOC(cmdLineSize);
+    if (*cmdLine == NULL) {
         return -1;
     }
 
+    (*cmdLine)[0] = '\0';
+
     for (arg = argv+1; *arg; arg++) {
-        /* Add a space to separates the arguments */
-        if (arg > argv + 1) {
-            *p++ = ' '; 
+        if (arg > argv +1) {
+            strcat(*cmdLine, " ");
         }
-        q = *arg;
-        numBackslashes = 0;
-
-        while (*q) {
-            if (*q == '\\') {
-                numBackslashes++;
-                q++;
-            } else if (*q == '"') {
-                if (numBackslashes) {
-                    /*
-                     * Double the backslashes since they are followed
-                     * by a quote
-                     */
-                    for (i = 0; i < 2 * numBackslashes; i++) {
-                        *p++ = '\\';
-                    }
-                    numBackslashes = 0;
-                }
-                /* To escape the quote */
-                *p++ = '\\';
-                *p++ = *q++;
-            } else {
-                if (numBackslashes) {
-                    /*
-                     * Backslashes are not followed by a quote, so
-                     * don't need to double the backslashes.
-                     */
-                    for (i = 0; i < numBackslashes; i++) {
-                        *p++ = '\\';
-                    }
-                    numBackslashes = 0;
-                }
-                *p++ = *q++;
-            }
-        }
-
-        /* Now we are at the end of this argument */
-        if (numBackslashes) {
-            for (i = 0; i < numBackslashes; i++) {
-                *p++ = '\\';
-            }
-        }
-        if(arg == argv)
-           *p++ = ' ';
+        strcat(*cmdLine, *arg);
     } 
-
-    /* Add a null at the end */
-    *p = '\0';
     return 0;
 }
 
@@ -295,7 +238,7 @@ PRProcess * _PR_CreateOS2Process(
 {
     PRProcess *proc = NULL;
     char *cmdLine = NULL;
-    char **newEnvp;
+    char **newEnvp = NULL;
     char *envBlock = NULL;
    
     STARTDATA startData = {0};
@@ -380,7 +323,7 @@ PRProcess * _PR_CreateOS2Process(
         strcpy(pszFormatString, "/3 %s %s");
     }
  
-    startData.InheritOpt = SSF_INHERTOPT_PARENT;
+    startData.InheritOpt = SSF_INHERTOPT_SHELL;
  
     if (pszEXEName[0]) {
         pszFormatResult = PR_MALLOC(strlen(pszFormatString)+strlen(path)+strlen(cmdLine));
@@ -437,12 +380,9 @@ errorExit:
 
 PRStatus _PR_DetachOS2Process(PRProcess *process)
 {
-    /* This is basically what they did on Windows (CloseHandle)
-     * but I don't think it will do much on OS/2. A process is
-     * either created as a child or not.  You can't 'detach' it
-     * later on.
+    /* On OS/2, a process is either created as a child or not. 
+     * You can't 'detach' it later on.
      */
-    DosClose(process->md.pid);
     PR_DELETE(process);
     return PR_SUCCESS;
 }
@@ -509,14 +449,12 @@ _PR_MD_WAKEUP_CPUS( void )
 
 PRStatus _MD_CreateFileMap(PRFileMap *fmap, PRInt64 size)
 {
-    PR_ASSERT(!"Not implemented");
     PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0);
     return PR_FAILURE;
 }
 
 PRInt32 _MD_GetMemMapAlignment(void)
 {
-    PR_ASSERT(!"Not implemented");
     PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0);
     return -1;
 }
@@ -526,21 +464,18 @@ void * _MD_MemMap(
     PROffset64 offset,
     PRUint32 len)
 {
-    PR_ASSERT(!"Not implemented");
     PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0);
     return NULL;
 }
 
 PRStatus _MD_MemUnmap(void *addr, PRUint32 len)
 {
-    PR_ASSERT(!"Not implemented");
     PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0);
     return PR_FAILURE;
 }
 
 PRStatus _MD_CloseFileMap(PRFileMap *fmap)
 {
-    PR_ASSERT(!"Not implemented");
     PR_SetError(PR_NOT_IMPLEMENTED_ERROR, 0);
     return PR_FAILURE;
 }
@@ -580,43 +515,3 @@ unsigned long _System _DLL_InitTerm( unsigned long mod_handle, unsigned long fla
    return rc;
 }
 
-#ifndef XP_OS2_VACPP
-
-PRInt32 _PR_MD_ATOMIC_SET(PRInt32 *intp, PRInt32 val)
-{
-  PRInt32 result;
-  asm volatile ("lock ; xchg %0, %1" 
-                : "=r"(result), "=m"(intp)
-                : "0"(val), "m"(intp));
-  return result;
-}
-
-PRInt32 _PR_MD_ATOMIC_ADD(PRInt32 *intp, PRInt32 val)
-{
-  PRInt32 result;
-  asm volatile ("lock ; xadd %0, %1" 
-                : "=r"(result), "=m"(intp)
-                : "0"(val), "m"(intp));
-  return result + val;
-}
-
-PRInt32 _PR_MD_ATOMIC_INCREMENT(PRInt32 *val)
-{    
-  PRInt32 result;
-  asm volatile ("lock ; xadd %0, %1" 
-                : "=r"(result), "=m"(*val)
-                : "0"(1), "m"(*val));
-  return result + 1;
-}
-
-PRInt32 _PR_MD_ATOMIC_DECREMENT(PRInt32 *val)
-{
-  PRInt32 result;
-  asm volatile ("lock ; xadd %0, %1" 
-                : "=r"(result), "=m"(*val)
-                : "0"(1), "m"(*val));
-  return result - 1;
-}
-
-#endif
-
