@@ -116,7 +116,7 @@ static PRIntervalTime pr_PredictNextNotifyTime(PRAlarmID *id)
     id->lastNotify = id->nextNotify;  /* just keeping track of things */
     id->nextNotify = (PRIntervalTime)(offsetFromEpoch + 0.5);
 
-    delta = id->nextNotify - id->nextNotify;
+    delta = id->nextNotify - id->lastNotify;
     return delta;
 }  /* pr_PredictNextNotifyTime */
 
