@@ -34,7 +34,7 @@
 /*
  * Test program for client-side OCSP.
  *
- * $Id: ocspclnt.c,v 1.2.16.1 2002/04/10 03:25:29 cltbld%netscape.com Exp $
+ * $Id: ocspclnt.c,v 1.5 2003/04/01 18:55:16 bishakhabanerjee%netscape.com Exp $
  */
 
 #include "secutil.h"
@@ -1211,7 +1211,9 @@ nssdone:
 	(void) CERT_DisableOCSPChecking (handle);
     }
 
-    NSS_Shutdown ();
+    if (NSS_Shutdown () != SECSuccess) {
+	exit(1);
+    }
 
 prdone:
     PR_Cleanup ();
