@@ -209,6 +209,7 @@ usage (void)
   PR_fprintf(outputFD, "    -m\"metafile\"\t\tinclude custom meta-information\n");
   PR_fprintf(outputFD, "    --norecurse\t\t\tdo not operate on subdirectories\n");
   PR_fprintf(outputFD, "    -o\t\t\t\toptimize - omit optional headers\n");
+  PR_fprintf(outputFD, "    -O\t\t\t\tenableOCSP - enable OCSP checking\n");
   PR_fprintf(outputFD, "    --outfile \"filename\"\tredirect output to file\n");
   PR_fprintf(outputFD, "    -p\"password\"\t\tfor password on command line (insecure)\n");
   PR_fprintf(outputFD, "    -s keysize\t\t\tkeysize in bits of generated cert\n");
@@ -285,7 +286,15 @@ VerifyCertDir(char *dir, char *keyName)
 {
   char fn [FNSIZE];
 
-  sprintf (fn, "%s/cert7.db", dir);
+  /* don't try verifying if we don't have a local directory */
+  if (strncmp(dir,"multiaccess:",sizeof("multiaccess:")-1) == 0) {
+     return;
+  }
+
+  /* This code is really broken because it makes underlying assumptions about
+   * how the NSS profile directory is laid out, but these names can change
+   * from release to release. */
+  sprintf (fn, "%s/cert8.db", dir);
 
   if (PR_Access (fn, PR_ACCESS_EXISTS))
     {
@@ -460,6 +469,9 @@ char *
 pk11_password_hardcode(PK11SlotInfo *slot, PRBool retry, void *arg)
 {
   char *pw;
+  if (retry) {
+    return NULL; /* the password is incorrect, fail */
+  }
   pw = password ? PORT_Strdup (password) : NULL;
   /* XXX don't do this, or FIPS won't work */
   /*password = NULL;*/
@@ -605,7 +617,7 @@ displayVerifyLog(CERTVerifyLog *log)
 		/* Get a name for this cert */
 		if(cert->nickname != NULL) {
 			name = cert->nickname;
-		} else if(cert->emailAddr != NULL) {
+		} else if(cert->emailAddr && cert->emailAddr[0]) {
 			name = cert->emailAddr;
 		} else {
 			name = cert->subjectName;
@@ -724,7 +736,7 @@ chop(char *str)
 		}
 
 		/* Nip trailing whitespace */
-		if(strlen(start) > 0) {
+		if(*start) {
 			end = start + strlen(start) - 1;
 			while(isspace(*end) && end > start) {
 				end--;
@@ -801,8 +813,10 @@ InitCrypto(char *cert_dir, PRBool readOnly)
 			  NULL /*wincx*/) != SECSuccess) {
 				fprintf(stderr, "%s: Unable to authenticate to %s.\n",
 					PROGRAM_NAME, PK11_GetSlotName(slotinfo));
+				PK11_FreeSlot(slotinfo);
 				return -1;
 			}
+			PK11_FreeSlot(slotinfo);
 		}
 
 		/* Make sure there is a password set on the internal key slot */
@@ -825,9 +839,11 @@ InitCrypto(char *cert_dir, PRBool readOnly)
 			  NULL /*wincx*/) != SECSuccess) {
 				fprintf(stderr, "%s: Unable to authenticate to %s.\n",
 					PROGRAM_NAME, PK11_GetSlotName(slotinfo));
+				PK11_FreeSlot(slotinfo);
 				return -1;
 			}
 		}
+		PK11_FreeSlot(slotinfo);
 	}
 
 	return 0;
