@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.3 $ $Date: 2001/10/08 19:26:01 $ $Name:  $";
+static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.5 $ $Date: 2002/04/04 16:14:26 $ $Name: MOZILLA_1_7_3_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -52,6 +52,8 @@ static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.3 $ $Date:
 #include "prlock.h"
 #include "plarena.h"
 
+#include <string.h>
+
 /*
  * NSSArena
  *
@@ -1125,14 +1127,15 @@ nss_ZRealloc
     new_h = (struct pointer_header *)p;
     new_h->arena = h->arena;
     new_h->size = newSize;
-    rv = (void *)((char *)h + sizeof(struct pointer_header));
-    (void)nsslibc_memcpy(rv, pointer, h->size);
-    (void)nsslibc_memset(&((char *)rv)[ h->size ], 0, 
-                         (newSize - h->size));
-    (void)nsslibc_memset(pointer, 0, h->size);
+    rv = (void *)((char *)new_h + sizeof(struct pointer_header));
+    if (rv != pointer) {
+	(void)nsslibc_memcpy(rv, pointer, h->size);
+	(void)nsslibc_memset(pointer, 0, h->size);
+    }
+    (void)nsslibc_memset(&((char *)rv)[ h->size ], 0, (newSize - h->size));
     h->arena = (NSSArena *)NULL;
     h->size = 0;
-    PR_Unlock(h->arena->lock);
+    PR_Unlock(new_h->arena->lock);
     return rv;
   }
   /*NOTREACHED*/
