@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: wrap.c,v $ $Revision: 1.6.6.1 $ $Date: 2002/04/10 03:26:46 $ $Name: MOZILLA_1_0_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: wrap.c,v $ $Revision: 1.9 $ $Date: 2002/10/31 22:02:10 $ $Name: MOZILLA_1_7_3_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -2500,7 +2500,6 @@ NSSCKFWC_FindObjects
   NSSCKFWSession *fwSession;
   NSSCKFWFindObjects *fwFindObjects;
   CK_ULONG i;
-  NSSArena *arena;
 
   if( (NSSCKFWInstance *)NULL == fwInstance ) {
     error = CKR_CRYPTOKI_NOT_INITIALIZED;
@@ -2513,13 +2512,6 @@ NSSCKFWC_FindObjects
     goto loser;
   }
 
-#ifdef notdef
-  arena = nssCKFWSession_GetArena(fwSession, &error);
-  if( (NSSArena *)NULL == arena ) {
-    goto loser;
-  }
-#endif
-
   if( (CK_OBJECT_HANDLE_PTR)CK_NULL_PTR == phObject ) {
     error = CKR_ARGUMENTS_BAD;
     goto loser;
