@@ -34,7 +34,7 @@
 /*
  * PKCS7 decoding, verification.
  *
- * $Id: p7decode.c,v 1.7 2001/12/07 01:36:06 relyea%netscape.com Exp $
+ * $Id: p7decode.c,v 1.15 2004/01/07 23:07:23 jpierre%netscape.com Exp $
  */
 
 #include "nssrenam.h"
@@ -263,10 +263,6 @@ static SECStatus
 sec_pkcs7_decoder_start_digests (SEC_PKCS7DecoderContext *p7dcx, int depth,
 				 SECAlgorithmID **digestalgs)
 {
-    SECAlgorithmID *algid;
-    SECOidData *oiddata;
-    const SECHashObject *digobj;
-    void *digcx;
     int i, digcnt;
 
     if (digestalgs == NULL)
@@ -281,11 +277,8 @@ sec_pkcs7_decoder_start_digests (SEC_PKCS7DecoderContext *p7dcx, int depth,
 
     /*
      * No algorithms means no work to do.
-     * This is not expected, so cause an assert.
-     * But if it does happen, just act as if there were
-     * no algorithms specified.
+     * Just act as if there were no algorithms specified.
      */
-    PORT_Assert (digcnt != 0);
     if (digcnt == 0)
 	return SECSuccess;
 
@@ -305,26 +298,10 @@ sec_pkcs7_decoder_start_digests (SEC_PKCS7DecoderContext *p7dcx, int depth,
      * Create a digest context for each algorithm.
      */
     for (i = 0; i < digcnt; i++) {
-	algid = digestalgs[i];
-	oiddata = SECOID_FindOID(&(algid->algorithm));
-	if (oiddata == NULL) {
-	    digobj = NULL;
-	} else {
-	    switch (oiddata->offset) {
-	      case SEC_OID_MD2:
-		digobj = HASH_GetHashObject(HASH_AlgMD2);
-		break;
-	      case SEC_OID_MD5:
-		digobj = HASH_GetHashObject(HASH_AlgMD5);
-		break;
-	      case SEC_OID_SHA1:
-		digobj = HASH_GetHashObject(HASH_AlgSHA1);
-		break;
-	      default:
-		digobj = NULL;
-		break;
-	    }
-	}
+	SECAlgorithmID *     algid  = digestalgs[i];
+	SECOidTag            oidTag = SECOID_FindOIDTag(&(algid->algorithm));
+	const SECHashObject *digobj = HASH_GetHashObjectByOidTag(oidTag);
+	void *digcx;
 
 	/*
 	 * Skip any algorithm we do not even recognize; obviously,
@@ -796,6 +773,7 @@ sec_pkcs7_decoder_finish_decrypt (SEC_PKCS7DecoderContext *p7dcx,
      * All done, destroy it.
      */
     sec_PKCS7DestroyDecryptObject (worker->decryptobj);
+    worker->decryptobj = NULL;
 
     return SECSuccess;
 }
@@ -1245,6 +1223,10 @@ SEC_PKCS7DecoderFinish(SEC_PKCS7DecoderContext *p7dcx)
 	    cinfo = NULL;
 	}
     }
+    /* free any NSS data structures */
+    if (p7dcx->worker.decryptobj) {
+        sec_PKCS7DestroyDecryptObject (p7dcx->worker.decryptobj);
+    }
     PORT_FreeArena (p7dcx->tmp_poolp, PR_FALSE);
     PORT_Free (p7dcx);
     return cinfo;
@@ -1267,6 +1249,16 @@ SEC_PKCS7DecodeItem(SECItem *p7item,
     return SEC_PKCS7DecoderFinish(p7dcx);
 }
 
+/*
+ * Abort the ASN.1 stream. Used by pkcs 12
+ */
+void
+SEC_PKCS7DecoderAbort(SEC_PKCS7DecoderContext *p7dcx, int error)
+{
+    PORT_Assert(p7dcx);
+    SEC_ASN1DecoderAbort(p7dcx->dcx, error);
+}
+
 
 /*
  * If the thing contains any certs or crls return true; false otherwise.
@@ -1455,7 +1447,7 @@ sec_pkcs7_verify_signature(SEC_PKCS7ContentInfo *cinfo,
     SECKEYPublicKey *publickey;
     SECItem *content_type;
     PK11SymKey *sigkey;
-    SECItem *utc_stime;
+    SECItem *encoded_stime;
     int64 stime;
     SECStatus rv;
 
@@ -1579,10 +1571,10 @@ sec_pkcs7_verify_signature(SEC_PKCS7ContentInfo *cinfo,
      * both on the cert verification and for importing the sender
      * email profile.
      */
-    utc_stime = SEC_PKCS7GetSigningTime (cinfo);
-    if (utc_stime != NULL) {
-	if (DER_UTCTimeToTime (&stime, utc_stime) != SECSuccess)
-	    utc_stime = NULL;	/* conversion failed, so pretend none */
+    encoded_stime = SEC_PKCS7GetSigningTime (cinfo);
+    if (encoded_stime != NULL) {
+	if (DER_DecodeTimeChoice (&stime, encoded_stime) != SECSuccess)
+	    encoded_stime = NULL;	/* conversion failed, so pretend none */
     }
 
     /*
@@ -1594,7 +1586,7 @@ sec_pkcs7_verify_signature(SEC_PKCS7ContentInfo *cinfo,
      * maybe make them pass in the current time, always?).
      */
     if (CERT_VerifyCert (certdb, cert, PR_TRUE, certusage,
-			 utc_stime != NULL ? stime : PR_Now(),
+			 encoded_stime != NULL ? stime : PR_Now(),
 			 cinfo->pwfn_arg, NULL) != SECSuccess)
 	{
 	/*
@@ -1632,32 +1624,14 @@ sec_pkcs7_verify_signature(SEC_PKCS7ContentInfo *cinfo,
     algiddata = SECOID_FindOID (&(signerinfo->digestAlg.algorithm));
 
     if (detached_digest != NULL) {
-	switch (digest_type) {
-	  default:
-	  case HASH_AlgNULL:
+	HASH_HashType found_type = HASH_GetHashTypeByOidTag(algiddata->offset);
+	unsigned int hashLen     = HASH_ResultLen(digest_type);
+
+	if (digest_type != found_type || 
+	    digest_type == HASH_AlgNULL || 
+	    detached_digest->len != hashLen) {
 	    PORT_SetError (SEC_ERROR_PKCS7_BAD_SIGNATURE);
 	    goto done;
-	  case HASH_AlgMD2:
-	    PORT_Assert (detached_digest->len == MD2_LENGTH);
-	    if (algiddata->offset != SEC_OID_MD2) {
-		PORT_SetError (SEC_ERROR_PKCS7_BAD_SIGNATURE);
-		goto done;
-	    }
-	    break;
-	  case HASH_AlgMD5:
-	    PORT_Assert (detached_digest->len == MD5_LENGTH);
-	    if (algiddata->offset != SEC_OID_MD5) {
-		PORT_SetError (SEC_ERROR_PKCS7_BAD_SIGNATURE);
-		goto done;
-	    }
-	    break;
-	  case HASH_AlgSHA1:
-	    PORT_Assert (detached_digest->len == SHA1_LENGTH);
-	    if (algiddata->offset != SEC_OID_SHA1) {
-		PORT_SetError (SEC_ERROR_PKCS7_BAD_SIGNATURE);
-		goto done;
-	    }
-	    break;
 	}
 	digest = detached_digest;
     } else {
@@ -1816,8 +1790,8 @@ sec_pkcs7_verify_signature(SEC_PKCS7ContentInfo *cinfo,
 
 	    rv = sec_PKCS7Decrypt (decryptobj, holder.data, &holder.len, buflen,
 				   sig->data, sig->len, PR_TRUE);
+	    sec_PKCS7DestroyDecryptObject (decryptobj);
 	    if (rv != SECSuccess) {
-		sec_PKCS7DestroyDecryptObject (decryptobj);
 		goto done;
 	    }
 
@@ -1861,7 +1835,7 @@ savecert:
      * Only save the smime profile if we are checking an email message and
      * the cert has an email address in it.
      */
-    if ( ( cert->emailAddr != NULL ) &&
+    if ( cert->emailAddr && cert->emailAddr[0] &&
 	( ( certusage == certUsageEmailSigner ) ||
 	 ( certusage == certUsageEmailRecipient ) ) ) {
 	SECItem *profile = NULL;
@@ -1886,7 +1860,7 @@ savecert:
 	    profile = sec_PKCS7AttributeValue (attr);
 	}
 
-	rv = CERT_SaveSMimeProfile (cert, profile, utc_stime);
+	rv = CERT_SaveSMimeProfile (cert, profile, encoded_stime);
 
 	/*
 	 * Restore the saved error in case the calls above set a new
@@ -2031,7 +2005,7 @@ sec_pkcs7_get_signer_cert_info(SEC_PKCS7ContentInfo *cinfo, int selector)
 	container = CERT_GetCommonName (&signercert->subject);
 	break;
       case sec_email_address:
-	if(signercert->emailAddr) {
+	if(signercert->emailAddr && signercert->emailAddr[0]) {
 	    container = PORT_Strdup(signercert->emailAddr);
 	} else {
 	    container = NULL;
