@@ -37,7 +37,7 @@
 
 #ifndef nsViewManager_h___
 #define nsViewManager_h___
-
+#include "nsCOMPtr.h"
 #include "nsIViewManager.h"
 #include "nsCRT.h"
 #include "nsIPresContext.h"
@@ -50,6 +50,7 @@
 #include "nsIScrollableView.h"
 #include "nsIRegion.h"
 #include "nsIBlender.h"
+#include "nsIEventQueueService.h"
 #include "nsIEventQueue.h"
 #include "nsView.h"
 
@@ -59,6 +60,7 @@ class nsIPresContext;
 class nsISupportsArray;
 struct DisplayListElement2;
 struct DisplayZTreeNode;
+class BlendingBuffers;
 
 //Uncomment the following line to enable generation of viewmanager performance data.
 #ifdef MOZ_PERF_METRICS
@@ -69,10 +71,6 @@ struct DisplayZTreeNode;
 #include "nsTimer.h"
 #endif
 
-// compositor per-view flags
-#define IS_PARENT_OF_REFRESHED_VIEW  0x00000001
-#define IS_Z_PLACEHOLDER_VIEW        0x80000000
-
 /**
    FIXED-POSITION FRAMES AND Z-ORDERING
 
@@ -102,10 +100,9 @@ public:
 
   void RemoveReparentedView() { mReparentedView = nsnull; }
   void SetReparentedView(nsView* aView) { mReparentedView = aView; }
-  nsView* GetReparentedView() { return mReparentedView; }
+  nsView* GetReparentedView() const { return mReparentedView; }
 
-  NS_IMETHOD GetCompositorFlags(PRUint32 *aFlags)
-  { nsView::GetCompositorFlags(aFlags); *aFlags |= IS_Z_PLACEHOLDER_VIEW; return NS_OK; }
+  virtual PRBool IsZPlaceholderView() const { return PR_TRUE; }
 
 protected:
   virtual ~nsZPlaceholderView() {
@@ -144,14 +141,6 @@ public:
   NS_IMETHOD  UpdateView(nsIView *aView, PRUint32 aUpdateFlags);
   NS_IMETHOD  UpdateView(nsIView *aView, const nsRect &aRect, PRUint32 aUpdateFlags);
   NS_IMETHOD  UpdateAllViews(PRUint32 aUpdateFlags);
-  /**
-   * Called to inform the view manager that a view has scrolled.
-   * The view manager will invalidate any widgets which may need
-   * to be rerendered.
-   * @param aView view to paint. should be root view
-   * @param aUpdateFlags see bottom of nsIViewManager.h for description
-   */
-  NS_IMETHOD  UpdateViewAfterScroll(nsIView *aView, PRInt32 aDX, PRInt32 aDY);
 
   NS_IMETHOD  DispatchEvent(nsGUIEvent *aEvent, nsEventStatus* aStatus);
 
@@ -178,7 +167,7 @@ public:
 
   NS_IMETHOD  ResizeView(nsIView *aView, const nsRect &aRect, PRBool aRepaintExposedAreaOnly = PR_FALSE);
 
-  NS_IMETHOD  SetViewChildClipRegion(nsIView *aView, nsIRegion *aRegion);
+  NS_IMETHOD  SetViewChildClipRegion(nsIView *aView, const nsRegion *aRegion);
 
   NS_IMETHOD  SetViewBitBltEnabled(nsIView *aView, PRBool aEnable);
 
@@ -188,8 +177,7 @@ public:
 
   NS_IMETHOD  SetViewVisibility(nsIView *aView, nsViewVisibility aVisible);
 
-  NS_IMETHOD  SetViewZIndex(nsIView *aView, PRBool aAuto, PRInt32 aZIndex);
-
+  NS_IMETHOD  SetViewZIndex(nsIView *aView, PRBool aAuto, PRInt32 aZIndex, PRBool aTopMost=PR_FALSE);
   NS_IMETHOD  SetViewContentTransparency(nsIView *aView, PRBool aTransparent);
   NS_IMETHOD  SetViewOpacity(nsIView *aView, float aOpacity);
 
@@ -198,10 +186,6 @@ public:
 
   NS_IMETHOD  GetDeviceContext(nsIDeviceContext *&aContext);
 
-  NS_IMETHOD  ShowQuality(PRBool aShow);
-  NS_IMETHOD  GetShowQuality(PRBool &aResult);
-  NS_IMETHOD  SetQuality(nsContentQuality aQuality);
-
   NS_IMETHOD  DisableRefresh(void);
   NS_IMETHOD  EnableRefresh(PRUint32 aUpdateFlags);
 
@@ -218,6 +202,7 @@ public:
 
   NS_IMETHOD GetWidgetForView(nsIView *aView, nsIWidget **aWidget);
   NS_IMETHOD GetWidget(nsIWidget **aWidget);
+  nsIWidget* GetWidget() { return mRootWindow; }
   NS_IMETHOD ForceUpdate();
  
   NS_IMETHOD IsCachingWidgetChanges(PRBool* aCaching);
@@ -228,78 +213,96 @@ public:
   NS_IMETHOD SetDefaultBackgroundColor(nscolor aColor);
   NS_IMETHOD GetDefaultBackgroundColor(nscolor* aColor);
   NS_IMETHOD GetLastUserEventTime(PRUint32& aTime);
-  nsresult ProcessInvalidateEvent();
+  void ProcessInvalidateEvent();
   static PRInt32 GetViewManagerCount();
   static const nsVoidArray* GetViewManagerArray();
   static PRUint32 gLastUserEventTime;
 
   /**
-	 * Determine if a rectangle specified in the view's coordinate system 
-	 * is completely, or partially visible.
+   * Determine if a rectangle specified in the view's coordinate system 
+   * is completely, or partially visible.
    * @param aView view that aRect coordinates are specified relative to
    * @param aRect rectangle in twips to test for visibility 
-   * @param aMinTwips is the min. pixel rows or cols at edge of screen needed for object to be counted visible
-   * @param aIsVisible returns PR_TRUE if the rect is visible, PR_FALSE otherwise.
-	 */
-  NS_IMETHOD IsRectVisible(nsIView *aView, const nsRect &aRect, PRUint16 aMinTwips, PRBool *aIsVisible);
+   * @param aMinTwips is the min. pixel rows or cols at edge of screen 
+   *                  needed for object to be counted visible
+   * @param aRectVisibility returns eVisible if the rect is visible, 
+   *                        otherwise it returns an enum indicating why not
+   */
+  NS_IMETHOD GetRectVisibility(nsIView *aView, const nsRect &aRect, 
+                               PRUint16 aMinTwips, 
+                               nsRectVisibility *aRectVisibility);
 
 protected:
   virtual ~nsViewManager();
   void ProcessPendingUpdates(nsView *aView);
 
 private:
+  void ReparentChildWidgets(nsIView* aView, nsIWidget *aNewWidget);
+  void ReparentWidgets(nsIView* aView, nsIView *aParent);
   nsIRenderingContext *CreateRenderingContext(nsView &aView);
   void AddRectToDirtyRegion(nsView* aView, const nsRect &aRect) const;
-  void UpdateTransCnt(nsView *oldview, nsView *newview);
 
-  PRBool UpdateAllCoveringWidgets(nsView *aView, nsView *aTarget, nsRect &aDamagedRect, PRBool aOnlyRepaintIfUnblittable);
+  PRBool UpdateWidgetArea(nsView *aWidgetView, const nsRect &aDamagedRect, nsView* aIgnoreWidgetView);
 
   void UpdateViews(nsView *aView, PRUint32 aUpdateFlags);
 
   void Refresh(nsView *aView, nsIRenderingContext *aContext,
                nsIRegion *region, PRUint32 aUpdateFlags);
   void DefaultRefresh(nsView* aView, const nsRect* aRect);
-  void RenderViews(nsView *aRootView, nsIRenderingContext& aRC, const nsRect& aRect,
-                   PRBool &aResult);
+  PRBool BuildRenderingDisplayList(nsIView* aRootView,
+    const nsRegion& aRegion, nsVoidArray* aDisplayList);
+  void RenderViews(nsView *aRootView, nsIRenderingContext& aRC,
+                   const nsRegion& aRegion, nsDrawingSurface aRCSurface,
+                   const nsVoidArray& aDisplayList);
 
-  void RenderDisplayListElement(DisplayListElement2* element, nsIRenderingContext &aRC);
+  void RenderDisplayListElement(DisplayListElement2* element,
+                                nsIRenderingContext* aRC);
 
   void PaintView(nsView *aView, nsIRenderingContext &aRC, nscoord x, nscoord y,
-                const nsRect &aDamageRect);
+                 const nsRect &aDamageRect);
 
   void InvalidateRectDifference(nsView *aView, const nsRect& aRect, const nsRect& aCutOut, PRUint32 aUpdateFlags);
   void InvalidateHorizontalBandDifference(nsView *aView, const nsRect& aRect, const nsRect& aCutOut,
                                           PRUint32 aUpdateFlags, nscoord aY1, nscoord aY2, PRBool aInCutOut);
 
-  nsresult CreateBlendingBuffers(nsIRenderingContext &aRC);
-  
-  void BuildDisplayList(nsView* aView, const nsRect& aRect, PRBool aEventProcessing, PRBool aCaptured);
-  void BuildEventTargetList(nsAutoVoidArray &aTargets, nsView* aView, nsGUIEvent* aEvent, PRBool aCaptured);
+  BlendingBuffers* CreateBlendingBuffers(nsIRenderingContext *aRC, PRBool aBorrowContext,
+                                         nsDrawingSurface aBorrowSurface, PRBool aNeedAlpha,
+                                         const nsRect& aArea);
 
-  PRBool CreateDisplayList(nsView *aView, PRBool aReparentedViewsPresent, DisplayZTreeNode* &aResult, nscoord aOriginX, nscoord aOriginY,
-                           PRBool aInsideRealView, nsView *aRealView, const nsRect *aDamageRect,
-                           nsView *aTopView, nscoord aX, nscoord aY, PRBool aPaintFloaters, PRBool aEventProcessing);
-  PRBool AddToDisplayList(nsView *aView, DisplayZTreeNode* &aParent, nsRect &aClipRect,
-    nsRect& aDirtyRect, PRUint32 aFlags, nscoord aAbsX, nscoord aAbsY, PRBool aAssumeIntersection);
-  void ReapplyClipInstructions(PRBool aHaveClip, nsRect& aClipRect, PRInt32& aIndex);
-  nsresult OptimizeDisplayList(const nsRect& aDamageRect, nsRect& aFinalTransparentRect);
-    // Remove redundant PUSH/POP_CLIP pairs.
-  void ComputeViewOffset(nsView *aView, nsPoint *aOrigin);
+  void ReparentViews(DisplayZTreeNode* aNode);
+  void BuildDisplayList(nsView* aView, const nsRect& aRect, PRBool aEventProcessing,
+                        PRBool aCaptured, nsVoidArray* aDisplayList);
+  void BuildEventTargetList(nsVoidArray &aTargets, nsView* aView, nsGUIEvent* aEvent, PRBool aCaptured);
 
-  void AddCoveringWidgetsToOpaqueRegion(nsIRegion* aRgn, nsIDeviceContext* aContext,
+  PRBool CreateDisplayList(nsView *aView,
+                           PRBool aReparentedViewsPresent, DisplayZTreeNode* &aResult,
+                           nscoord aOriginX, nscoord aOriginY,
+                           nsView *aRealView, const nsRect *aDamageRect,
+                           nsView *aTopView, nscoord aX, nscoord aY,
+                           PRBool aPaintFloats, PRBool aEventProcessing);
+  PRBool AddToDisplayList(nsView *aView,
+                          DisplayZTreeNode* &aParent, nsRect &aClipRect,
+                          nsRect& aDirtyRect, PRUint32 aFlags, nscoord aAbsX, nscoord aAbsY,
+                          PRBool aAssumeIntersection);
+  void OptimizeDisplayList(const nsVoidArray* aDisplayList, const nsRegion& aDirtyRegion,
+                           nsRect& aFinalTransparentRect, nsRegion& aOpaqueRgn,
+                           PRBool aTreatUniformAsOpaque);
+
+  void AddCoveringWidgetsToOpaqueRegion(nsRegion &aRgn, nsIDeviceContext* aContext,
                                         nsView* aRootView);
 
   // Predicates
   PRBool DoesViewHaveNativeWidget(nsView* aView);
-  PRBool IsClipView(nsView* aView);
 
   void PauseTimer(void);
   void RestartTimer(void);
-  void OptimizeDisplayListClipping(PRBool aHaveClip, nsRect& aClipRect, PRInt32& aIndex,
+  void OptimizeDisplayListClipping(const nsVoidArray* aDisplayList, PRBool aHaveClip,
+                                   nsRect& aClipRect, PRInt32& aIndex,
                                    PRBool& aAnyRendered);
-#ifdef NS_DEBUG
-	void ShowDisplayList(PRInt32 flatlen);
-#endif
+  nsRect OptimizeTranslucentRegions(const nsVoidArray& aDisplayList,
+                                    PRInt32* aIndex, nsRegion* aOpaqueRegion);
+
+  void ShowDisplayList(const nsVoidArray* aDisplayList);
 
   // Utilities
 
@@ -357,16 +360,30 @@ public: // NOT in nsIViewManager, so private to the view module
 
   nsEventStatus HandleEvent(nsView* aView, nsGUIEvent* aEvent, PRBool aCaptured);
 
+  /**
+   * Called to inform the view manager that a view has scrolled.
+   * The view manager will invalidate any widgets which may need
+   * to be rerendered.
+   * @param aView view to paint. should be root view
+   * @param aUpdateFlags see bottom of nsIViewManager.h for description
+   */
+  void UpdateViewAfterScroll(nsIView *aView, PRInt32 aDX, PRInt32 aDY);
+
   PRBool CanScrollWithBitBlt(nsView* aView);
 
+  nsresult CreateRegion(nsIRegion* *result);
+
+  // return the sum of all view offsets from aView right up to the
+  // root of this view hierarchy (the view with no parent, which might
+  // not be in this view manager).
+  static nsPoint ComputeViewOffset(const nsView *aView);
+
 private:
   nsIDeviceContext  *mContext;
   float             mTwipsToPixels;
   float             mPixelsToTwips;
   nsIViewObserver   *mObserver;
   nsIWidget         *mRootWindow;
-  PRIntervalTime    mLastRefresh;
-  PRInt32           mTransCnt;
   PRBool            mRefreshEnabled;
   PRBool            mPainting;
   PRBool            mRecursiveRefreshPending;
@@ -374,10 +391,6 @@ private:
   nsView            *mKeyGrabber;
   PRInt32           mUpdateCnt;
   PRInt32           mUpdateBatchCnt;
-  PRInt32           mDisplayListCount;
-  nsAutoVoidArray   mDisplayList;
-  PRInt32           mTranslucentViewCount;
-  nsRect            mTranslucentArea;       // bounding box of all translucent views.
   nsIScrollableView *mRootScrollable;
   PRInt32           mCachingWidgetChanges;
   nscolor           mDefaultBackgroundColor;
@@ -387,32 +400,17 @@ private:
   //from here to public should be static and locked... MMP
   static PRInt32           mVMCount;        //number of viewmanagers
 
-  //blending buffers
-  static nsDrawingSurface  gOffScreen;
-  static nsDrawingSurface  gBlack;
-  static nsDrawingSurface  gWhite;
-  static nsSize            gOffScreenSize;
-
   //Rendering context used to cleanup the blending buffers
   static nsIRenderingContext* gCleanupContext;
 
   //list of view managers
   static nsVoidArray       *gViewManagers;
 
-  //compositor regions
-  nsIRegion         *mOpaqueRgn;
-  nsIRegion         *mTmpRgn;
-
-  nsIBlender        *mBlender;
-
-  nsIRenderingContext *mOffScreenCX;
-  nsIRenderingContext *mBlackCX;
-  nsIRenderingContext *mWhiteCX;
+  nsCOMPtr<nsIBlender> mBlender;
 
   nsISupportsArray  *mCompositeListeners;
   void DestroyZTreeNode(DisplayZTreeNode* aNode);
 
-  nsresult CreateRegion(nsIRegion* *result);
   nsCOMPtr<nsIFactory> mRegionFactory;
 protected:
   nsView            *mRootView;
@@ -420,8 +418,8 @@ protected:
   nscoord           mY;
   PRBool            mAllowDoubleBuffering;
   PRBool            mHasPendingInvalidates;
-  PRBool            mPendingInvalidateEvent;
-  nsCOMPtr<nsIEventQueue>  mEventQueue;
+  nsCOMPtr<nsIEventQueueService>  mEventQueueService;
+  nsCOMPtr<nsIEventQueue>         mInvalidateEventQueue;
   void PostInvalidateEvent();
 
 #ifdef NS_VM_PERF_METRICS
