@@ -146,7 +146,11 @@ static nsEventStatus
 HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta,
                      Point inMouseLoc, nsIWidget* inWidget )
 {
-  nsMouseScrollEvent scrollEvent;
+  NS_ASSERTION(inWidget, "HandleScrollEvent doesn't work with a null widget");
+  if (!inWidget)
+    return nsEventStatus_eIgnore;
+
+  nsMouseScrollEvent scrollEvent(NS_MOUSE_SCROLL, inWidget);
   
   scrollEvent.scrollFlags = 
     (inAxis == kEventMouseWheelAxisX) ? nsMouseScrollEvent::kIsHorizontal : nsMouseScrollEvent::kIsVertical;
@@ -162,18 +166,10 @@ HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta
   inWidget->ConvertToDeviceCoordinates(widgetOrigin.x, widgetOrigin.y);
   mouseLocRelativeToWidget.MoveBy(-widgetOrigin.x, -widgetOrigin.y);
 		
-  scrollEvent.eventStructType = NS_MOUSE_SCROLL_EVENT;
-  scrollEvent.isShift = PR_FALSE;
-  scrollEvent.isControl = PR_FALSE;
-  scrollEvent.isMeta = PR_FALSE;
-  scrollEvent.isAlt = PR_FALSE;
-	scrollEvent.message = NS_MOUSE_SCROLL;
   scrollEvent.delta = inDelta;
 	scrollEvent.point.x = mouseLocRelativeToWidget.x;
 	scrollEvent.point.y = mouseLocRelativeToWidget.y;
 	scrollEvent.time = PR_IntervalNow();
-	scrollEvent.widget = inWidget;
-	scrollEvent.nativeMsg = nsnull;
 
   // dispatch scroll event
   nsEventStatus rv;
@@ -231,14 +227,8 @@ void nsMacEventDispatchHandler::DispatchGuiEvent(nsWindow *aWidget, PRUint32 aEv
 	if (!aWidget)
 		return;
 
-	nsGUIEvent	guiEvent;
-	guiEvent.eventStructType = NS_GUI_EVENT;
-	guiEvent.point.x = 0;
-	guiEvent.point.y = 0;
+	nsGUIEvent	guiEvent(aEventType, aWidget);
 	guiEvent.time = PR_IntervalNow();
-	guiEvent.nativeMsg = nsnull;
-	guiEvent.message = aEventType;
-	guiEvent.widget = aWidget;
 	aWidget->DispatchWindowEvent(guiEvent);
 }
 
@@ -251,14 +241,8 @@ void nsMacEventDispatchHandler::DispatchSizeModeEvent(nsWindow *aWidget, nsSizeM
 	if (!aWidget)
 		return;
 
-	nsSizeModeEvent	event;
-	event.eventStructType = NS_SIZEMODE_EVENT;
-	event.point.x = 0;
-	event.point.y = 0;
+	nsSizeModeEvent	event(NS_SIZEMODE, aWidget);
 	event.time = PR_IntervalNow();
-	event.nativeMsg = nsnull;
-	event.message = NS_SIZEMODE;
-	event.widget = aWidget;
 	event.mSizeMode = aMode;
 	aWidget->DispatchWindowEvent(event);
 }
@@ -353,14 +337,26 @@ void nsMacEventDispatchHandler::SetDeactivated(nsWindow *aDeactivatedWidget)
       }
     }
       
-	// let the old one know it lost activation
-	if (mActiveWidget)
-	{	
-	    //printf("   nsMacEventDispatchHandler::SetDeactivated sends NS_DEACTIVATE\n");
-		mActiveWidget->RemoveDeleteObserver(this);
-		mActiveWidget = nsnull;
-	}
-	DispatchGuiEvent(aDeactivatedWidget, NS_DEACTIVATE);	
+    // If the deactivated toplevel window contains mActiveWidget, then
+    // clear out mActiveWidget.
+
+    if (mActiveWidget) {
+      nsCOMPtr<nsIWidget> curWin = do_QueryInterface(NS_STATIC_CAST(nsIWidget*, mActiveWidget));
+      for (;;) {
+        nsCOMPtr<nsIWidget> parent = dont_AddRef(curWin->GetParent());
+        if (!parent)
+          break;
+        curWin = parent;
+      }
+
+      if (NS_STATIC_CAST(nsWindow*, NS_STATIC_CAST(nsIWidget*, curWin)) == aDeactivatedWidget) {
+        //printf("   nsMacEventDispatchHandler::SetDeactivated sends NS_DEACTIVATE\n");
+        mActiveWidget->RemoveDeleteObserver(this);
+        mActiveWidget = nsnull;
+      }
+    }
+
+    DispatchGuiEvent(aDeactivatedWidget, NS_DEACTIVATE);	
 }
 
 //-------------------------------------------------------------------------
@@ -431,9 +427,6 @@ void nsMacEventDispatchHandler::SetGlobalPoint(Point inPoint)
 
 #pragma mark -
 
-static PRBool gUseUnicodeAPI = PR_FALSE;
-static PRBool gInitUseUnicodeAPI = PR_FALSE;
- 
 //-------------------------------------------------------------------------
 //
 // nsMacEventHandler constructor/destructor
@@ -460,12 +453,16 @@ nsMacEventHandler::nsMacEventHandler(nsMacWindow* aTopLevelWidget)
   }
   err = ::NewTSMDocument(1, supportedServices,&mTSMDocument, (long)this);
   NS_ASSERTION(err==noErr, "nsMacEventHandler::nsMacEventHandler: NewTSMDocument failed.");
-
 #ifdef DEBUG_TSM
   printf("nsMacEventHandler::nsMacEventHandler: created TSMDocument[%p]\n", mTSMDocument);
 #endif
 
-	mIMEIsComposing = PR_FALSE;
+  // make sure we do not use input widnow even some other code turn it for default by calling 
+  // ::UseInputWindow(nsnull, TRUE); 
+  if (mTSMDocument)
+    ::UseInputWindow(mTSMDocument, FALSE); 
+         
+  mIMEIsComposing = PR_FALSE;
   mIMECompositionStr = nsnull;
 
 #if !TARGET_CARBON
@@ -576,19 +573,16 @@ PRBool nsMacEventHandler::HandleMenuCommand(
 		focusedWidget = mTopLevelWidget;
 
 	// nsEvent
-	nsMenuEvent menuEvent;
-	menuEvent.eventStructType = NS_MENU_EVENT;
-	menuEvent.message		= NS_MENU_SELECTED;
+	nsMenuEvent menuEvent(NS_MENU_SELECTED, focusedWidget);
 	menuEvent.point.x		= aOSEvent.where.h;
 	menuEvent.point.y		= aOSEvent.where.v;
 	menuEvent.time			= PR_IntervalNow();
 
 	// nsGUIEvent
-	menuEvent.widget		= focusedWidget;
 	menuEvent.nativeMsg	= (void*)&aOSEvent;
 
 	// nsMenuEvent
-	menuEvent.mMenuItem	= nsnull;						//TODO: initialize mMenuItem
+  // TODO: initialize mMenuItem
 	menuEvent.mCommand	= aMenuResult;
 
 	// dispatch the menu event
@@ -642,8 +636,6 @@ PRBool nsMacEventHandler::HandleMenuCommand(
 //
 PRBool nsMacEventHandler::DragEvent ( unsigned int aMessage, Point aMouseGlobal, UInt16 aKeyModifiers )
 {
-	nsMouseEvent geckoEvent;
-
 	// convert the mouse to local coordinates. We have to do all the funny port origin
 	// stuff just in case it has been changed.
 	Point hitPointLocal = aMouseGlobal;
@@ -676,16 +668,12 @@ PRBool nsMacEventHandler::DragEvent ( unsigned int aMessage, Point aMouseGlobal,
 	// update the tracking of which widget the mouse is now over.
 	gEventDispatchHandler.SetWidgetPointed(widgetHit);
 	
+	nsMouseEvent geckoEvent(aMessage, widgetHit);
+
 	// nsEvent
-	geckoEvent.eventStructType = NS_DRAGDROP_EVENT;
-	geckoEvent.message = aMessage;
 	geckoEvent.point = widgetHitPoint;
 	geckoEvent.time	= PR_IntervalNow();
 
-	// nsGUIEvent
-	geckoEvent.widget = widgetHit;
-	geckoEvent.nativeMsg = nsnull;
-
 	// nsInputEvent
 	geckoEvent.isShift = ((aKeyModifiers & shiftKey) != 0);
 	geckoEvent.isControl = ((aKeyModifiers & controlKey) != 0);
@@ -916,10 +904,6 @@ void nsMacEventHandler::InitializeKeyEvent(nsKeyEvent& aKeyEvent,
 	//
 	// initalize the basic message parts
 	//
-	aKeyEvent.eventStructType = NS_KEY_EVENT;
-  aKeyEvent.message = aMessage;
-	aKeyEvent.point.x			= 0;
-	aKeyEvent.point.y			= 0;
 	aKeyEvent.time				= PR_IntervalNow();
 	
 	//
@@ -965,7 +949,7 @@ void nsMacEventHandler::InitializeKeyEvent(nsKeyEvent& aKeyEvent,
 		{
       if (!aKeyEvent.isMeta)
 			{
-				aKeyEvent.isShift = aKeyEvent.isControl = aKeyEvent.isAlt = aKeyEvent.isMeta = 0;
+				aKeyEvent.isControl = aKeyEvent.isAlt = aKeyEvent.isMeta = 0;
       } // if (!aKeyEvent.isMeta)
     
 			aKeyEvent.keyCode	= 0;
@@ -1157,47 +1141,55 @@ PRBool nsMacEventHandler::HandleKeyEvent(EventRecord& aOSEvent)
 		focusedWidget = mTopLevelWidget;
 	
 	// nsEvent
-	nsKeyEvent	keyEvent;
 	switch (aOSEvent.what)
 	{
 		case keyUp:
-			InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_UP);
-			result = focusedWidget->DispatchWindowEvent(keyEvent);
-			break;
-
-		case keyDown:	
-			InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_DOWN);
-			result = focusedWidget->DispatchWindowEvent(keyEvent);
-
-			// get the focused widget again in case something happened to it on the previous event
-			checkFocusedWidget = gEventDispatchHandler.GetActive();
-			if (!checkFocusedWidget)
-				checkFocusedWidget = mTopLevelWidget;
-
-			// if this isn't the same widget we had before, we should not send a keypress
-			if (checkFocusedWidget != focusedWidget)
-				return result;
-
-      InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_PRESS);
-
-      // before we dispatch this key, check if it's the contextmenu key.
-      // If so, send a context menu event instead.
-      if ( IsContextMenuKey(keyEvent) ) {
-        nsMouseEvent contextMenuEvent;
-        ConvertKeyEventToContextMenuEvent(&keyEvent, &contextMenuEvent);
-        result = focusedWidget->DispatchWindowEvent(contextMenuEvent);
-        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      {
+        nsKeyEvent keyUpEvent(NS_KEY_UP);
+        InitializeKeyEvent(keyUpEvent, aOSEvent, focusedWidget, NS_KEY_UP);
+        result = focusedWidget->DispatchWindowEvent(keyUpEvent);
+        break;
       }
-      else {
-        result = focusedWidget->DispatchWindowEvent(keyEvent);
-        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+
+    case keyDown:	
+      {
+        nsKeyEvent keyDownEvent(NS_KEY_DOWN), keyPressEvent(NS_KEY_PRESS);
+        InitializeKeyEvent(keyDownEvent, aOSEvent, focusedWidget, NS_KEY_DOWN);
+        result = focusedWidget->DispatchWindowEvent(keyDownEvent);
+
+        // get the focused widget again in case something happened to it on the previous event
+        checkFocusedWidget = gEventDispatchHandler.GetActive();
+        if (!checkFocusedWidget)
+          checkFocusedWidget = mTopLevelWidget;
+
+        // if this isn't the same widget we had before, we should not send a keypress
+        if (checkFocusedWidget != focusedWidget)
+          return result;
+
+        InitializeKeyEvent(keyPressEvent, aOSEvent, focusedWidget, NS_KEY_PRESS);
+
+        // before we dispatch this key, check if it's the contextmenu key.
+        // If so, send a context menu event instead.
+        if ( IsContextMenuKey(keyPressEvent) ) {
+          nsMouseEvent contextMenuEvent;
+          ConvertKeyEventToContextMenuEvent(&keyPressEvent, &contextMenuEvent);
+          result = focusedWidget->DispatchWindowEvent(contextMenuEvent);
+          NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+        }
+        else {
+          result = focusedWidget->DispatchWindowEvent(keyPressEvent);
+          NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+        }
+        break;
+      }
+
+    case autoKey:
+      {
+        nsKeyEvent keyPressEvent(NS_KEY_PRESS);
+        InitializeKeyEvent(keyPressEvent, aOSEvent, focusedWidget, NS_KEY_PRESS);
+        result = focusedWidget->DispatchWindowEvent(keyPressEvent);
+        break;
       }
-			break;
-		
-		case autoKey:
-			InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_PRESS);
-			result = focusedWidget->DispatchWindowEvent(keyEvent);
-			break;
 	}
 
 	return result;
@@ -1217,7 +1209,6 @@ ConvertKeyEventToContextMenuEvent(const nsKeyEvent* inKeyEvent, nsMouseEvent* ou
 {
   *(nsInputEvent*)outCMEvent = *(nsInputEvent*)inKeyEvent;
   
-  outCMEvent->eventStructType = NS_MOUSE_EVENT;
   outCMEvent->message = NS_CONTEXTMENU_KEY;
   outCMEvent->isShift = outCMEvent->isControl = outCMEvent->isAlt = outCMEvent->isMeta = PR_FALSE;
   
@@ -1247,7 +1238,7 @@ IsContextMenuKey(const nsKeyEvent& inKeyEvent)
 // HandleUKeyEvent
 //
 //-------------------------------------------------------------------------
-PRBool nsMacEventHandler::HandleUKeyEvent(PRUnichar* text, long charCount, EventRecord& aOSEvent)
+PRBool nsMacEventHandler::HandleUKeyEvent(const PRUnichar* text, long charCount, EventRecord& aOSEvent)
 {
   nsresult result;
   // get the focused widget
@@ -1255,38 +1246,74 @@ PRBool nsMacEventHandler::HandleUKeyEvent(PRUnichar* text, long charCount, Event
   if (!focusedWidget)
     focusedWidget = mTopLevelWidget;
   
-  // nsEvent
-  nsKeyEvent keyEvent;
   PRBool isCharacter = PR_FALSE;
-  InitializeKeyEvent(keyEvent, aOSEvent, focusedWidget, NS_KEY_PRESS, &isCharacter, PR_FALSE);
+
+  // simulate key down event if this isn't an autoKey event
+  if (aOSEvent.what == keyDown)
+  {
+    nsKeyEvent keyDownEvent(NS_KEY_DOWN);
+    InitializeKeyEvent(keyDownEvent, aOSEvent, focusedWidget, NS_KEY_DOWN, &isCharacter, PR_FALSE);
+    result = focusedWidget->DispatchWindowEvent(keyDownEvent);
+    NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent keydown");
+
+    // check if focus changed; see also HandleKeyEvent above
+    nsWindow *checkFocusedWidget = gEventDispatchHandler.GetActive();
+    if (!checkFocusedWidget)
+      checkFocusedWidget = mTopLevelWidget;
+    if (checkFocusedWidget != focusedWidget)
+      return result;
+  }
+
+  // simulate key press events
+  nsKeyEvent keyPressEvent(NS_KEY_PRESS);
+  InitializeKeyEvent(keyPressEvent, aOSEvent, focusedWidget, NS_KEY_PRESS, &isCharacter, PR_FALSE);
+
   if (isCharacter) 
   {
     // it is a message with text, send all the unicode characters
-    PRUint32 i;
+    PRInt32 i;
     for (i = 0; i < charCount; i++)
     {
-      keyEvent.charCode = text[i];
-      if (keyEvent.isShift && keyEvent.charCode <= 'z' && keyEvent.charCode >= 'a') 
-        keyEvent.charCode -= 32;
+      keyPressEvent.charCode = text[i];
+
+      // control key is special in that it doesn't give us letters
+      // it generates a charcode of 0x01 for control-a
+      // so we offset to do the right thing for gecko (as in HandleKeyEvent)
+      // this doesn't happen for us in InitializeKeyEvent because we pass
+      // PR_FALSE so no character translation occurs.
+      // I'm guessing we don't want to do the translation there because
+      // translation already occurred for the string passed to this method.
+      if (keyPressEvent.isControl && keyPressEvent.charCode <= 26)       
+      {
+        if (keyPressEvent.isShift)
+          keyPressEvent.charCode += 'A' - 1;
+        else
+          keyPressEvent.charCode += 'a' - 1;
+      }
+
+      // this block of code is triggered when user presses
+      // a combination such as command-shift-M
+      if (keyPressEvent.isShift && keyPressEvent.charCode <= 'z' && keyPressEvent.charCode >= 'a') 
+        keyPressEvent.charCode -= 32;
 
       // before we dispatch a key, check if it's the context menu key.
       // If so, send a context menu event instead.
-      if ( IsContextMenuKey(keyEvent) ) {
+      if ( IsContextMenuKey(keyPressEvent) ) {
         nsMouseEvent contextMenuEvent;
-        ConvertKeyEventToContextMenuEvent(&keyEvent, &contextMenuEvent);
+        ConvertKeyEventToContextMenuEvent(&keyPressEvent, &contextMenuEvent);
         result = focusedWidget->DispatchWindowEvent(contextMenuEvent);
         NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
       }
       else {
         // command / shift keys, etc. only send once
-        result = focusedWidget->DispatchWindowEvent(keyEvent);
+        result = focusedWidget->DispatchWindowEvent(keyPressEvent);
         NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
       }
     }
   }
   else {
     // command / shift keys, etc. only send once
-    result = focusedWidget->DispatchWindowEvent(keyEvent);
+    result = focusedWidget->DispatchWindowEvent(keyPressEvent);
     NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
   }
   return result;
@@ -1324,10 +1351,13 @@ if (KeyDown(0x39))	// press [caps lock] to start the profile
 		//
 		// Activate The TSMDocument associated with this handler
 		//
-		if (mTSMDocument)
-			err = ::ActivateTSMDocument(mTSMDocument);
-		else
-			err = ::UseInputWindow(NULL, true); // get this line from mozilla-classic - mozilla/cmd/macfe/central/TSMProxy.cp
+		if (mTSMDocument) {
+		  // make sure we do not use input widnow even some other code turn it for default by calling 
+		  // ::UseInputWindow(nsnull, TRUE); 
+		  ::UseInputWindow(mTSMDocument, FALSE); 
+		  err = ::ActivateTSMDocument(mTSMDocument);
+		}
+
 #ifdef DEBUG_TSM
 #if 0
 		NS_ASSERTION(err==noErr,"nsMacEventHandler::HandleActivateEvent: ActivateTSMDocument failed");
@@ -1340,10 +1370,14 @@ if (KeyDown(0x39))	// press [caps lock] to start the profile
 		PRBool active;
 		mTopLevelWidget->IsActive(&active);
 		nsWindow*	focusedWidget = mTopLevelWidget;
-	  if(!active) {
+		if (!active) {
 		  gEventDispatchHandler.SetActivated(focusedWidget);
 		  mTopLevelWidget->SetIsActive(PR_TRUE);
 		}
+		else if (!gEventDispatchHandler.GetActive()) {
+		  NS_ASSERTION(0, "We think we're active, but there is no active widget!");
+		  gEventDispatchHandler.SetActivated(focusedWidget);
+		}
 		
 		// Twiddle menu bars
 		nsIMenuBar* menuBar = focusedWidget->GetMenuBar();
@@ -1368,8 +1402,13 @@ if (KeyDown(0x39))	// press [caps lock] to start the profile
 		//
 		// Deactivate the TSMDocument assoicated with this EventHandler
 		//
-		if (mTSMDocument)
-			err = ::DeactivateTSMDocument(mTSMDocument);
+		if (mTSMDocument) {
+		  // make sure we do not use input widnow even some other code turn it for default by calling 
+		  // ::UseInputWindow(nsnull, TRUE); 
+		  ::UseInputWindow(mTSMDocument, FALSE); 
+		  err = ::DeactivateTSMDocument(mTSMDocument);
+		}
+
 #ifdef DEBUG_TSM
 		NS_ASSERTION((noErr==err)||(tsmDocNotActiveErr==err),"nsMacEventHandler::HandleActivateEvent: DeactivateTSMDocument failed");
 		printf("nsEventHandler::HandleActivateEvent: DeactivateTSMDocument[%p] %s return %d\n",mTSMDocument,
@@ -1412,7 +1451,7 @@ PRBool nsMacEventHandler::ResizeEvent ( WindowRef inWindow )
 	::GetWindowPortBounds ( inWindow, &macRect );
 	::LocalToGlobal(&topLeft(macRect));
 	::LocalToGlobal(&botRight(macRect));
-	mTopLevelWidget->Resize(macRect.right - macRect.left + 1, macRect.bottom - macRect.top + 1, PR_FALSE);
+	mTopLevelWidget->Resize(macRect.right - macRect.left, macRect.bottom - macRect.top, PR_FALSE);
 	if (nsnull != gRollupListener && (nsnull != gRollupWidget) )
 		gRollupListener->Rollup();
 	mTopLevelWidget->UserStateForResize(); // size a zoomed window and it's no longer zoomed
@@ -1668,7 +1707,12 @@ PRBool nsMacEventHandler::HandleMouseMoveEvent( EventRecord& aOSEvent )
 	if (lastWidgetHit)
 	{
 		Point macPoint = aOSEvent.where;
-		::GlobalToLocal(&macPoint);
+		WindowRef wind = reinterpret_cast<WindowRef>(mTopLevelWidget->GetNativeData(NS_NATIVE_DISPLAY));
+		nsGraphicsUtils::SafeSetPortWindowPort(wind);
+		{
+			StOriginSetter  originSetter(wind);
+			::GlobalToLocal(&macPoint);
+		}
 		PRBool inWidgetHit = lastWidgetHit->PointInWidget(macPoint);
 		if (sMouseInWidgetHit != inWidgetHit)
 		{
@@ -1823,7 +1867,6 @@ void nsMacEventHandler::ConvertOSEventToMouseEvent(
         widgetHit = mTopLevelWidget;
 		
     // nsEvent
-    aMouseEvent.eventStructType = NS_MOUSE_EVENT;
     aMouseEvent.message     = aMessage;
     aMouseEvent.point       = widgetHitPoint;
     aMouseEvent.time        = PR_IntervalNow();
@@ -1881,7 +1924,7 @@ nsresult nsMacEventHandler::HandleOffsetToPosition(long offset,Point* thePoint)
 //-------------------------------------------------------------------------
 // See ftp://ftp.unicode.org/Public/MAPPINGS/VENDORS/APPLE/CORPCHAR.TXT for detail of IS_APPLE_HINT_IN_PRIVATE_ZONE
 #define IS_APPLE_HINT_IN_PRIVATE_ZONE(u) ((0xF850 <= (u)) && ((u)<=0xF883))
-nsresult nsMacEventHandler::HandleUpdateInputArea(char* text,Size text_size, ScriptCode textScript,long fixedLength,TextRangeArray* textRangeList)
+nsresult nsMacEventHandler::HandleUpdateInputArea(const char* text,Size text_size, ScriptCode textScript,long fixedLength,TextRangeArray* textRangeList)
 {
 #ifdef DEBUG_TSM
 	printf("********************************************************************************\n");
@@ -1935,7 +1978,7 @@ nsresult nsMacEventHandler::HandleUpdateInputArea(char* text,Size text_size, Scr
 	}
 	// Prepare buffer....
 	mIMECompositionStr->SetCapacity(text_size+1);
-	ubuf = (PRUnichar*)mIMECompositionStr->get();
+	ubuf = mIMECompositionStr->BeginWriting();
 	size_t len;
 
 	//====================================================================================================
@@ -2236,7 +2279,7 @@ error:
 }
 
 
-nsresult nsMacEventHandler::UnicodeHandleUpdateInputArea(PRUnichar* text, long charCount,
+nsresult nsMacEventHandler::UnicodeHandleUpdateInputArea(const PRUnichar* text, long charCount,
                                                          long fixedLength, TextRangeArray* textRangeList)
 {
 #ifdef DEBUG_TSM
@@ -2406,7 +2449,7 @@ nsresult nsMacEventHandler::UnicodeHandleUpdateInputArea(PRUnichar* text, long c
     // This is needed when we input some uncommitted text, and then delete all of them
     // When the last delete come, we will got a text_size = 0 and fixedLength = 0
     // In that case, we need to send a text event to clean up the input hole....
-    mIMECompositionStr->Assign(NS_LITERAL_STRING(""));      
+    mIMECompositionStr->Truncate();      
 #ifdef DEBUG_TSM
       printf("3.====================================\n");
 #endif
@@ -2425,6 +2468,26 @@ nsresult nsMacEventHandler::UnicodeHandleUpdateInputArea(PRUnichar* text, long c
 error:
   return res;
 }
+
+nsresult nsMacEventHandler::HandleUnicodeGetSelectedText(nsAString& outString)
+{
+  outString.Truncate(0);
+  nsWindow* focusedWidget = gEventDispatchHandler.GetActive();
+  if (!focusedWidget)
+    focusedWidget = mTopLevelWidget;
+
+  nsReconversionEvent reconversionEvent(NS_RECONVERSION_QUERY, focusedWidget);
+  reconversionEvent.time = PR_IntervalNow();
+
+  nsresult res = focusedWidget->DispatchWindowEvent(reconversionEvent);
+
+  if (NS_SUCCEEDED(res)) {
+     outString.Assign(reconversionEvent.theReply.mReconversionString);
+     nsMemory::Free(reconversionEvent.theReply.mReconversionString);
+  } 
+  return res; 
+  
+}
 //-------------------------------------------------------------------------
 //
 // HandleStartComposition
@@ -2442,7 +2505,7 @@ nsresult nsMacEventHandler::HandleStartComposition(void)
 	if(nsnull == mIMECompositionStr)
 	{
 		return NS_ERROR_OUT_OF_MEMORY;
-	}
+	}	
 	// 
 	// get the focused widget [tague: may need to rethink this later]
 	//
@@ -2453,20 +2516,9 @@ nsresult nsMacEventHandler::HandleStartComposition(void)
 	//
 	// create the nsCompositionEvent
 	//
-	nsCompositionEvent		compositionEvent;
-
-	compositionEvent.eventStructType = NS_COMPOSITION_START;
-	compositionEvent.message = NS_COMPOSITION_START;
-	compositionEvent.point.x = 0;
-	compositionEvent.point.y = 0;
+	nsCompositionEvent		compositionEvent(NS_COMPOSITION_START, focusedWidget);
 	compositionEvent.time = PR_IntervalNow();
 
-	//
-	// nsGUIEvent parts
-	//
-	compositionEvent.widget	= focusedWidget;
-	compositionEvent.nativeMsg	= (void*)nsnull;		// no native message for this
-
 	nsresult res = focusedWidget->DispatchWindowEvent(compositionEvent);
 	if(NS_SUCCEEDED(res)) {
 		mIMEPos.x = compositionEvent.theReply.mCursorPosition.x;
@@ -2501,19 +2553,9 @@ nsresult nsMacEventHandler::HandleEndComposition(void)
 	//
 	// create the nsCompositionEvent
 	//
-	nsCompositionEvent		compositionEvent;
-
-	compositionEvent.eventStructType = NS_COMPOSITION_END;
-	compositionEvent.message = NS_COMPOSITION_END;
-	compositionEvent.point.x = 0;
-	compositionEvent.point.y = 0;
+	nsCompositionEvent		compositionEvent(NS_COMPOSITION_END, focusedWidget);
 	compositionEvent.time = PR_IntervalNow();
 
-	//
-	// nsGUIEvent parts
-	//
-	compositionEvent.widget	= focusedWidget;
-	compositionEvent.nativeMsg	= (void*)nsnull;		// no native message for this
 	return(focusedWidget->DispatchWindowEvent(compositionEvent));
 }
 
@@ -2572,25 +2614,14 @@ nsresult nsMacEventHandler::HandleTextEvent(PRUint32 textRangeCount, nsTextRange
 		focusedWidget = mTopLevelWidget;
 	
 	//
-	// create the nsCompositionEvent
+	// create the nsTextEvent
 	//
-	nsTextEvent		textEvent;
-
-	textEvent.eventStructType = NS_TEXT_EVENT;
-	textEvent.message = NS_TEXT_EVENT;
-	textEvent.point.x = 0;
-	textEvent.point.y = 0;
+	nsTextEvent		textEvent(NS_TEXT_TEXT, focusedWidget);
 	textEvent.time = PR_IntervalNow();
-	textEvent.theText = (PRUnichar*)mIMECompositionStr->get();
+	textEvent.theText = mIMECompositionStr->get();
 	textEvent.rangeCount = textRangeCount;
 	textEvent.rangeArray = textRangeArray;
 
-	//
-	// nsGUIEvent parts
-	//
-	textEvent.widget	= focusedWidget;
-	textEvent.nativeMsg	= (void*)nsnull;		// no native message for this
-
 	nsresult res = NS_OK;
 	if (NS_SUCCEEDED(res = focusedWidget->DispatchWindowEvent(textEvent))) {
 		mIMEPos.x = textEvent.theReply.mCursorPosition.x;
@@ -2607,6 +2638,9 @@ nsresult nsMacEventHandler::ResetInputState()
 {
 	OSErr err = noErr;
 	if (mTSMDocument) {
+		// make sure we do not use input widnow even some other code turn it for default by calling 
+		// ::UseInputWindow(nsnull, TRUE); 
+		::UseInputWindow(mTSMDocument, FALSE); 
 		err = ::FixTSMDocument(mTSMDocument);
 		NS_ASSERTION( (noErr==err)||(tsmDocNotActiveErr==err)||(tsmTSNotOpenErr), "Cannot FixTSMDocument");
 	}
