@@ -67,7 +67,6 @@
 #include "prmem.h"
 #include "prprf.h"
 #include "rdf.h"
-#include "xp_core.h"
 
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetAlias.h"
@@ -177,9 +176,8 @@ NS_NewRelatedLinksStreamListener(nsIRDFDataSource* aDataSource,
 
 
 RelatedLinksStreamListener::RelatedLinksStreamListener(nsIRDFDataSource *aDataSource)
-	 : mDataSource(dont_QueryInterface(aDataSource))
+	 : mDataSource(aDataSource)
 {
-	 NS_INIT_REFCNT();
 }
 
 
@@ -224,20 +222,27 @@ RelatedLinksStreamListener::Init()
 				(nsISupports**)&charsetConv);
 		if (NS_SUCCEEDED(rv) && (charsetConv))
 		{
-			nsString	utf8(NS_LITERAL_STRING("UTF-8"));
-			rv = charsetConv->GetUnicodeDecoder(&utf8,
-				getter_AddRefs(mUnicodeDecoder));
+			rv = charsetConv->GetUnicodeDecoderRaw("UTF-8",
+												getter_AddRefs(mUnicodeDecoder));
 			NS_RELEASE(charsetConv);
 		}
 
-		gRDFService->GetResource(NC_NAMESPACE_URI "child", &kNC_Child);
-		gRDFService->GetResource(NC_NAMESPACE_URI "Name",  &kNC_Name);
-		gRDFService->GetResource(NC_NAMESPACE_URI "URL",   &kNC_URL);
-		gRDFService->GetResource(NC_NAMESPACE_URI "loading", &kNC_loading);
-		gRDFService->GetResource(NC_NAMESPACE_URI "BookmarkSeparator", &kNC_BookmarkSeparator);
-		gRDFService->GetResource(NC_NAMESPACE_URI "RelatedLinksTopic", &kNC_RelatedLinksTopic);
-		gRDFService->GetResource(RDF_NAMESPACE_URI "type", &kRDF_type);
-		gRDFService->GetResource(kURINC_RelatedLinksRoot, &kNC_RelatedLinksRoot);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "child"),
+                             &kNC_Child);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "Name"),
+                             &kNC_Name);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "URL"),
+                             &kNC_URL);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "loading"),
+                             &kNC_loading);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "BookmarkSeparator"),
+                             &kNC_BookmarkSeparator);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "RelatedLinksTopic"),
+                             &kNC_RelatedLinksTopic);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(RDF_NAMESPACE_URI "type"),
+                             &kRDF_type);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_RelatedLinksRoot),
+                             &kNC_RelatedLinksRoot);
 	 }
 
 	 mParentArray.AppendElement(kNC_RelatedLinksRoot);
@@ -379,7 +384,7 @@ RelatedLinksStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ct
 			mBuffer.Left(oneLiner, eol);
 			mBuffer.Cut(0, eol+1);
 		}
-		if (oneLiner.Length() < 1)	break;
+		if (oneLiner.IsEmpty())	break;
 
 #if 0
 		printf("RL: '%s'\n", NS_LossyConvertUCS2toASCII(oneLiner).get());
@@ -452,7 +457,7 @@ RelatedLinksStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ct
 				if (NS_SUCCEEDED(rv = gRDFService->GetAnonymousResource(getter_AddRefs(newTopic))))
 				{
 					mDataSource->Assert(newTopic, kRDF_type, kNC_RelatedLinksTopic, PR_TRUE);
-					if (title.Length() > 0)
+					if (!title.IsEmpty())
 					{
                         Unescape(title);
 
@@ -491,7 +496,7 @@ RelatedLinksStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ct
 			}
 		}
 
-		if (child.Length() > 0)
+		if (!child.IsEmpty())
 		{
 
 #if 0
@@ -505,7 +510,7 @@ RelatedLinksStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ct
 				if (NS_SUCCEEDED(rv))
 				{
 					title.Trim(" ");
-					if (title.Length() > 0)
+					if (!title.IsEmpty())
 					{
                         Unescape(title);
 
@@ -585,7 +590,6 @@ RelatedLinksStreamListener::Unescape(nsString &text)
 RelatedLinksHandlerImpl::RelatedLinksHandlerImpl()
 	: mRelatedLinksURL(nsnull)
 {
-	NS_INIT_REFCNT();
 }
 
 
@@ -627,10 +631,14 @@ RelatedLinksHandlerImpl::Init()
                                                   (nsISupports**) &gRDFService);
 		if (NS_FAILED(rv)) return rv;
 
-		gRDFService->GetResource(kURINC_RelatedLinksRoot, &kNC_RelatedLinksRoot);
-		gRDFService->GetResource(RDF_NAMESPACE_URI "type", &kRDF_type);
-		gRDFService->GetResource(NC_NAMESPACE_URI "RelatedLinksTopic", &kNC_RelatedLinksTopic);
-		gRDFService->GetResource(NC_NAMESPACE_URI "child", &kNC_Child);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(kURINC_RelatedLinksRoot),
+                             &kNC_RelatedLinksRoot);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(RDF_NAMESPACE_URI "type"),
+                             &kRDF_type);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "RelatedLinksTopic"),
+                             &kNC_RelatedLinksTopic);
+		gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "child"),
+                             &kNC_Child);
 
 		nsCOMPtr<nsIPref> prefServ(do_GetService(kPrefCID, &rv));
 		mRLServerURL = new nsString();
@@ -938,15 +946,6 @@ RelatedLinksHandlerImpl::RemoveObserver(nsIRDFObserver *aObserver)
 
 
 
-NS_IMETHODIMP
-RelatedLinksHandlerImpl::GetAllCommands(nsIRDFResource* aSource,
-					nsIEnumerator/*<nsIRDFResource>*/** aCommands)
-{
-	return mInner->GetAllCommands(aSource, aCommands);
-}
-
-
-
 NS_IMETHODIMP
 RelatedLinksHandlerImpl::GetAllCmds(nsIRDFResource* aSource,
 					nsISimpleEnumerator/*<nsIRDFResource>*/** aCommands)
@@ -974,3 +973,19 @@ RelatedLinksHandlerImpl::DoCommand(nsISupportsArray/*<nsIRDFResource>*/* aSource
 {
 	return mInner->DoCommand(aSources, aCommand, aArguments);
 }
+
+
+
+NS_IMETHODIMP
+RelatedLinksHandlerImpl::BeginUpdateBatch()
+{
+        return mInner->BeginUpdateBatch();
+}
+
+
+
+NS_IMETHODIMP
+RelatedLinksHandlerImpl::EndUpdateBatch()
+{
+        return mInner->EndUpdateBatch();
+}
